/*{ cm3_basic.h ***********************************************
 *
 * TheToolChain.com
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Basic datatypes, enums and functions
 *
}*/

#ifndef CM3_BASIC_H
#define CM3_BASIC_H

typedef unsigned char        u8_t;
typedef unsigned short int  u16_t;
typedef unsigned long       u32_t;
typedef signed char          s8_t;
typedef signed short int    s16_t;
typedef signed int          s32_t;
typedef enum { FALSE=0, TRUE } BOOL;
typedef BOOL bool;

#define  Base_t u32_t         // basic (native) datatype of STM32 architecture
#define  Base_signed_t s32_t  // signed basic (native) datatype

#ifdef EXTENSION_200_cpu_stm32f10x
  #include "stm32f10x.h"
#endif
#ifdef EXTENSION_200_cpu_stm32f2xx
  #include "stm32f2xx.h"
#endif
#ifdef EXTENSION_200_cpu_stm32l1xx
  #include "stm32l1xx.h"
#endif
#ifdef EXTENSION_200_cpu_stm32w1xx
  #include "stm32w108xx.h"
#endif

#endif

#define DRIVER_ttc_atomic_read_write_32(Address, NewValue) cm3_atomic_read_write_32(Address, NewValue)
#define DRIVER_ttc_atomic_read_write_16(Address, NewValue) cm3_atomic_read_write_16(Address, NewValue)
#define DRIVER_ttc_atomic_read_write_8(Address, NewValue)  cm3_atomic_read_write_8(Address, NewValue)
#define DRIVER_ttc_atomic_add_32(Address, NewValue)        cm3_atomic_add_32(Address, NewValue)
#define DRIVER_ttc_atomic_add_16(Address, NewValue)        cm3_atomic_add_16(Address, NewValue)
#define DRIVER_ttc_atomic_add_8(Address, NewValue)         cm3_atomic_add_8(Address, NewValue)

/** Atomic read+write operation required for thread safe memory operations (E.g. for mutexes, ...)
  *
  * @param Address  memory location to read and write
  * @param NewValue *Address = NewValue
  * @return         previous value of *Address
  */
u32_t cm3_atomic_read_write_32(volatile u32_t* Address, u32_t NewValue);
u16_t cm3_atomic_read_write_16(volatile u16_t* Address, u16_t NewValue);
u8_t cm3_atomic_read_write_8(volatile u8_t* Address, u8_t NewValue);

/** Atomic add/subtract operation required for thread safe memory operations (E.g. for semaphores, ...)
  *
  * @param Address  memory location to read and write
  * @param Value    *Address += Value
  * @return         current value of *Address
  */
u32_t cm3_atomic_add_32(volatile u32_t* Address, s32_t Amount);
u16_t cm3_atomic_add_16(volatile u16_t* Address, s16_t Amount);
 u8_t cm3_atomic_add_8(volatile u8_t* Address, s8_t Amount);
