#ifndef cm3_semaphore_H
#define cm3_semaphore_H

/*{ cm3_semaphore.h ***********************************************
 * 
 * Written by Gregor Rebel 2010-2012
 *
 * Low-level driver for Mutual Exclusive Amounts (Semaphores) on Cortex M3 architecture.
 * 
}*/

//{ includes

#include "ttc_basic.h"
#include "cm3_basic.h"
#include "core_cm3.h"

//}includes
//{ Default settings (change via makefile) ******************************


//}Default settings
//{ Types & Structures ***************************************************


//}
//{ Low-Level driver macros for ttc_semaphore.h ******************************

#define DRIVER_ttc_semaphore_create()                             cm3_semaphore_create()
#define DRIVER_ttc_semaphore_init(Semaphore)                      cm3_semaphore_init(Semaphore)
#define DRIVER_ttc_semaphore_take(Semaphore, Amount, TimeOut)     cm3_semaphore_take(Semaphore, Amount, TimeOut)
#define DRIVER_ttc_semaphore_try_take(Semaphore, Amount)          cm3_semaphore_try_take(Semaphore, Amount)
#define DRIVER_ttc_semaphore_take_isr(Semaphore, Amount)          cm3_semaphore_try_take_isr(Semaphore, Amount)
#define DRIVER_ttc_semaphore_can_provide(Semaphore, Amount)       cm3_semaphore_can_provide(Semaphore, Amount)
#define DRIVER_ttc_semaphore_give(Semaphore, Amount)              cm3_semaphore_give(Semaphore, Amount)
#define DRIVER_ttc_semaphore_give_isr(Semaphore, Amount)          cm3_semaphore_give(Semaphore, Amount)
#define DRIVER_ttc_semaphore_available(Semaphore)                 (*(Semaphore))


//}
//{ includes 2 ***********************************************************

#include "cm3_semaphore_types.h"

// these includes require macros and types above
#include "ttc_semaphore_types.h"
#include "ttc_memory.h"

//}includes 2
//{ Function prototypes **************************************************


/** allocates + initializes a new non-recursive semaphore Amount
 * @return                 =! NULL: semaphore has been created successfully (use handle to operate on semaphore)
 */
cm3_Semaphore_t* cm3_semaphore_create();

/** initializes an existing non-recursive semaphore to zero
 * @Amount                 storage to be initialized as semaphore
 */
void cm3_semaphore_init(cm3_Semaphore_t* Semaphore);

/** Tries to take given amount from semaphore within timeout
 * @param Semaphore  handle of already created semaphore
 * @param TimeOut  !=-1: amount of system ticks to wait for Amount
 *                 ==-1: wait forever
 * @return         == tsme_OK: Amount has been successfully obtained within TimeOut
 */
ttc_semaphore_error_e cm3_semaphore_take(cm3_Semaphore_t* Semaphore, Base_t Amount, Base_t TimeOut);

/** Tries to take given amount from semaphore; this function will never block and must only be called with disabled interrupts!
 * @param Semaphore                handle of already created semaphore
 * @return         == tsme_OK: Amount has been successfully obtained within TimeOut
 *                 >= tsme_Error: Error condition
 */
ttc_semaphore_error_e cm3_semaphore_try_take(cm3_Semaphore_t* Semaphore, Base_t Amount);

/** Tries to take given amount from semaphore; this function will never block and must only be called with disabled interrupts! (only to be called by interrupt service routines!)
 * @param Semaphore    handle of already created semaphore
 * @return         == tsme_OK: Amount has been successfully obtained within TimeOut
 *                 >= tsme_Error: Error condition
 */
ttc_semaphore_error_e cm3_semaphore_try_take_isr(cm3_Semaphore_t* Semaphore, Base_t Amount);
#define cm3_semaphore_try_take_isr(Semaphore, Amount) cm3_semaphore_try_take(Semaphore, Amount)

/** Checks if value of given Semaphore >= Amount
 * @param Semaphore    handle of already created semaphore
 * @return         == TRUE: given Semaphore can provide given amount; FALSE otherwise
 */
// BOOL cm3_semaphore_can_provide(cm3_Semaphore* Semaphore, Base_t Amount);
#define cm3_semaphore_can_provide(Semaphore, Amount) ( ( *(Semaphore) >= Amount ) ? 1 : 0 )

/** Gives amount of tokens back to semaphore
 * @param Semaphore  handle of already created smart semaphore
 * @return         == tsme_OK: Amount has been given back successfully
 */
ttc_semaphore_error_e cm3_semaphore_give(cm3_Semaphore_t* Semaphore, Base_t Amount);

//}Function prototypes

#endif
