/*{ freertos_semaphore.c ***********************************************
 *
 * Written by Gregor Rebel 2010-2013
 *
 * Architecture independent support for inter task communication & synchronization.
 * 
 * Currently supported schedulers: FreeRTOS
}*/
#include "freertos_semaphore.h"

//{ functions *************************************************

     DRIVER_ttc_semaphore_t* freertos_semaphore_create() {

    DRIVER_ttc_semaphore_t* NewSemaphore = ttc_memory_alloc_zeroed( sizeof(DRIVER_ttc_semaphore_t) );
    Assert_SEMAPHORES(NewSemaphore, ec_Malloc);
    freertos_semaphore_init(NewSemaphore);

    return NewSemaphore;
}
                 void freertos_semaphore_init(DRIVER_ttc_semaphore_t* Semaphore) {


#define fsi_uxItemSize 0
#define fsi_uxQueueLength -1
#define fsi_ucQueueType queueQUEUE_TYPE_COUNTING_SEMAPHORE
#define fsi_uxInitialCount 0


    // copied from queue.c/xQueueGenericCreate()
    // (FreeRTOS does not provide an _init() by itself

    /* Create the list of pointers to queue items.  The queue is one byte
       longer than asked for to make wrap checking easier/faster. */
    Base_t xQueueSizeInBytes = ( size_t ) ( fsi_uxQueueLength * fsi_uxItemSize ) + ( size_t ) 1;

    Semaphore->pcHead = ( signed char * ) pvPortMalloc( xQueueSizeInBytes );
    if( Semaphore->pcHead != NULL )
    {
        /* Initialise the queue members as described above where the queue type is defined. */
        Semaphore->uxLength   = fsi_uxQueueLength;
        Semaphore->uxItemSize = fsi_uxItemSize;
        xQueueGenericReset( Semaphore, pdTRUE );
#if ( configUSE_TRACE_FACILITY == 1 )
        {
            Semaphore->ucQueueType = fsi_ucQueueType;
        }
#endif /* configUSE_TRACE_FACILITY */

        traceQUEUE_CREATE( Semaphore );
    }
    else
    {
        traceQUEUE_CREATE_FAILED( fsi_ucQueueType );
        vPortFree( Semaphore );
    }

    // copied from queue.c/xQueueCreateCountingSemaphore()
    // (FreeRTOS does not provide an _init() by itself
    if( Semaphore != NULL )
    {
        Semaphore->uxMessagesWaiting = fsi_uxInitialCount;

        traceCREATE_COUNTING_SEMAPHORE();
    }
    else
    {
        traceCREATE_COUNTING_SEMAPHORE_FAILED();
    }
}
ttc_semaphore_error_e freertos_semaphore_give(DRIVER_ttc_semaphore_t* Semaphore, Base_t Amount) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

    signed portBASE_TYPE Result;
    for (; Amount > 0; Amount--) {
        Result = xSemaphoreGive(Semaphore);
        Assert_SEMAPHORES(Result == pdTRUE, ec_UNKNOWN);
    }
    return tsme_OK;
}
ttc_semaphore_error_e freertos_semaphore_take(DRIVER_ttc_semaphore_t* Semaphore, Base_t Amount, Base_t TimeOut) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

    // calculate ticks from usecs
    TimeOut = freertos_calculate_ticks(TimeOut);

    if (Amount > 1) { // FreeRTOS does not provide taking >1 from a semaphore at once
        for (Base_t I = Amount; I > 0; I--) { // take one from semaphore several times within TimeOut
            Base_t StartTime = freertos_get_tick_count();
            if (xSemaphoreTake(Semaphore, TimeOut) != pdTRUE) { // timeout occured: abort complete operation
                for (; I < Amount; I++) { // roll back operation
                    xSemaphoreGive(Semaphore);
                }
                return tsme_TimeOut;
            }
            if (Amount > 1) {
                Base_t Duration = freertos_get_tick_count() - StartTime;
                if (TimeOut < Duration)
                    TimeOut -= Duration;
                else
                    TimeOut = 0;
            }
        }
    }
    else {
        if (xSemaphoreTake(Semaphore, TimeOut) != pdTRUE)
            return tsme_TimeOut;
    }
    return tsme_OK;
}
ttc_semaphore_error_e freertos_semaphore_give_isr(DRIVER_ttc_semaphore_t* Semaphore, Base_t Amount) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

    signed portBASE_TYPE Result;
    for (; Amount > 0; Amount--) {
        Result = xSemaphoreGiveFromISR(Semaphore, NULL); // second argument may be NULL (FreeRTOS >= 7.3.0)
        if (Result == errQUEUE_FULL)
            return tsme_SemaphoreFull;
        Assert_SEMAPHORES(Result == pdTRUE, ec_UNKNOWN);
    }
    return tsme_OK;
}
ttc_semaphore_error_e freertos_semaphore_take_isr(DRIVER_ttc_semaphore_t* Semaphore, Base_t Amount) {

    signed portBASE_TYPE Result;
    for (Base_t I = Amount; I > 0; I--) { // FreeRTOS does not provide taking >1 from semaphore at once
        Result = xSemaphoreTakeFromISR(Semaphore, NULL); // second argument may be NULL (FreeRTOS >= 7.3.0)
        if (Result != pdTRUE) {
            for (; I < Amount; I++) { // roll back operation
                xSemaphoreGiveFromISR(Semaphore, NULL); // second argument may be NULL (FreeRTOS >= 7.3.0)
            }
            return tsme_SemaphoreExpired;
        }
    }
    return tsme_OK;
}

// private functions ********************************************

//}functions
