/** { stm32f1_timer.c ************************************************
 
                           The ToolChain
                      
   High-Level interface for TIMER device.
     
   Note: See ttc_timer.h for description of stm32f1 independent TIMER implementation.
  
   Authors: 
}*/

#include "stm32f1_timer.h"

//{ Function definditions ***************************************************

                       bool stm32f1_timer_check_initialized(ttc_timer_config_t* Config) {

    if (Config->Architecture)
        return TRUE;

    return FALSE;
}
     ttc_timer_errorcode_e stm32f1_timer_load_defaults(ttc_timer_config_t* Config) {
    Assert_TIMER(Config != NULL, ec_NULL);
    Assert_TIMER(Config->LogicalIndex > 0, ec_InvalidArgument);

    Config->Flags.All                        = 0;
    // TIMER_Cfg->Flags.Bits.Master                = 1;

    return ec_timer_OK;
}
stm32f1_timer_config_t* stm32f1_timer_get_configuration(ttc_timer_config_t* Config) {
    Assert_TIMER(Config != NULL, ec_NULL);
    Assert_TIMER(Config->LogicalIndex > 0, ec_InvalidArgument);

    stm32f1_timer_config_t* Config_stm32f1 = Config->Architecture;
    if (Config_stm32f1 == NULL) {  // first call: allocate + init configuration
      Config_stm32f1 = ttc_memory_alloc_zeroed( sizeof(ttc_timer_arch_t) );
      
      Config_stm32f1->PhysicalIndex = stm32f1_timer_logical_2_physical_index(Config->LogicalIndex);
      
      switch (Config_stm32f1->PhysicalIndex) {           // determine base register and other low-level configuration data
         case 0: { // load low-level configuration of first timer device 
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;
         }
         case 1: { // load low-level configuration of second timer device
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;
         } 
         case 2: { // load low-level configuration of third timer device
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;             
         }
         case 3: { // load low-level configuration of fourth timer device
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;
         }
         case 4: { // load low-level configuration of fifth timer device
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;
         }
         case 5: { // load low-level configuration of sixth timer device
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;
         }
         case 6: { // load low-level configuration of seveth timer device
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;
         }
         case 7: { // load low-level configuration of eighth timer device
           Config_stm32f1->BaseRegister = NULL; // load adress of register base of TIMER device
           break;
         }
      default: Assert_TIMER(0, ec_UNKNOWN); break; // No TTC_TIMERn defined! Check your makefile.100_board_* file!
      }
    }
    
    Assert_TIMER(Config_stm32f1, ec_InvalidConfiguration);
    return Config_stm32f1;
}
     ttc_timer_errorcode_e stm32f1_timer_get_features(ttc_timer_config_t* Config) {
    Assert_TIMER(Config,    ec_InvalidArgument);
    Assert_TIMER(Config->LogicalIndex > 0, ec_InvalidArgument);
    if (Config->LogicalIndex > TTC_TIMER_AMOUNT) return ec_timer_DeviceNotFound;

    Config->Flags.All                        = 0;
    //Config->Flags.Bits.Master                = 1;

    switch (Config->LogicalIndex) {           // determine features of indexed TIMER device
#ifdef TTC_TIMER1
      case 1: break;
#endif
      default: Assert_TIMER(0, ec_UNKNOWN); break; // No TTC_TIMERn defined! Check your makefile.100_board_* file!
    }
    
    return ec_timer_OK;
}
     ttc_timer_errorcode_e stm32f1_timer_init(ttc_timer_config_t* Config) {
    Assert_TIMER(Config, ec_NULL);
    Assert_TIMER(Config->LogicalIndex > 0, ec_InvalidArgument);
    stm32f1_timer_config_t* Config_stm32f1 = stm32f1_timer_get_configuration(Config);
    (void) Config_stm32f1; // avoid warning: "unused variable"

    if (1) { // validate TIMER_Features
        ttc_timer_config_t TIMER_Features;
        TIMER_Features.LogicalIndex = Config->LogicalIndex;
        ttc_timer_errorcode_e Error = stm32f1_timer_get_features(&TIMER_Features);
        if (Error) return Error;
        Config->Flags.All &= TIMER_Features.Flags.All; // mask out unavailable flags
    }
    switch (Config->LogicalIndex) {                // find TIMER corresponding to TIMER_index as defined by makefile.100_board_*
#ifdef TTC_TIMER1
      case 1: Config_stm32f1->Base = (TIMER_t*) TTC_TIMER1; break;
#endif
#ifdef TTC_TIMER2
      case 2: Config_stm32f1->Base = (TIMER_t*) TTC_TIMER2; break;
#endif
#ifdef TTC_TIMER3
      case 3: Config_stm32f1->Base = (TIMER_t*) TTC_TIMER3; break;
#endif
      default: Assert_TIMER(0, ec_UNKNOWN); break; // No TTC_TIMERn defined! Check your makefile.100_board_* file!
    }
    
    return ec_timer_OK;
}
     ttc_timer_errorcode_e stm32f1_timer_deinit(ttc_timer_config_t* Config) {
    Assert_TIMER(Config, ec_NULL);
    Assert_TIMER(Config->LogicalIndex > 0, ec_InvalidArgument);
    stm32f1_timer_config_t* Config_stm32f1 = stm32f1_timer_get_configuration(Config);
    (void) Config_stm32f1; // avoid warning: "unused variable"

    // ToDo: Deinitialize hardware device
    
    return ec_timer_OK;
}
                       u8_t stm32f1_timer_logical_2_physical_index(u8_t LogicalIndex) {
  switch (LogicalIndex) {           // determine base register and other low-level configuration data
#ifdef TTC_TIMER1
           case 1: return TTC_TIMER1;
#endif
#ifdef TTC_TIMER2
           case 2: return TTC_TIMER2;
#endif
#ifdef TTC_TIMER3
           case 3: return TTC_TIMER3;
#endif
#ifdef TTC_TIMER4
           case 4: return TTC_TIMER4;
#endif
#ifdef TTC_TIMER5
           case 5: return TTC_TIMER5;
#endif
#ifdef TTC_TIMER6
           case 6: return TTC_TIMER6;
#endif
#ifdef TTC_TIMER7
           case 7: return TTC_TIMER7;
#endif
#ifdef TTC_TIMER8
           case 8: return TTC_TIMER8;
#endif
#ifdef TTC_TIMER9
           case 9: return TTC_TIMER9;
#endif
      default: Assert_TIMER(0, ec_UNKNOWN); break; // No TTC_TIMERn defined! Check your makefile.100_board_* file!
    }
    
    return -1;
}

//} Function definitions
//{ private functions (ideally) ********************************************

//} private functions
