#!/usr/bin/perl

my $DriverName       = shift(@ARGV);
my $ArchitectureName = shift(@ARGV);
my $Option           = shift(@ARGV);

my $DRIVERNAME       = uc($DriverName);
my $ARCHITECTURENAME = uc($ArchitectureName);

my $SkipExistingFiles = 1;
if ($Option eq 'OVERWRITE') { $SkipExistingFiles = 0; }

unless ($ArchitectureName) { # help + exit
  print <<"END_OF_TEXT";
$0 DRIVER ARCHITECTURE [OVERWRITE]

Will create a set of high-level +low-level driver code.

DRIVER        name of driver set to create
ARCHITECTURE  name of architecture for low-level driver part
OVERWRITE      if given, output files will be overwritten silently

Files being created:
../ttc_DRIVER.c        - high level driver implementation
../ttc_DRIVER.h        - high level driver prototypes
../ttc_DRIVER_types.c  - high level driver datatypes (includes ../ARCHITECTURE/ARCHITECTURE_DRIVER_types.h)
../ARCHITECTURE/ARCHITECTURE_DRIVER.c       - low level driver implementation
../ARCHITECTURE/ARCHITECTURE_DRIVER.h       - low level driver prototypes + macros
../ARCHITECTURE/ARCHITECTURE_DRIVER_types.h - low level driver datatypes

Note: Existing files will not be changed.

END_OF_TEXT
  exit 5;
}

foreach my $File ('architecture_driver.c', 
                  'architecture_driver.h', 
                  'architecture_driver_types.h', 
                  'create_DriverSet.pl', 
                  'ttc_driver.c', 
                  'ttc_driver.h', 
                  'ttc_driver_types.h',
                  'install_NN_TTC_DRIVER.sh'
                ) {
  my $TargetFile;
  my $IsLowLevelDriver  = 0;
  my $IsHighLevelDriver = 0;
  my $IsInstallScript   = 0;
  
  if (substr($File, 0, 4) eq 'ttc_') {           # low-level driver
    $TargetFile = "../$File";
    $IsLowLevelDriver = 1;
  }
  if (substr($File, 0, 13) eq 'architecture_') { # high-level driver
    if (! -d "../$ArchitectureName") { mkdir("../$ArchitectureName"); }
    $TargetFile = "../$ArchitectureName/$File";
    $IsHighLevelDriver = 1;
  }
  if (substr($File, 0, 8) eq 'install_') {       # install script
    $TargetFile = "../../ToolChain//$File";
    
    # find rank of last install_NN_TTC_file
    my $ExisitingTTC_Files = `ls ../../ToolChain/install_*_TTC_* | sort -n`;
    my @ExisitingTTC_Files = split("\n", $ExisitingTTC_Files);
    my $LastTTC_File = $ExisitingTTC_Files[-1];
    # E.g.: "../../ToolChain/install_24_TTC_USART.sh"
    my $StartPos = index($LastTTC_File, "_") + 1;
    my $EndPos   = index($LastTTC_File, "_", $StartPos);
    my $HighestRank = substr($LastTTC_File, $StartPos, $EndPos - $StartPos) + 0;
    $TargetFile =~ s/NN/$HighestRank/g;
    
    $IsInstallScript = 1;
  }
  if ($TargetFile) { # create target file from source file by replacing strings: driver DRIVER architecture Architecture
    $TargetFile =~ s/DRIVER/$DRIVERNAME/g;
    $TargetFile =~ s/driver/$DriverName/g;
    $TargetFile =~ s/ARCHITECTURE/$ARCHITECTURENAME/g;
    $TargetFile =~ s/architecture/$ArchitectureName/g;
    
    unless ( $SkipExistingFiles && (-e $TargetFile) ) {
      print "creating '$TargetFile' ...\n";
      my $Content = readFile($File);
      
      $Content =~ s/driver/$DriverName/g;
      $Content =~ s/DRIVER/$DRIVERNAME/g;
      $Content =~ s/architecture/$ArchitectureName/g;
      $Content =~ s/ARCHITECTURE/$ARCHITECTURENAME/g;
      
      writeFile($TargetFile, $Content);
      if ($IsInstallScript) {
        system("chmod +x '$TargetFile'");
      }
    }
    else {
      print "skipping existing file '$TargetFile'\n";
    }
  }
}

sub readFile {          # read in content of given file
  my $FilePath = shift;
  
  my $Content;
  open(IN, "<$FilePath") or die ("$0 - ERROR: Cannot read from file '$FilePath' ($!)");
  while (<IN>) { $Content .= $_; }
  close(IN);
  
  return $Content;
}
sub writeFile {         # write content to given file
  my $FilePath = shift;
  my $Content  = shift;
  
  open(OUT, ">$FilePath") or die ("$0 - ERROR: Cannot write to file '$FilePath' ($!)");
  print OUT $Content;
  close(OUT);
  
  return $Content;
}
