/** { architecture_driver.c ************************************************
 
                           The ToolChain
                      
   High-Level interface for DRIVER device.
     
   Note: See ttc_driver.h for description of architecture independent DRIVER implementation.
  
   Authors: 
}*/

#include "architecture_driver.h"

//{ Function definditions ***************************************************

                       bool architecture_driver_check_initialized(ttc_driver_config_t* Config) {

    if (Config->Architecture)
        return TRUE;

    return FALSE;
}
     ttc_driver_errorcode_e architecture_driver_load_defaults(ttc_driver_config_t* Config) {
    Assert_DRIVER(Config != NULL, ec_NULL);
    Assert_DRIVER(Config->LogicalIndex > 0, ec_InvalidArgument);

    Config->Flags.All                        = 0;
    // DRIVER_Cfg->Flags.Bits.Master                = 1;

    return ec_driver_OK;
}
architecture_driver_config_t* architecture_driver_get_configuration(ttc_driver_config_t* Config) {
    Assert_DRIVER(Config != NULL, ec_NULL);
    Assert_DRIVER(Config->LogicalIndex > 0, ec_InvalidArgument);

    architecture_driver_config_t* Config_architecture = Config->Architecture;
    if (Config_architecture == NULL) {  // first call: allocate + init configuration
      Config_architecture = ttc_memory_alloc_zeroed( sizeof(ttc_driver_arch_t) );
      
      Config_architecture->PhysicalIndex = architecture_driver_logical_2_physical_index(Config->LogicalIndex);
      
      switch (Config_architecture->PhysicalIndex) {           // determine base register and other low-level configuration data
         case 0: { // load low-level configuration of first driver device 
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;
         }
         case 1: { // load low-level configuration of second driver device
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;
         } 
         case 2: { // load low-level configuration of third driver device
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;             
         }
         case 3: { // load low-level configuration of fourth driver device
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;
         }
         case 4: { // load low-level configuration of fifth driver device
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;
         }
         case 5: { // load low-level configuration of sixth driver device
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;
         }
         case 6: { // load low-level configuration of seveth driver device
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;
         }
         case 7: { // load low-level configuration of eighth driver device
           Config_architecture->BaseRegister = NULL; // load adress of register base of DRIVER device
           break;
         }
      default: Assert_DRIVER(0, ec_UNKNOWN); break; // No TTC_DRIVERn defined! Check your makefile.100_board_* file!
      }
    }
    
    Assert_DRIVER(Config_architecture, ec_InvalidConfiguration);
    return Config_architecture;
}
     ttc_driver_errorcode_e architecture_driver_get_features(ttc_driver_config_t* Config) {
    Assert_DRIVER(Config,    ec_InvalidArgument);
    Assert_DRIVER(Config->LogicalIndex > 0, ec_InvalidArgument);
    if (Config->LogicalIndex > TTC_DRIVER_AMOUNT) return ec_driver_DeviceNotFound;

    Config->Flags.All                        = 0;
    //Config->Flags.Bits.Master                = 1;

    switch (Config->LogicalIndex) {           // determine features of indexed DRIVER device
#ifdef TTC_DRIVER1
      case 1: break;
#endif
      default: Assert_DRIVER(0, ec_UNKNOWN); break; // No TTC_DRIVERn defined! Check your makefile.100_board_* file!
    }
    
    return ec_driver_OK;
}
     ttc_driver_errorcode_e architecture_driver_init(ttc_driver_config_t* Config) {
    Assert_DRIVER(Config, ec_NULL);
    Assert_DRIVER(Config->LogicalIndex > 0, ec_InvalidArgument);
    architecture_driver_config_t* Config_architecture = architecture_driver_get_configuration(Config);
    (void) Config_architecture; // avoid warning: "unused variable"

    if (1) { // validate DRIVER_Features
        ttc_driver_config_t DRIVER_Features;
        DRIVER_Features.LogicalIndex = Config->LogicalIndex;
        ttc_driver_errorcode_e Error = architecture_driver_get_features(&DRIVER_Features);
        if (Error) return Error;
        Config->Flags.All &= DRIVER_Features.Flags.All; // mask out unavailable flags
    }
    switch (Config->LogicalIndex) {                // find DRIVER corresponding to DRIVER_index as defined by makefile.100_board_*
#ifdef TTC_DRIVER1
      case 1: Config_architecture->Base = (DRIVER_t*) TTC_DRIVER1; break;
#endif
#ifdef TTC_DRIVER2
      case 2: Config_architecture->Base = (DRIVER_t*) TTC_DRIVER2; break;
#endif
#ifdef TTC_DRIVER3
      case 3: Config_architecture->Base = (DRIVER_t*) TTC_DRIVER3; break;
#endif
      default: Assert_DRIVER(0, ec_UNKNOWN); break; // No TTC_DRIVERn defined! Check your makefile.100_board_* file!
    }
    
    return ec_driver_OK;
}
     ttc_driver_errorcode_e architecture_driver_deinit(ttc_driver_config_t* Config) {
    Assert_DRIVER(Config, ec_NULL);
    Assert_DRIVER(Config->LogicalIndex > 0, ec_InvalidArgument);
    architecture_driver_config_t* Config_architecture = architecture_driver_get_configuration(Config);
    (void) Config_architecture; // avoid warning: "unused variable"

    // ToDo: Deinitialize hardware device
    
    return ec_driver_OK;
}
                       u8_t architecture_driver_logical_2_physical_index(u8_t LogicalIndex) {
  switch (LogicalIndex) {           // determine base register and other low-level configuration data
#ifdef TTC_DRIVER1
           case 1: return TTC_DRIVER1;
#endif
#ifdef TTC_DRIVER2
           case 2: return TTC_DRIVER2;
#endif
#ifdef TTC_DRIVER3
           case 3: return TTC_DRIVER3;
#endif
#ifdef TTC_DRIVER4
           case 4: return TTC_DRIVER4;
#endif
#ifdef TTC_DRIVER5
           case 5: return TTC_DRIVER5;
#endif
#ifdef TTC_DRIVER6
           case 6: return TTC_DRIVER6;
#endif
#ifdef TTC_DRIVER7
           case 7: return TTC_DRIVER7;
#endif
#ifdef TTC_DRIVER8
           case 8: return TTC_DRIVER8;
#endif
#ifdef TTC_DRIVER9
           case 9: return TTC_DRIVER9;
#endif
      default: Assert_DRIVER(0, ec_UNKNOWN); break; // No TTC_DRIVERn defined! Check your makefile.100_board_* file!
    }
    
    return -1;
}

//} Function definitions
//{ private functions (ideally) ********************************************

//} private functions
