#ifndef ARCHITECTURE_DRIVER_H
#define ARCHITECTURE_DRIVER_H

/** { architecture_driver.h **********************************************
 
                           The ToolChain
                      
   High-Level interface for DRIVER device.

   Structures, Enums and Defines being required by high-level driver and application.

   Note: See ttc_driver.h for description of architecture independent DRIVER implementation.
  
   Authors: 

 
}*/
//{ Defines/ TypeDefs ****************************************************


//} Defines
//{ Includes *************************************************************

#include "architecture_driver_types.h"
#include "../ttc_driver_types.h"
#include "../ttc_task.h"
#include "../ttc_memory.h"

//} Includes
//{ Macro definitions ****************************************************

#define Driver_driver_load_defaults(Config) architecture_driver_load_defaults(Config)
#define Driver_driver_deinit(Config)        architecture_driver_deinit(Config)
#define Driver_driver_init(Config)          architecture_driver_init(Config)


//} Includes
//{ Function prototypes **************************************************

/** checks if indexed DRIVER bus already has been initialized
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return              == TRUE: indexed DRIVER unit is already initialized
 */
bool architecture_driver_check_initialized(ttc_driver_config_t* Config);

/** returns reference to low-level configuration struct of indexed DRIVER device
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return              pointer to struct ttc_driver_config_t (will assert if no configuration available)
 */
architecture_driver_config_t* architecture_driver_get_configuration(ttc_driver_config_t* Config);

/** loads configuration of indexed DRIVER interface with default values
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return  == 0:       configuration was loaded successfully
 */
ttc_driver_errorcode_e architecture_driver_load_defaults(ttc_driver_config_t* Config);

/** fills out given Config with maximum valid values for indexed DRIVER
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return  == 0:       *Config has been initialized successfully; != 0: error-code
 */
ttc_driver_errorcode_e architecture_driver_get_features(ttc_driver_config_t* Config);

/** reset configuration of indexed DRIVER device into default state
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return              == 0: DRIVER has been initialized successfully; != 0: error-code
 */
ttc_driver_errorcode_e architecture_driver_reset(ttc_driver_config_t* Config);

/** initializes single DRIVER
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return              == 0: DRIVER has been initialized successfully; != 0: error-code
 */
ttc_driver_errorcode_e architecture_driver_init(ttc_driver_config_t* Config);

/** shutdown single DRIVER device
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return              == 0: DRIVER has been shutdown successfully; != 0: error-code
 */
ttc_driver_errorcode_e architecture_driver_deinit(ttc_driver_config_t* Config);

/** maps from logical to physical device index
 *
 * High-level drivers (ttc_driver_*() functions) use logical device indices.
 * Logical index n is mapped via "COMPILE_OPTS += -DTTC_DRIVERn=p" lines in the makefile to physical index p.
 *
 * Boards usually connect only some of the available devices to connectors. Therefore, some devices may not be 
 * usable. The mapping from logical to physical indices allows to create a consecutive range of indices in any 
 * order.
 *
 * @param LogicalIndex  logical index of driver device (1..ttc_driver_get_max_index() )
 * @return              physical index of driver device (0 = first physical driver device, ...)
 */
u8_t architecture_driver_logical_2_physical_index(u8_t LogicalIndex);

//} Function prototypes

#endif //ARCHITECTURE_DRIVER_H
