/** { ttc_driver.c *******************************************************

                           The ToolChain
                           
   High-Level interface for DRIVER device.

   Implementation of high-level interface.

   Authors:
   
}*/
//{ Defines/ TypDefs *****************************************************
//} Defines

#include "ttc_driver.h"

//{ Global Variables *****************************************************

#if TTC_DRIVER_AMOUNT == 0
  #warning No devices defined, check your makefile!
#endif

// for each initialized device, a pointer to its generic and architecture definitions is stored
ttc_array_define(ttc_driver_config_t*, ttc_driver_configs, TTC_DRIVER_AMOUNT);

// Set to TRUE on first call of ttc_driver_init()
BOOL Driver_Initialized = 0;

//} Global Variables
//{ Function definitions *************************************************

void ttc_driver_prepare() {
  // add your startup code here (Singletasking!)
}
u8_t ttc_driver_get_max_index() {
    return TTC_DRIVER_AMOUNT;
}
ttc_driver_config_t* ttc_driver_get_configuration(u8_t LogicalIndex) {
    Assert_DRIVER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_driver_config_t* Config = A(ttc_driver_configs, LogicalIndex-1); // will assert if outside array bounds

    if (!Config) {
        Config = A(ttc_driver_configs, LogicalIndex-1) = ttc_memory_alloc_zeroed(sizeof(ttc_driver_config_t));
        ttc_driver_reset(LogicalIndex);
    }

    return Config;
}
void ttc_driver_init(u8_t LogicalIndex) {
    Assert_DRIVER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_driver_config_t* Config = ttc_driver_get_configuration(LogicalIndex);

    ttc_driver_errorcode_e Result = Driver_driver_init(Config);
    if (Result == ec_driver_OK)
      Config->Flags.Bits.Initialized = 1;
}
void ttc_driver_deinit(u8_t LogicalIndex) {
    Assert_DRIVER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_driver_config_t* Config = ttc_driver_get_configuration(LogicalIndex);
  
    ttc_driver_errorcode_e Result = Driver_driver_deinit(Config);
    if (Result == ec_driver_OK)
      Config->Flags.Bits.Initialized = 0;
}
void ttc_driver_reset(u8_t LogicalIndex) {
    Assert_DRIVER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_driver_config_t* Config = ttc_driver_get_configuration(LogicalIndex);

    ttc_memory_set( Config, 0, sizeof(ttc_driver_config_t) );

    // load generic default values
    Config->LogicalIndex = LogicalIndex;

    Driver_driver_load_defaults(Config);    
}

// ToDo: additional functions go here...

//} Function definitions
