/** { ttc_driver.h *******************************************************

                           The ToolChain
                           
   High-Level interface for DRIVER device.

   Structures, Enums and Defines being required by high-level driver and application.

   Authors:
   
}*/

#ifndef TTC_DRIVER_H
#define TTC_DRIVER_H

//{ Includes *************************************************************

#include "ttc_basic.h"
#include "ttc_memory.h"

#ifdef EXTENSION_450_driver_architecture
#  include "architecture/architecture_driver.h"
#endif

#include "ttc_driver_types.h"

//} Includes
//{ Check for missing low-level implementations **************************

// Note: functions declared below must be redefined to implementations provided by low-level Driver!

/** Prepares driver Driver for basic operation.
 *
 * Note: This function is called automatically at system start before the scheduler has been started.
 */
void ttc_driver_prepare();

/** loads configuration of indexed DRIVER interface with default values
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return  == 0:       configuration was loaded successfully
 */
ttc_driver_errorcode_e Driver_driver_load_defaults(ttc_driver_config_t* Config);
#ifndef Driver_driver_load_defaults
#  error: missing low-level implementation!
#endif

/** shutdown single DRIVER device
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return              == 0: DRIVER has been shutdown successfully; != 0: error-code
 */
ttc_driver_errorcode_e Driver_driver_deinit(ttc_driver_config_t* Config);
#ifndef Driver_driver_deinit
#  error: missing low-level implementation!
#endif

/** initializes single DRIVER
 * @param Config        pointer to struct ttc_driver_config_t (must have valid value for LogicalIndex)
 * @return              == 0: DRIVER has been initialized successfully; != 0: error-code
 */
ttc_driver_errorcode_e Driver_driver_init(ttc_driver_config_t* Config);
#ifndef Driver_driver_init
#  error: missing low-level implementation!
#endif

//}
//{ Defines/ TypeDefs ****************************************************

//} Defines
//{ Structures/ Enums ****************************************************
// Structures and Enums required by High-Level driver only go here...

//} Structures/ Enums
//{ Function prototypes **************************************************

/** returns amount of devices available in current configuration
 *
 * @return amount of available devices
 */
u8_t ttc_driver_get_max_index();

/** returns configuration of indexed device (asserts if no valid configuration was found)
 *
 * @param LogicalIndex    index of device to init (1..ttc_DRIVER_get_max_LogicalIndex())
 * @return                configuration of indexed device
 */
ttc_driver_config_t* ttc_driver_get_configuration(u8_t LogicalIndex);

/** initialize indexed device for use
 *
 * @param LogicalIndex    device index of device to init (1..ttc_DRIVER_get_max_LogicalIndex())
 */
void ttc_driver_init(u8_t LogicalIndex);

/** shutdown indexed device
 *
 * @param LogicalIndex    device index of device to shutdown (1..ttc_DRIVER_get_max_LogicalIndex())
 */
void ttc_driver_deinit(u8_t LogicalIndex);

/** reset configuration of indexed device back to default
 *
 * @param LogicalIndex    device index of device to init (1..ttc_DRIVER_get_max_LogicalIndex())
 */
void ttc_driver_reset(u8_t LogicalIndex);

// additional prototypes go here...

//} Function prototypes
//{ private Function prototypes ******************************************

/** 
 *
 * Note: This function is marked private and should be called from outside!
 *
 * @param 
 * @return
 */
void _ttc_driver_();


//}private functions
//{ Macros ***************************************************************

//}Macros

#endif //TTC_DRIVER_H
