/** { ttc_driver_types.h ************************************************

                           The ToolChain

   High-Level interface for DRIVER device.
   
   Structures, Enums and Defines being required by both, high- and low-level driver.
   

   Authors:
   
}*/

#ifndef TTC_DRIVER_TYPES_H
#define TTC_DRIVER_TYPES_H

#include "ttc_basic.h"
#include "architecture/architecture_driver_types.h"

//{ Defines ***************************************************


//} Defines
//{ Static Configuration **************************************

// TTC_DRIVERn has to be defined as constant by makefile.100_board_*
#ifdef TTC_DRIVER5
  #ifndef TTC_DRIVER4
    #error TTC_DRIVER5 is defined, but not TTC_DRIVER4 - all lower TTC_DRIVERn must be defined!
  #endif
  #ifndef TTC_DRIVER3
    #error TTC_DRIVER5 is defined, but not TTC_DRIVER3 - all lower TTC_DRIVERn must be defined!
  #endif
  #ifndef TTC_DRIVER2
    #error TTC_DRIVER5 is defined, but not TTC_DRIVER2 - all lower TTC_DRIVERn must be defined!
  #endif
  #ifndef TTC_DRIVER1
    #error TTC_DRIVER5 is defined, but not TTC_DRIVER1 - all lower TTC_DRIVERn must be defined!
  #endif

  #define TTC_DRIVER_AMOUNT 5
#else
  #ifdef TTC_DRIVER4
    #define TTC_DRIVER_AMOUNT 4

    #ifndef TTC_DRIVER3
      #error TTC_DRIVER5 is defined, but not TTC_DRIVER3 - all lower TTC_DRIVERn must be defined!
    #endif
    #ifndef TTC_DRIVER2
      #error TTC_DRIVER5 is defined, but not TTC_DRIVER2 - all lower TTC_DRIVERn must be defined!
    #endif
    #ifndef TTC_DRIVER1
      #error TTC_DRIVER5 is defined, but not TTC_DRIVER1 - all lower TTC_DRIVERn must be defined!
    #endif
  #else
    #ifdef TTC_DRIVER3

      #ifndef TTC_DRIVER2
        #error TTC_DRIVER5 is defined, but not TTC_DRIVER2 - all lower TTC_DRIVERn must be defined!
      #endif
      #ifndef TTC_DRIVER1
        #error TTC_DRIVER5 is defined, but not TTC_DRIVER1 - all lower TTC_DRIVERn must be defined!
      #endif

      #define TTC_DRIVER_AMOUNT 3
    #else
      #ifdef TTC_DRIVER2

        #ifndef TTC_DRIVER1
          #error TTC_DRIVER5 is defined, but not TTC_DRIVER1 - all lower TTC_DRIVERn must be defined!
        #endif

        #define TTC_DRIVER_AMOUNT 2
      #else
        #ifdef TTC_DRIVER1
          #define TTC_DRIVER_AMOUNT 1
        #else
          #define TTC_DRIVER_AMOUNT 0
        #endif
      #endif
    #endif
  #endif
#endif

#ifndef TTC_ASSERT_DRIVER    // any previous definition set (Makefile)?
#define TTC_ASSERT_DRIVER 1  // string asserts are enabled by default
#endif
#if (TTC_ASSERT_DRIVER == 1)  // use Assert()s in DRIVER code (somewhat slower but alot easier to debug)
  #define Assert_DRIVER(Condition, ErrorCode) Assert(Condition, ErrorCode)
#else  // us no Assert()s in DRIVER code (somewhat smaller + faster, but crashes are hard to debug)
  #define Assert_DRIVER(Condition, ErrorCode)
#endif

//}Static Configuration
//{ Enums/ Structures *****************************************

typedef enum {    // return codes of DRIVER devices
  ec_driver_OK = 0,
  
  // other warnings go here..

  ec_driver_ERROR,           // general failure
  ec_driver_DeviceNotFound,
  ec_driver_InvalidImplementation
  
  // other failures go here..
} ttc_driver_errorcode_e;
typedef struct ttc_driver_config_s { // architecture independent configuration data

    // Note: Write-access to this structure is only allowed before first ttc_driver_init() call!
    //       After initialization, only read accesses are allowed for high-level code.

                 u8_t  LogicalIndex; // automatically set: logical index of device to use (1 = TTC_DRIVER1, ...)
    ttc_driver_arch_t* Architecture; // low-level configuration
    
    union  { // generic configuration bits common for all low-level Drivers
        u16_t All;
        struct {
          unsigned Initialized        : 1;  // ==1: device has been initialized successfully
          // ToDo: additional high-level flags go here..
            
            unsigned Reserved1        : 16; // pad to 16 bits
        } Bits;
    } Flags;

    // Additional high-level attributes go here..
    
} __attribute__((__packed__)) ttc_driver_config_t;

//}Structures

#endif // TTC_DRIVER_TYPES_H
