/*{ ttc_basic.c ***********************************************
 *
 * TheToolChain.com
 *
 * Written by Gregor Rebel 2010-2011
 *
 * Basic set of helper functions.
 * 
}*/
#include "ttc_basic.h"

extern void ttc_task_begin_criticalsection_owner(void (*Owner)());
extern void ttc_task_end_criticalsection_owner();
extern void _ttc_task_yield();

void ttc_hardware_init() {

#ifdef EXTENSION_250_stm_std_peripherals
    stm32_hardware_init();
#endif
#ifdef TARGET_ARCHITECTURE_EM357
    em357_hardware_init();
#endif
}
bool Assert_On_False(BOOL Condition) {
    if (!Condition)
        Assert_Halt_EC(ec_UNKNOWN);

    return Condition;
}

void Assert_Halt_EC(volatile ErrorCode_e ErrorCode) {  // block endless in case Condition is false
    volatile bool HoldOnAssert=TRUE;

    while (HoldOnAssert)
        _ttc_task_yield();
        ErrorCode = ErrorCode; // <-- breakpoint here!

    // Only returns if HoldOnAssert==FALSE
}
void Assert_Halt_NULL() {
    Assert_Halt_EC(ec_NULL);
}
void ttc_testpoint() {
    u8_t Foo = 1;
    (void) Foo; // break here
}
u32_t _ttc_atomic_read_write_32(u32_t* Address, u32_t NewValue) {
    Assert(Address, ec_InvalidArgument);

    ttc_task_begin_criticalsection_owner( (void (*)()) _ttc_atomic_read_write_32);
    u32_t OldValue = *Address;
    *Address = NewValue;
    ttc_task_end_criticalsection_owner();

    return OldValue;
}
u16_t _ttc_atomic_read_write_16(u16_t* Address, u16_t NewValue) {
    Assert(Address, ec_InvalidArgument);

    ttc_task_begin_criticalsection_owner( (void (*)()) _ttc_atomic_read_write_16);
    u16_t OldValue = *Address;
    *Address = NewValue;
    ttc_task_end_criticalsection_owner();

    return OldValue;
}
u8_t  _ttc_atomic_read_write_8(u8_t* Address, u8_t NewValue) {
    Assert(Address, ec_InvalidArgument);

    ttc_task_begin_criticalsection_owner( (void (*)()) _ttc_atomic_read_write_8);
    u8_t OldValue = *Address;
    *Address = NewValue;
    ttc_task_end_criticalsection_owner();

    return OldValue;
}
u32_t _ttc_atomic_add_32(u32_t* Address, s32_t Amount) {
    Assert(Address, ec_InvalidArgument);

    ttc_task_begin_criticalsection_owner( (void (*)()) _ttc_atomic_add_32);
    u32_t Value = *Address;
    if (Amount > 0)
        Value += (u32_t) Amount;
    else
        Value -= (u32_t) -Amount;
    *Address = Value;
    ttc_task_end_criticalsection_owner();

    return Value;
}
u16_t _ttc_atomic_add_16(u16_t* Address, s16_t Amount) {
    Assert(Address, ec_InvalidArgument);

    ttc_task_begin_criticalsection_owner( (void (*)()) _ttc_atomic_add_16);
    u16_t Value = *Address;
    if (Amount > 0)
        Value += (u16_t) Amount;
    else
        Value -= (u16_t) -Amount;
    *Address = Value;
    ttc_task_end_criticalsection_owner();

    return Value;
}
u8_t  _ttc_atomic_add_8(u8_t* Address, s8_t Amount) {
    Assert(Address, ec_InvalidArgument);

    ttc_task_begin_criticalsection_owner( (void (*)()) _ttc_atomic_add_8);
    u8_t Value = *Address;
    if (Amount > 0)
        Value += (u8_t) Amount;
    else
        Value -= (u8_t) -Amount;
    *Address = Value;
    ttc_task_end_criticalsection_owner();

    return Value;
}
