/** { ttc_can.c *******************************************************

                           The ToolChain
                           
   High-Level interface for CAN device.

   Implementation of high-level interface.

   Authors:
   
}*/
//{ Defines/ TypDefs *****************************************************
//} Defines

#include "ttc_can.h"

//{ Global Variables *****************************************************

#if TTC_CAN_AMOUNT == 0
  #warning No devices defined, check your makefile!
#endif

// for each initialized device, a pointer to its generic and stm32f1 definitions is stored
ttc_array_define(ttc_can_config_t*, ttc_can_configs, TTC_CAN_AMOUNT);

// Set to TRUE on first call of ttc_can_init()
BOOL Driver_Initialized = 0;

//} Global Variables
//{ Function definitions *************************************************

void ttc_can_prepare() {
  // add your startup code here (Singletasking!)
}
u8_t ttc_can_get_max_index() {
    return TTC_CAN_AMOUNT;
}

ttc_can_config_t* ttc_can_get_configuration(u8_t LogicalIndex) {
    Assert_CAN(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_can_config_t* Config = A(ttc_can_configs, LogicalIndex-1); // will assert if outside array bounds

    if (!Config) {
        Config = A(ttc_can_configs, LogicalIndex-1) = ttc_memory_alloc_zeroed(sizeof(ttc_can_config_t));
        ttc_can_reset(LogicalIndex);
    }

    return Config;
}
void ttc_can_init(u8_t LogicalIndex) {
    Assert_CAN(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_can_config_t* Config = ttc_can_get_configuration(LogicalIndex);

    ttc_can_errorcode_e Result = Driver_can_init(Config);
    if (Result == ec_can_OK)
      Config->Flags.Bits.Initialized = 1;
}
void ttc_can_deinit(u8_t LogicalIndex) {
    Assert_CAN(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_can_config_t* Config = ttc_can_get_configuration(LogicalIndex);
  
    ttc_can_errorcode_e Result = Driver_can_deinit(Config);
    if (Result == ec_can_OK)
      Config->Flags.Bits.Initialized = 0;
}
void ttc_can_reset(u8_t LogicalIndex) {
    Assert_CAN(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_can_config_t* Config = ttc_can_get_configuration(LogicalIndex);

    ttc_memory_set( Config, 0, sizeof(ttc_can_config_t) );

    // load generic default values
    Config->LogicalIndex = LogicalIndex;

    Driver_can_load_defaults(Config);    
}

// ToDo: additional functions go here...
bool ttc_can_check_initialized(ttc_can_config_t* CAN_Generic){
#ifdef TARGET_ARCHITECTURE_STM32
    return stm32f1_can_check_initialized(CAN_Generic);
#endif
    return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_get_features(ttc_can_config_t* CAN_Generic){
#ifdef TARGET_ARCHITECTURE_STM32
    return stm32f1_can_get_features(CAN_Generic);
#endif
    return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_filter_load_defaults(ttc_can_filter_config_t* FilterConfig){
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32f1_can_load_filter_defaults(FilterConfig);
#endif
  return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_filter_init(ttc_can_filter_config_t* FilterConfig){
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32f1_can_filter_init(FilterConfig);
#endif
  return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_send_raw(ttc_can_config_t* CAN_Generic, const u8_t* Buffer, u16_t Amount){
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32f1_can_send_raw(CAN_Generic, (const char*) Buffer, Amount);
#endif
  return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_send_string(ttc_can_config_t* CAN_Generic, const char* Buffer, u16_t MaxLength){
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32f1_can_send_string(CAN_Generic, Buffer, MaxLength);
#endif
  return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_send_word(ttc_can_config_t* CAN_Generic, const u16_t Word){
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32f1_can_send_word(CAN_Generic, Word);
#endif
  return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_read_word(ttc_can_config_t* CAN_Generic, u16_t* Word){
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32f1_can_read_word(CAN_Generic, Word);
#endif
  return ec_NotImplemented;
}

inline ttc_can_errorcode_e ttc_can_read_byte(ttc_can_config_t* CAN_Generic, u8_t* Byte){
#ifdef TARGET_ARCHITECTURE_STM32
  return stm32f1_can_read_byte(CAN_Generic, Byte);
#endif
  return ec_NotImplemented;
}

//} Function definitions
