/** { ttc_can_types.h ************************************************

                           The ToolChain

   High-Level interface for CAN device.
   
   Structures, Enums and Defines being required by both, high- and low-level can.
   

   Authors:
   
}*/

#ifndef TTC_CAN_TYPES_H
#define TTC_CAN_TYPES_H

#include "ttc_basic.h"
#include "stm32f1/stm32f1_can_types.h"

//{ Defines ***************************************************


//} Defines
//{ Static Configuration **************************************

// TTC_CANn has to be defined as constant by makefile.100_board_*
#ifdef TTC_CAN5
  #ifndef TTC_CAN4
    #error TTC_CAN5 is defined, but not TTC_CAN4 - all lower TTC_CANn must be defined!
  #endif
  #ifndef TTC_CAN3
    #error TTC_CAN5 is defined, but not TTC_CAN3 - all lower TTC_CANn must be defined!
  #endif
  #ifndef TTC_CAN2
    #error TTC_CAN5 is defined, but not TTC_CAN2 - all lower TTC_CANn must be defined!
  #endif
  #ifndef TTC_CAN1
    #error TTC_CAN5 is defined, but not TTC_CAN1 - all lower TTC_CANn must be defined!
  #endif

  #define TTC_CAN_AMOUNT 5
#else
  #ifdef TTC_CAN4
    #define TTC_CAN_AMOUNT 4

    #ifndef TTC_CAN3
      #error TTC_CAN5 is defined, but not TTC_CAN3 - all lower TTC_CANn must be defined!
    #endif
    #ifndef TTC_CAN2
      #error TTC_CAN5 is defined, but not TTC_CAN2 - all lower TTC_CANn must be defined!
    #endif
    #ifndef TTC_CAN1
      #error TTC_CAN5 is defined, but not TTC_CAN1 - all lower TTC_CANn must be defined!
    #endif
  #else
    #ifdef TTC_CAN3

      #ifndef TTC_CAN2
        #error TTC_CAN5 is defined, but not TTC_CAN2 - all lower TTC_CANn must be defined!
      #endif
      #ifndef TTC_CAN1
        #error TTC_CAN5 is defined, but not TTC_CAN1 - all lower TTC_CANn must be defined!
      #endif

      #define TTC_CAN_AMOUNT 3
    #else
      #ifdef TTC_CAN2

        #ifndef TTC_CAN1
          #error TTC_CAN5 is defined, but not TTC_CAN1 - all lower TTC_CANn must be defined!
        #endif

        #define TTC_CAN_AMOUNT 2
      #else
        #ifdef TTC_CAN1
          #define TTC_CAN_AMOUNT 1
        #else
          #define TTC_CAN_AMOUNT 0
        #endif
      #endif
    #endif
  #endif
#endif

#ifndef TTC_ASSERT_CAN    // any previous definition set (Makefile)?
#define TTC_ASSERT_CAN 1  // string asserts are enabled by default
#endif
#if (TTC_ASSERT_CAN == 1)  // use Assert()s in CAN code (somewhat slower but alot easier to debug)
  #define Assert_CAN(Condition, ErrorCode) Assert(Condition, ErrorCode)
#else  // us no Assert()s in CAN code (somewhat smaller + faster, but crashes are hard to debug)
  #define Assert_CAN(Condition, ErrorCode)
#endif

//}Static Configuration
//{ Enums/ Structures *****************************************

typedef enum { // ttc_can_mode
    CAN_Normal_Mode,
    CAN_LoopBack_Mode,
    CAN_Silent_Mode,
    CAN_Silent_LoopBack_Mode
} ttc_can_mode;

typedef enum { //ttc_can_synchro
    CAN_Synchro_1q,    //1 time quantum
    CAN_Synchro_2q,    //2 time quantum
    CAN_Synchro_3q,    //3 time quantum
    CAN_Synchro_4q    //4 time quantum
} ttc_can_synchro;

typedef enum { //ttc_can_bit1_tq
    CAN_Bit1_1q,     //1 time quantum
    CAN_Bit1_2q,     //2 time quantum
    CAN_Bit1_3q,     //3 time quantum
    CAN_Bit1_4q,     //4 time quantum
    CAN_Bit1_5q,     //5 time quantum
    CAN_Bit1_6q,     //6 time quantum
    CAN_Bit1_7q,     //7 time quantum
    CAN_Bit1_8q,     //8 time quantum
    CAN_Bit1_9q,     //9 time quantum
    CAN_Bit1_10q,     //10 time quantum
    CAN_Bit1_11q,     //11 time quantum
    CAN_Bit1_12q,     //12 time quantum
    CAN_Bit1_13q,     //13 time quantum
    CAN_Bit1_14q,     //14 time quantum
    CAN_Bit1_15q,     //15 time quantum
    CAN_Bit1_16q     //16 time quantum
}ttc_can_bit1_tq;

typedef enum { //ttc_can_bit2_tq
    CAN_Bit2_1q,     // 1 time quantum
    CAN_Bit2_2q,     // 2 time quantum
    CAN_Bit2_3q,     // 3 time quantum
    CAN_Bit2_4q,     //4 time quantum
    CAN_Bit2_5q,     //5 time quantum
    CAN_Bit2_6q,     //6 time quantum
    CAN_Bit2_7q,     //7 time quantum
    CAN_Bit2_8q     //8 time quantum
}ttc_can_bit2_tq;

typedef struct { //Bits
    unsigned Reserved : 24;
    unsigned TimeTriggerComm : 1; //Enable time triggered communication mode (TTCM).
    unsigned AutoBusOff : 1; // Enable Auto Bus-Off Management (ABOM)
    unsigned AutoWakeUp : 1; //Enable Auto Wake Up Mode (AWUM)
    unsigned NoAutoRetransmission: 1; //Enable Non Auto Retransmission mode (NART)
    unsigned ReceivedFifoLock: 1; //Enable the Receive FIFO Locked mode (RFLM)
    unsigned TransmitFifoPriority: 1; //Enable the Transmit FIFO Priority (TXFP)
}tsg_Bits_t;


/********************************************************************
  CAN Filter Structures

********************************************************************/
typedef struct { //Bits
    unsigned FIFO : 1; //0= Filter FIFO0 Assignment for Filter, 1=FIFO1
    unsigned FilterMode : 1; //0=Identifier Mask Mode (IdMask), 1=Identifier List Mode (IdList)
    unsigned FilterScale : 1; //0=16 bits scale, 1=32 bits scale
    unsigned Activation: 1; // 0=Disable Filter, 1=Enable
}tsg_Filter_Bits_t;

typedef struct { //ttc_can_filter_generic_t
    u16_t CAN_FilterIdHigh;
    u16_t CAN_FilterIdLow;
    u16_t CAN_FilterMaskHigh;
    u16_t CAN_FilterMaskLow;
    u8_t  CAN_FilterNumber;
//    ttc_can_fifo CAN_FilterFifo;
//    ttc_can_filter_mode CAN_FilterMode;
//    ttc_can_filter_scale CAN_FilterScale;
    tsg_Filter_Bits_t Bits;
} ttc_can_filter_config_t;

/********************************************************************
  CAN Messages Structures

********************************************************************/

typedef enum { //ttc_can_id_type
    CAN_Standard,
    CAN_Extended
} ttc_can_id_type;

typedef enum { //ttc_can_trans_req
    CAN_Data,
    CAN_Remote
} ttc_can_trans_req;

typedef struct {  //ttc_can_tx_msg_t
u32_t CAN_Std_ID; //StdId
u32_t CAN_Ext_ID; //ExtId
ttc_can_id_type CAN_Transmit_Id; //IDE
ttc_can_trans_req CAN_Transmit_Request; //RTR
u8_t CAN_Length; //DLC
u8_t CAN_Buffer[8]; //Data
} ttc_can_tx_msg_t;

typedef struct {  //ttc_can_rx_msg_t
u32_t CAN_Std_ID; //StdId
u32_t CAN_Ext_ID; //ExtId
ttc_can_id_type CAN_Transmit_Id; //IDE
ttc_can_trans_req CAN_Transmit_Request; //RTR
u8_t CAN_Length; //DLC
u8_t CAN_Buffer[8]; //Data
u8_t CAN_Filter_Index; //FMI
} ttc_can_rx_msg_t;



typedef enum {    // return codes of CAN devices
    ec_can_OK = 0,
    ec_can_Stuff_Error,
    ec_can_Form_Error,
    ec_can_ACK_Error,
    ec_can_Bit_Recessive_Error,
    ec_can_Bit_Dominant_Error,
    ec_can_CRC_Error,
    ec_can_Software_Set_Error,

    ec_can_ERROR,           // general failure
    ec_can_DeviceNotFound,
    ec_can_InvalidArgument,
    ec_can_DeviceNotReady,

    ec_can_UnknownError
  
  // other failures go here..
} ttc_can_errorcode_e;

typedef struct ttc_can_config_s { // stm32f1 independent configuration data

    // Note: Write-access to this structure is only allowed before first ttc_can_init() call!
    //       After initialization, only read accesses are allowed for high-level code.

    u8_t  LogicalIndex; // automatically set: logical index of device to use (1 = TTC_CAN1, ...)
    ttc_can_arch_t* Architecture; // low-level configuration
    u16_t CAN_Prescaler;
    ttc_can_mode CAN_Mode;
    ttc_can_synchro CAN_Synchro;
    ttc_can_bit1_tq CAN_Bit1;
    ttc_can_bit2_tq CAN_Bit2;
    tsg_Bits_t Bits;
    u8_t Layout;
    ttc_can_tx_msg_t CAN_TX_Message;
    ttc_can_rx_msg_t CAN_RX_Message;
    
    union  { // generic configuration bits common for all low-level Drivers
        u16_t All;
        struct {
          unsigned Initialized        : 1;  // ==1: device has been initialized successfully
          // ToDo: additional high-level flags go here..
            
            unsigned Reserved1        : 16; // pad to 16 bits
        } Bits;
    } Flags;

    // Additional high-level attributes go here..
    
} __attribute__((__packed__)) ttc_can_config_t;

//Additional structures defines




//}Structures

#endif // TTC_CAN_TYPES_H
