/*{ extensions.c *********************************************************
 *
 *               ToolChain_STM32
 *
 * This code manages activated source-code extensions
 * written 2010-2012 by Gregor Rebel
 *
 * Basic framework template for
 *   - Olimex STM32-P107/ STM32-H107/ STM32-LCD eval board
 *   - StdPeripheralsLibrary CMSIS V3.4.0 (tested)
 *   - GNU toolchain from CodeSourcery G++
 *   - FreeRTOS v7.0.1
 *   - FixPointLib 
 *   - uIP TCP/IP-Stack
 *   - Color LCD Panel 320x240
 *   - local/ remote debugging (JTAG-Adapter may be connected to different computer)
 *
 * Coding-style
 *  + gnu99
 *    It's year 2012, get a decent compiler if your's does not comply with this.
 *    arm-none-eabi-gcc-4.4.1 does support gnu99.
 *     
 *  + Text Folding
 *    This source makes use of text folding to improve hierarchical readability 
 *    It has been written with jEdit + plugin Configurable Fold Handler.
 *    Activate folding in your favourite text-editor for { and } 
 *    (fold-start and -end) for best visual experience.
 *
}*/
//{ Includes *************************************************************

#include "ttc_extensions.h"

//} Includes
//{ Global Variables *****************************************************

    struct FirstTaskArguments TaskArguments;

//} Global Variables
//{ Function definitions *************************************************

void startExtensions(void(*MainTask)(void*), u16_t StackSize, void* MainArgument) {
    ttc_hardware_init();
    TaskArguments.MainTask     = MainTask;
    TaskArguments.MainArgument = MainArgument;

    // extensions.active/ttc_extensions_init() is created dynamically by _/compile.sh
    ttc_extensions_init();

    ttc_task_create(firstTask,        // function to start as thread
                    "firstTask",      // thread name (just for debugging)
                    StackSize,        // stack size
                    &TaskArguments,   // passed as argument to firstTask()
                    (Base_t) 1,       // task priority (higher values mean more process time)
                    NULL              // can return a handle to created task
                    );
    
    ttc_task_start_scheduler();
}
void firstTask(void *pvParameters) {
   // u8_t Temp[1000]; //D

    struct FirstTaskArguments* Arguments=(struct FirstTaskArguments*) pvParameters;


    if (Arguments->MainTask != NULL) {
      void(*MainTask)(void*)=Arguments->MainTask;
      MainTask(Arguments->MainArgument);
    }
    while (1)
        mSleep(1000);
}

//} Function definitions
