#ifndef ttc_extensions_H
#define ttc_extensions_H

/*{ extensions.h *********************************************************
 *
 *               ToolChain_STM32
 *
 * This code manages activated source-code extensions
 * written 2010-2012 by Gregor Rebel
 *
 * Basic framework template for
 *   - Olimex STM32-P107/ STM32-H107/ STM32-LCD eval board
 *   - StdPeripheralsLibrary CMSIS V3.4.0 (tested)
 *   - GNU toolchain from CodeSourcery G++
 *   - FreeRTOS v7.0.1
 *   - FixPointLib 
 *   - uIP TCP/IP-Stack
 *   - Color LCD Panel 320x240
 *   - local/ remote debugging (JTAG-Adapter may be connected to different computer)
 *
 * Coding-style
 *  + gnu99
 *    It's year 2012, get a decent compiler if your's does not comply with this.
 *    arm-none-eabi-gcc-4.4.1 does support gnu99.
 *     
 *  + Text Folding
 *    This source makes use of text folding to improve hierarchical readability 
 *    It has been written with jEdit + plugin Configurable Fold Handler.
 *    Activate folding in your favourite text-editor for { and } 
 *    (fold-start and -end) for best visual experience.
 *
}*/
//{ check build environment **********************************************

#ifndef SOURCE_COMPILER
  #error SOURCE_COMPILER not defined -> activate.050_
#endif
#ifndef BOARD
  #error BOARD not defined -> activate.100_board_
#endif
#ifndef uCONTROLLER
  #error uCONTROLLER not defined -> activate.100_board_
#endif
#ifndef UC_DEVICE
  #error UC_DEVICE  not defined -> activate.100_board_
#endif
#ifndef UC_PACKAGE_PINS
  #error UC_PACKAGE_PINS not defined -> activate.100_board_
#endif
//#ifndef RT_SCHEDULER
//  #error RT_SCHEDULER not defined -> activate.300_scheduler_
//#endif
//#ifndef SCHEDULER_HEAP
//  #error SCHEDULER_HEAP not defined -> activate.300_scheduler_heap
//#endif
#ifdef EXTENSION_400_lcd_320x240_olimex
  #ifdef EXTENSION_100_board_olimex_lcd
    #define LCD_BOARD EXTENSION_100_board_olimex_lcd
  #endif
  #ifndef LCD_BOARD
    #error EXTENSION_400_lcd_320x240_olimex requires an LCD-board!
  #endif
#endif

//}
//{ Defines/ TypDefs *****************************************************

struct FirstTaskArguments {
    void (*MainTask)(void*); // function that will be called as main task
    void* MainArgument;
};

//} Defines
//{ Includes *************************************************************

/**
 * IO definitions
 *
 * define access restrictions to peripheral registers
 *
 * (Copied from core_cm3.h because its required to include stm32f10x.h which is required t include core_cm3.h!)
 */

// Basic set of helper functions
#include "ttc_basic.h"

#ifdef EXTENSION_500_ttc_task
#include "ttc_task.h"
#endif
#include "extensions.active/ttc_extensions_active.h"

#ifdef EXTENSION_500_ttc_debug_registers
#ifdef TARGET_ARCHITECTURE_STM32
  #include "stm32_registers.h"
#endif
#endif

//} Includes
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************

/* Starts all activated extensions, spawns first task and then calls given main task
 * @param MainTask      function being called from first task
 * @param StackSize     stack size to use for first task
 * @param MainArgument  passed as argument to MainTask()
 */
void startExtensions(void(*MainTask)(void*), u16_t StackSize, void* MainArgument);

/* task which spawns all other tasks + runs main eventloop
 */
void firstTask(void *pvParameters);

//} Function prototypes

#endif //ttc_extensions_H
