/*{ ttc_gpio.c ***********************************************

 Written by Gregor Rebel 2010-2012

 Architecture independent support for General Purpose Input Output (GPIO).

 Currently implemented architectures: stm32

}*/

#include "ttc_gpio.h"

/** Missing implementation
 *
 * You will find only very little implementation in this file.
 * This is because extra function calls can slow down GPIO manipulations a lot.
 * In ttc_gpio.h, you may find function prototypes and macro defines to DRIVER_xxx() macros.
 * These DRIVER_xxx() macros are implemented in a low-level driver like stm32_gio.c/.h.
 * Decent IDEs like QtCreator or Eclipse should be able to directly show you the implementation
 * used for your actual architecture.
 */

//X bool ttc_gpio_get(GPIO_TypeDef* GPIOx, u8_t Pin);
//X void ttc_gpio_set(GPIO_TypeDef* GPIOx, u8_t Pin);
//X void ttc_gpio_clr(GPIO_TypeDef* GPIOx, u8_t Pin);
//X void ttc_gpio_setn(GPIO_TypeDef* GPIOx, u8_t Pin, BOOL Value);
//X u8_t ttc_gpio_create_index8(ttc_GPIO_BANK GPIOx, u8_t Pin);
//X void ttc_gpio_from_index8(u8_t PhysicalIndex, ttc_GPIO_BANK* GPIOx, u8_t* Pin);

//X void ttc_gpio_init(ttc_GPIO_BANK GPIOx, u8_t Pin, ttc_gpio_mode_e Type) {
//X     ttc_gpio_init2(GPIOx, Pin, Type, tgs_Max);
//X }
u8_t ttc_gpio_create_index8(ttc_GPIO_BANK Bank, u8_t Pin) {
#ifdef DRIVER_ttc_gpio_create_index8
   return DRIVER_ttc_gpio_create_index8(Bank, Pin);
#else
#  warning Missing implementation! // see description above for missing low-level implementation
    return 0;
#endif
}
void ttc_gpio_init(ttc_GPIO_BANK Bank, u8_t Pin, ttc_gpio_mode_e Type) {
#ifdef DRIVER_ttc_gpio_init
    DRIVER_ttc_gpio_init(Bank, Pin, Type, tgs_Max);
#else
    (void) Bank;
    (void) Pin;
    (void) Type;
#  warning Missing implementation! // see description above for missing low-level implementation
#endif
}
inline void ttc_gpio_init_variable(ttc_Port_t* Port, ttc_GPIO_BANK Bank, u8_t Pin, ttc_gpio_mode_e Type) {
#ifdef DRIVER_ttc_gpio_init_variable
    DRIVER_ttc_gpio_init_variable(Port, Bank, Pin, Type, tgs_Max);
#else
#   warning Missing implementation! // see description above for missing low-level implementation
#endif
}
void ttc_gpio_variable(ttc_Port_t* Port, ttc_GPIO_BANK Bank, u8_t Pin) {

#ifdef DRIVER_ttc_gpio_variable
    DRIVER_ttc_gpio_variable(Port, Bank, Pin);
#else
#  warning Missing implementation! // see description above for missing low-level implementation
#endif
}
void _ttc_gpio_init_u16(ttc_GPIO_BANK Bank, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {
    ttc_gpio_init_u8_pin4(Bank, 0, Type, Speed);
    ttc_gpio_init_u8_pin4(Bank, 8, Type, Speed);
}
u16_t _ttc_gpio_get_u16(ttc_GPIO_BANK GPIOx) {

    // fallback implementation if no special low-level support is available
    u16_t Value = ttc_gpio_get_u8_pin2(GPIOx, 8);
    Value = Value << 8;
    Value |= ttc_gpio_get_u8_pin2(GPIOx, 0);

    return Value;
}
void _ttc_gpio_set_u16(ttc_GPIO_BANK Bank, u16_t Value) {

    // fallback implementation if no special low-level support is available
    ttc_gpio_set_u8_pin3(Bank, 0, 0x00ff & Value);
    ttc_gpio_set_u8_pin3(Bank, 8, (0xff00 & Value) >> 8);
}
void _ttc_gpio_init_u8_pin(ttc_GPIO_BANK Bank, u8_t FirstPin, ttc_gpio_mode_e Type, ttc_gpio_speed_e Speed) {

    // fallback implementation if no special low-level support is available
    ttc_gpio_init4(Bank, FirstPin    , Type, Speed);
    ttc_gpio_init4(Bank, FirstPin + 1, Type, Speed);
    ttc_gpio_init4(Bank, FirstPin + 2, Type, Speed);
    ttc_gpio_init4(Bank, FirstPin + 3, Type, Speed);
    ttc_gpio_init4(Bank, FirstPin + 4, Type, Speed);
    ttc_gpio_init4(Bank, FirstPin + 5, Type, Speed);
    ttc_gpio_init4(Bank, FirstPin + 6, Type, Speed);
    ttc_gpio_init4(Bank, FirstPin + 7, Type, Speed);
}
u8_t _ttc_gpio_get_u8_pin(ttc_GPIO_BANK GPIOx, u8_t FirstPin) {

    // fallback implementation if no special low-level support is available
    u8_t Value = 0;
    if (ttc_gpio_get2(GPIOx, FirstPin  ))  Value  = 0x01;
    if (ttc_gpio_get2(GPIOx, FirstPin+1))  Value |= 0x02;
    if (ttc_gpio_get2(GPIOx, FirstPin+2))  Value |= 0x04;
    if (ttc_gpio_get2(GPIOx, FirstPin+3))  Value |= 0x08;
    if (ttc_gpio_get2(GPIOx, FirstPin+4))  Value |= 0x10;
    if (ttc_gpio_get2(GPIOx, FirstPin+5))  Value |= 0x20;
    if (ttc_gpio_get2(GPIOx, FirstPin+6))  Value |= 0x40;
    if (ttc_gpio_get2(GPIOx, FirstPin+7))  Value |= 0x80;

    return Value;
}
void _ttc_gpio_set_u8_pin(ttc_GPIO_BANK GPIOx, u8_t FirstPin, u8_t Value) {

    // fallback implementation if no special low-level support is available
    ttc_gpio_setn3(GPIOx, FirstPin,   Value & 0x01);
    ttc_gpio_setn3(GPIOx, FirstPin+1, Value & 0x02);
    ttc_gpio_setn3(GPIOx, FirstPin+2, Value & 0x04);
    ttc_gpio_setn3(GPIOx, FirstPin+3, Value & 0x08);
    ttc_gpio_setn3(GPIOx, FirstPin+4, Value & 0x10);
    ttc_gpio_setn3(GPIOx, FirstPin+5, Value & 0x20);
    ttc_gpio_setn3(GPIOx, FirstPin+6, Value & 0x40);
    ttc_gpio_setn3(GPIOx, FirstPin+7, Value & 0x80);
}
