/*{ ttc_gui.c *******************************



}*/

#include "ttc_gui.h"

//{ Global Variables *****************************************************

gni_NumberArea_t* NumberAreas_Used  = NULL;

// for each initialized input, a pointer to its generic and architecture definitions is stored

//} Global Variables

// functions that can be provided by low-level driver (set during ttc_gfx_reset() )


//{ Function definitions *************************************************

void ttc_gui_create_push_button(void (*TouchHandler)(void* Argument), void* Argument, u16_t Left, u16_t Top, u16_t Right, u16_t Bottom, const char* Text)
{
    u8_t Dist=(Right-Left-strlen(Text)*16)/2;
    ttc_input_register_input_area(TouchHandler,Argument,Left,Top,Right,Bottom,Text,gss_Rounded,me_MouseMove);
    ttc_gfx_print_between(Left+Dist,Top+(Bottom-Top)/2-10,Text,15);

    ttc_gfx_rect(Left,Top,Right-Left,Bottom-Top);

}

void ttc_gui_clear()
{
    function_clear();
    ttc_input_delete_all_input_areas();
    ttc_gui_delete_all_number_areas();
}

void ttc_gui_plus_button(u16_t Left, u16_t Top, u16_t Right, u16_t Bottom,u16_t* value)
{
    ttc_input_register_input_area(_ttc_add_one,(void*)value,Left,Top,Right,Bottom,NULL,gss_Rounded,me_MouseMove);
    function_rect(Left,Top,Right-Left,Bottom-Top);
    function_rect_fill(Left+(Right-Left)/8,Top+(Bottom-Top)/3+1,Right-Left-(Right-Left)/4,(Bottom-Top)/3); //draw vertikal square
    function_rect_fill(Left+(Right-Left)/3+1,Top+(Bottom-Top)/8,(Right-Left)/3,Bottom-Top-(Bottom-Top)/4); //draw horizontal square
}

void ttc_gui_minus_button(u16_t Left, u16_t Top, u16_t Right, u16_t Bottom,u16_t* value)
{
    ttc_input_register_input_area(_ttc_sub_one,(void*)value,Left,Top,Right,Bottom,NULL,gss_Rounded,me_MouseMove);
    function_rect(Left,Top,Right-Left,Bottom-Top);
    function_rect_fill(Left+(Right-Left)/8,Top+(Bottom-Top)/3+1,Right-Left-(Right-Left)/4,(Bottom-Top)/3);

}

void ttc_gui_create_number_field(u16_t X,u16_t Y,u16_t* number)
{
    gni_NumberArea_t* Number_Data;

    if(!NumberAreas_Used) //if Input area list is empty
    {
        Number_Data = NumberAreas_Used = (gni_NumberArea_t*) ttc_memory_alloc_zeroed(sizeof(gni_NumberArea_t)); //the first pointer get the address of allocated memory
        Number_Data->X=X;
        Number_Data->Y=Y;
        Number_Data->number=number;
    }
    else //if there is already a first pointer
    {
        Number_Data = NumberAreas_Used; //hand the adress over to Number_Data
        while(Number_Data->Next)
            Number_Data=Number_Data->Next; //while there is a next pointer, Number_Data become the next pointer
        //at the end of the list
        Number_Data->Next = (gni_NumberArea_t*) ttc_memory_alloc_zeroed(sizeof(gni_NumberArea_t)); //link old part with the new memory space
        Number_Data=Number_Data->Next; //the current pointer is the new pointer now
        Number_Data->X=X; //defines X to new struct
        Number_Data->Y=Y; //defines Y to new struct
        Number_Data->number=number; //hand over the number pointer
    }
}

void ttc_gui_delete_all_number_areas() {
    gni_NumberArea_t* Number_Data= NumberAreas_Used;
    while(Number_Data)
    {
        NumberAreas_Used=Number_Data->Next;
        ttc_memory_free((void*)Number_Data);
        Number_Data=NumberAreas_Used;
    }

    return;

}

void ttc_gui_number_task()
{
    char Buffer[7];
    gni_NumberArea_t* Number_Data= NumberAreas_Used;
    while(1)
    {
        for(Number_Data= NumberAreas_Used;Number_Data;Number_Data=Number_Data->Next)
        {
            ttc_gfx_text_cursor_set(Number_Data->X,Number_Data->Y);
            ttc_string_snprintf((u8_t*)&Buffer,6,"%i",*(Number_Data->number));
            ttc_gfx_print_solid((const char*)&Buffer,7);
            ttc_gfx_put_char_solid(' ');

        }
        mSleep(3000);

    }
}

void ttc_gui_number_refresh()
{
    char Buffer[7];
    gni_NumberArea_t* Number_Data= NumberAreas_Used;

    for(Number_Data= NumberAreas_Used;Number_Data;Number_Data=Number_Data->Next)
    {
        ttc_gfx_text_cursor_set(Number_Data->X,Number_Data->Y);
        ttc_string_snprintf((u8_t*)&Buffer,6,"%i",*(Number_Data->number));
        ttc_gfx_print_solid((const char*)&Buffer,7);
        ttc_gfx_put_char_solid(' ');

    }
}

void ttc_gui_start_number()
{
    ttc_task_create(ttc_gui_number_task,       // function to start as thread
                    "task_number",              // thread name (just for debugging)
                    128,                // stack size
                    (void *) NULL,      // passed as argument to taskGUIs()
                    1,                  // task priority (higher values mean more process time)
                    NULL                // can return a handle to created task
                    );
}

void _ttc_add_one(void* wert)
{

    u16_t* value= (u16_t*)wert;
    *value=1+*value;
    ttc_gui_number_refresh();
    mSleep(ADD_DELAY);

}

void _ttc_sub_one(void* wert)
{
    u16_t* value= (u16_t*)wert;
    if (*value>0){
        *value=-1+*value;
        ttc_gui_number_refresh();
        mSleep(SUB_DELAY);
    }
}
//} Function definitions
