#ifndef ttc_list_types_H
#define ttc_list_types_H

/** ttc_list.h ***********************************************{
 *
 * Written by Gregor Rebel 2013
 *
 * Architecture independent support for single linked lists.
 * 
}*/

//{ includes

#include "ttc_basic.h"
#include "ttc_mutex_types.h"
#include "ttc_semaphore_types.h"

//}includes

#ifndef TTC_LIST_STATISTICS
#define TTC_LIST_STATISTICS 1
#endif
#if TTC_LIST_STATISTICS == 1
#  define LIST_STATS_INCREASE(LIST,FIELD)  LIST->Stats.FIELD++
#else
#  define LIST_STATS_INCREASE(LIST,FIELD)
#endif

// value used as Next-pointer in last list item
#define TTC_LIST_TERMINATOR (ttc_list_item_t*) 0xfefefefe

//{ enums/ structures *******************************************************

typedef enum ttc_list_error_E {      // return type of most functions in ttc_list
    tle_OK,                          // no error
    tle_ListInUseByTask,             // an _isr()-call occured while the list was in use by a task

    tle_Error,                       // errors come below
    // Error Conditions below
    tle_TimeOut,                     // an operation has timed out
    tle_NotImplemented,              // required function not implemented for current architecture
    tle_ListIsEmpty,                 // pulling data from a list has failed because its empty
    tle_UNKNOWN
} ttc_list_error_e;

/** Header of all kinds of data being able to be queued
  */
typedef struct ttc_list_item_s {
    struct ttc_list_item_s* Next;
} ttc_list_item_t;

typedef enum {
    tlo_None,
    tlo_PushBackSingle,
    tlo_PushBackSingleISR,
    tlo_PushBackMultiple,
    tlo_PushBackMultipleISR,
    tlo_PopFrontSingle,
    tlo_PopFrontSingleISR,
    tlo_PopFrontMultiple,
    tlo_PopFrontMultipleISR,
    tlo_End
} ttc_list_operation_e;

typedef struct ttc_list_statistics_s { // statistic data of single list
    Base_t AmountPending;    // total amount of pending operations so far
    Base_t AmountPushBack;   // total amount of push back operations so far
    Base_t AmountPopFront;   // total amount of pull front operations so far
    ttc_list_operation_e Operation;
    const char* Name;
} ttc_list_statistics_t;

// ttc_ListSize_t always uses native data width of current architecture
// This avoids mutex locks in ttc_list_size()
#if TARGET_DATA_WIDTH == 32
  typedef u32_t ttc_ListSize_t;
#elif TARGET_DATA_WIDTH == 16
  typedef u16_t ttc_ListSize_t;
#else
  typedef u8_t ttc_ListSize_t;
#endif

/** Management data of single linked list
  */
typedef struct {
    ttc_mutex_smart_t Lock;        // protects this list from concurrent accesses
    ttc_semaphore_smart_t AmountItems; // used by tasks waiting for data
#ifdef TTC_REGRESSION
    volatile ttc_list_operation_e  LockedBy; // software only lock used to detect faulty ttc_mutex implementation of Lock
    ttc_list_statistics_t Stats; // statistic data of this list
#endif
    ttc_list_item_t* First; // pointer to first item
    ttc_list_item_t* Last;  // pointer to last item

    // list of items being added by interrupt-service routine while list was locked
    SmallBase_t      Pending_Push_Amount;
    ttc_list_item_t* Pending_Push_First;
    ttc_list_item_t* Pending_Push_Last;

    ttc_ListSize_t   Size;  // amount of items currently stored
} ttc_list_t;

//}

#endif
