/** ttc_semaphore.c ***********************************************
 *
 * Written by Gregor Rebel 2010-2012
 *
 * Architecture independent support for inter task communication & synchronization.
 * 
 */
#include "ttc_semaphore.h"

//{ global variables ********************************************

#if TTC_SMART_SEMAPHORE_HISTORY_SIZE > 0

    // stores pointers to smart semaphores given for latest ttc_semaphore_smart_take() calls
    ttc_semaphore_smart_t* tml_LastSmartSemaphore[TTC_SMART_SEMAPHORE_HISTORY_SIZE];

    // history of smart semaphores without doubles
    ttc_semaphore_smart_t* tml_UniqueSmartSemaphore[TTC_SMART_SEMAPHORE_HISTORY_SIZE];

    // index of next valid entry inside tml_LastSmartSemaphore[]
    u8_t tml_Index_LastSmartSemaphore = 0;
    u8_t tml_Index_UniqueSmartSemaphore = 0;

    u32_t tml_AmountSmartAmounts = 0;
#endif

//}global variables
//{ functions *************************************************

    void ttc_semaphore_wait(ttc_semaphore_t* Semaphore) {

    if (1) ttc_task_yield(); // active waiting for semaphore
    else {                   // put task to sleep (will be awaken by _give()/ _give_isr() )  ToDo: Activating this gives strange hardfault crashes and deadlocks!
        /* Note: Write access to Semaphore->FirstWaitingTask/ LastWaitingTask is allowed only here! */

        ttc_semaphore_waiting_task_t ThisWaitingTask;
        ThisWaitingTask.Task = ttc_task_current_handle();
        ThisWaitingTask.Next = NULL;
        Assert_SEMAPHORES(ttc_memory_is_writable(Semaphore), ec_InvalidArgument);

        ttc_task_begin_criticalsection(ttc_semaphore_wait); //? ToDo: Required
        if (Semaphore->FirstWaitingTask) { // >=1 task waiting: append ourself at end of list
            Semaphore->LastWaitingTask->Next = &ThisWaitingTask;
            Semaphore->LastWaitingTask       = &ThisWaitingTask;
        }
        else {                         // we're the only task waiting
            Semaphore->LastWaitingTask  = &ThisWaitingTask;
            Semaphore->FirstWaitingTask = &ThisWaitingTask;
        }

        ttc_task_end_criticalsection();
        ttc_task_suspend(ThisWaitingTask.Task);

        // got woken up: remove ourself from list of waiting tasks
        Semaphore->FirstWaitingTask = ThisWaitingTask.Next;
    }
}

// private functions ********************************************

      ttc_semaphore_t* _ttc_semaphore_create() {

#ifdef DRIVER_ttc_semaphore_create
    ttc_semaphore_t* NewSemaphore = ttc_memory_alloc_zeroed( sizeof(ttc_semaphore_t) );
    _ttc_semaphore_init(NewSemaphore);

    return NewSemaphore;
#else
    return NULL;
#endif
}
                  void _ttc_semaphore_init(ttc_semaphore_t* Semaphore) {

    ttc_memory_set(Semaphore, 0, sizeof(ttc_semaphore_t));
#ifdef DRIVER_ttc_semaphore_init
    DRIVER_ttc_semaphore_init(&(Semaphore->Value));
#endif
}
 ttc_semaphore_error_e _ttc_semaphore_take(ttc_semaphore_t* Semaphore, Base_t Amount, Base_t TimeOut) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

#if TTC_SEMAPHORE_TIMEOUT_USECS > 0
    if (TimeOut == -1)
        TimeOut = TTC_SEMAPHORE_TIMEOUT_USECS;
#endif

    ttc_semaphore_error_e Error = tsme_NotImplemented;

//? #ifdef DRIVER_ttc_semaphore_try_take
//?     while (DRIVER_ttc_semaphore_try_take(&(Semaphore->Value), Amount) != tsme_OK) {
//?         ttc_semaphore_wait(Semaphore);
//?     }
//? #else
#ifdef DRIVER_ttc_semaphore_take
    Error = DRIVER_ttc_semaphore_take(&(Semaphore->Value), Amount, TimeOut);
#endif
//? #endif

#if TTC_SEMAPHORE_TIMEOUT_USECS > 0
    if (Error == tsme_TimeOut) {
         Assert(TimeOut != TTC_SEMAPHORE_TIMEOUT_USECS, ec_TimeOut); // endless semaphore timed out!
    }
#endif

    return Error;
}
 ttc_semaphore_error_e _ttc_semaphore_try_take(ttc_semaphore_t* Semaphore, Base_t Amount) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

#ifdef DRIVER_ttc_semaphore_try_take
    return DRIVER_ttc_semaphore_try_take(&(Semaphore->Value), Amount);
#else
    return tsme_NotImplemented;
#endif
}
 ttc_semaphore_error_e _ttc_semaphore_take_isr(ttc_semaphore_t* Semaphore, Base_t Amount) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);
    //X static signed portBASE_TYPE HigherPrioTaskWoken; HigherPrioTaskWoken = FALSE;

#ifdef DRIVER_ttc_semaphore_take_isr
    return DRIVER_ttc_semaphore_take_isr(&(Semaphore->Value), Amount);
#else
    return tsme_NotImplemented;
#endif
}
                  BOOL _ttc_semaphore_can_provide(ttc_semaphore_t* Semaphore, Base_t Amount) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

#ifdef DRIVER_ttc_semaphore_can_provide
    return DRIVER_ttc_semaphore_can_provide(&(Semaphore->Value), Amount);
#else
    return tsme_NotImplemented;
#endif
}
 ttc_semaphore_error_e _ttc_semaphore_give(ttc_semaphore_t* Semaphore, Base_t Amount) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

    ttc_task_begin_criticalsection(_ttc_semaphore_give);
    ttc_semaphore_error_e Error;

#ifdef DRIVER_ttc_semaphore_give
    Error = DRIVER_ttc_semaphore_give(&(Semaphore->Value), Amount);
#else
    Error = tsme_NotImplemented;
#endif

    if ( (Error == tsme_OK) && Semaphore->FirstWaitingTask ) { // a task is waiting on this semaphore: wake it up
        ttc_task_resume(Semaphore->FirstWaitingTask->Task);
    }
    ttc_task_end_criticalsection();

    return Error;
}
 ttc_semaphore_error_e _ttc_semaphore_give_isr(ttc_semaphore_t* Semaphore, Base_t Amount) {
    Assert_SEMAPHORES(Semaphore != 0, ec_InvalidArgument);

    ttc_semaphore_error_e Error;

#ifdef DRIVER_ttc_semaphore_give_isr
    Error = DRIVER_ttc_semaphore_give_isr(&(Semaphore->Value), Amount);
#else
    Error = tsme_NotImplemented;
#endif

    if ( (Error == tsme_OK) && Semaphore->FirstWaitingTask ) { // a task is waiting on this semaphore: wake it up
        ttc_task_resume(Semaphore->FirstWaitingTask->Task);
    }

    return Error;
}

#if TTC_SMART_SEMAPHORES == 1
                Base_t _ttc_semaphore_smart_available(ttc_semaphore_smart_t* SmartSemaphore) {
    Assert_SEMAPHORES(SmartSemaphore, ec_InvalidArgument);

    return DRIVER_ttc_semaphore_available(&(SmartSemaphore->Semaphore.Value));
}
ttc_semaphore_smart_t* _ttc_semaphore_smart_create() {

     ttc_semaphore_smart_t* NewSemaphore = ttc_memory_alloc( sizeof(ttc_semaphore_smart_t) );
     ttc_semaphore_init(NewSemaphore);

     return NewSemaphore;
}
                  void _ttc_semaphore_smart_init(ttc_semaphore_smart_t* Semaphore) {

     _ttc_semaphore_init(&(Semaphore->Semaphore));
     Semaphore->Owner = NULL;
}
#if TTC_SMART_SEMAPHORE_HISTORY_SIZE > 0
void _ttc_semaphore_record_history(ttc_semaphore_smart_t* SmartSemaphore, void (*Owner)()) {

    tml_AmountSmartAmounts++;
    tml_LastSmartSemaphore[tml_Index_LastSmartSemaphore++] = SmartSemaphore;
    if (tml_Index_LastSmartSemaphore >= TTC_SMART_SEMAPHORE_HISTORY_SIZE)
        tml_Index_LastSmartSemaphore = 0;

    BOOL SemaphoreIsKnown = FALSE;
    for (int I = 0; (I < tml_Index_UniqueSmartSemaphore) && (I < TTC_SMART_SEMAPHORE_HISTORY_SIZE); I++) {
        if (tml_UniqueSmartSemaphore[I] == SmartSemaphore) {
            SemaphoreIsKnown = TRUE;
            break;
        }
    }
    if ( (!SemaphoreIsKnown) && (tml_Index_UniqueSmartSemaphore < TTC_SMART_SEMAPHORE_HISTORY_SIZE) )
        tml_UniqueSmartSemaphore[tml_Index_UniqueSmartSemaphore++] = SmartSemaphore;

}
#endif
 ttc_semaphore_error_e _ttc_semaphore_smart_take(ttc_semaphore_smart_t* SmartSemaphore, Base_t Amount, Base_t TimeOut, void (*Owner)() ) {
    Assert_SEMAPHORES(SmartSemaphore, ec_InvalidArgument);

#if TTC_SMART_SEMAPHORE_HISTORY_SIZE > 0
    _ttc_semaphore_record_history(SmartSemaphore, Owner);
#endif
    ttc_semaphore_error_e Error = _ttc_semaphore_take(&(SmartSemaphore->Semaphore), Amount, TimeOut);
    if (Error == tsme_OK) {
#ifdef TTC_REGRESSION
        // copy info about current task into smart semaphore
        SmartSemaphore->OwnerTask = ttc_task_info(NULL);
#endif
        SmartSemaphore->Owner = Owner;
    }

    return Error;
}
 ttc_semaphore_error_e _ttc_semaphore_smart_try_take(ttc_semaphore_smart_t* SmartSemaphore, Base_t Amount, void (*Owner)() ) {
    Assert_SEMAPHORES(SmartSemaphore, ec_InvalidArgument);

#if TTC_SMART_SEMAPHORE_HISTORY_SIZE > 0
    _ttc_semaphore_record_history(SmartSemaphore, Owner);
#endif
    ttc_semaphore_error_e Error = _ttc_semaphore_try_take(&(SmartSemaphore->Semaphore), Amount);
    if (Error == tsme_OK) {
#ifdef TTC_REGRESSION
        // copy info about current task into smart semaphore
        SmartSemaphore->OwnerTask = ttc_task_info(NULL);
#endif
        SmartSemaphore->Owner = Owner;
    }

    return Error;
}
 ttc_semaphore_error_e _ttc_semaphore_smart_take_isr(ttc_semaphore_smart_t* SmartSemaphore, Base_t Amount, void (*Owner)() ) {
    Assert_SEMAPHORES(SmartSemaphore, ec_InvalidArgument);

#if TTC_SMART_SEMAPHORE_HISTORY_SIZE > 0
    _ttc_semaphore_record_history(SmartSemaphore, Owner);
#endif
    ttc_semaphore_error_e Error = _ttc_semaphore_take_isr(&(SmartSemaphore->Semaphore), Amount);
    if (Error == tsme_OK) {
#ifdef TTC_REGRESSION
        // copy info about current task into smart semaphore
        SmartSemaphore->OwnerTask = ttc_task_info(NULL);
#endif
        SmartSemaphore->Owner = Owner;
    }

    return Error;
}
 ttc_semaphore_error_e _ttc_semaphore_smart_give(ttc_semaphore_smart_t* SmartSemaphore, Base_t Amount) {
    Assert_SEMAPHORES(SmartSemaphore, ec_InvalidArgument);

    if (0) SmartSemaphore->Owner = NULL;

    return _ttc_semaphore_give(&(SmartSemaphore->Semaphore), Amount);
}
 ttc_semaphore_error_e _ttc_semaphore_smart_give_isr(ttc_semaphore_smart_t* SmartSemaphore, Base_t Amount) {
    Assert_SEMAPHORES(SmartSemaphore, ec_InvalidArgument);

    if (0) SmartSemaphore->Owner = NULL;

    return _ttc_semaphore_give_isr(&(SmartSemaphore->Semaphore), Amount);
}
                  BOOL _ttc_semaphore_smart_can_provide(ttc_semaphore_smart_t* SmartSemaphore, Base_t Amount) {
    Assert_SEMAPHORES(SmartSemaphore != 0, ec_InvalidArgument);

    return _ttc_semaphore_can_provide(&(SmartSemaphore->Semaphore), Amount);
}
#endif

//}functions
