/*{ ttc_task.c ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Scheduler and architecture independent multitasking support.
 * 
}*/
#include "ttc_task.h"

// increased by _ttc_task_begin_criticalsection_owner(); decreased by ttc_task_end_criticalsection_owner()
s16_t ttc_task_CountCriticalSection = 0;

// single linked list of task info data of all created tasks
ttc_task_info_t* ttc_task_TaskInfos = NULL;

// amount of tasks currently running
u8_t ttc_task_AmountTasks = 0;

// set to TRUE once ttc_task_start_scheduler() has been called
BOOL tt_SchedulerRunning = FALSE;

            BOOL ttc_task_create( void TaskFunction(void*), const char* TaskName, u16_t StackSize, void* Argument, u8_t Priority, ttc_task_info_t** Handle) {

    Assert(TaskFunction, ec_InvalidArgument); // this argument must not be NULL!
    Assert(StackSize,    ec_InvalidArgument); // this argument must not be zero!

    BOOL Result = FALSE;
    ttc_task_info_t* TaskInfo = NULL;
    if (ttc_memory_is_writable(TaskName)) { // got task name in RAM: copy to new buffer
        TaskInfo = ttc_memory_alloc_zeroed(sizeof(ttc_task_info_t) + ttc_string_length8(TaskName, -1) + 1);
        TaskInfo->Name = (const char*) (TaskInfo + 1); // storing task name right after task info
        ttc_string_copy( (u8_t*) TaskInfo->Name, (const u8_t*) TaskName, 255); // casting away const for TaskInfo->Name!
    }
    else {
        TaskInfo = ttc_memory_alloc_zeroed(sizeof(ttc_task_info_t));
        TaskInfo->Name = TaskName;  // task name is constant: store only reference
    }

    TaskInfo->StackSize_Bytes = StackSize;
    TaskInfo->Function        = TaskFunction;
    TaskInfo->Argument        = Argument;
    TaskInfo->Priority        = Priority;

    // append new entry at list head (faster than at end)
    ttc_task_begin_criticalsection(ttc_task_create);
    TaskInfo->Next = ttc_task_TaskInfos;
    ttc_task_TaskInfos = TaskInfo;
    ttc_task_end_criticalsection();

    if (Handle) {
        Assert(ttc_memory_is_writable(Handle), ec_InvalidArgument);
    }

#ifdef DRIVER_ttc_task_create
    Result = DRIVER_ttc_task_create(TaskFunction, StackSize, Argument, Priority, TaskInfo);
#else // no scheduler available: use plain function call
    (void) TaskFunction;
    (void) TaskName;
    (void) StackSize;
    (void) Argument;
    (void) Priority;
    (void) Handle;

#warning ttc_task_create() will use plain function calls
    TaskFunction(Argument);
    return FALSE;
#endif

    Assert(Result, ec_UNKNOWN); // task could not be created
    Assert(TaskInfo->Handle, ec_NULL); // Result was OK, but no handle returned (broken implementation of DRIVER_ttc_task_create()?)
    if (Handle)
        *Handle = TaskInfo;

    return Result;
}
            void ttc_task_calibrate_usleep() {
#ifdef TTC_LED1
  ttc_Port_t Port_LED1;
  ttc_gpio_init_variable(&Port_LED1, TTC_LED1, tgm_output_push_pull);

  while (1) {
    ttc_gpio_set_variable(&Port_LED1);
    uSleep(1000);           // 1ms high time
    ttc_gpio_clr_variable(&Port_LED1);

    uSleep(100);            // 100us low time

  }
#else
  Assert(0, ec_UNKNOWN); // ttc_task_calibrate_usleep() requires definition of TTC_LED1! (E.g.; #define TTC_LED1 PA7)
#endif

}
            void _ttc_task_msleep(Base_t Delay) {
    if (tt_SchedulerRunning) {
        for (;Delay > 10; Delay-=10)
#ifdef DRIVER_ttc_task_usleep
            DRIVER_ttc_task_usleep(10000);
#else
            for (u16_t Delay2 = 10000; Delay2 > 0; Delay2--);
#endif

        for (;Delay > 0; Delay--)
#ifdef DRIVER_ttc_task_usleep
            DRIVER_ttc_task_usleep(1000);
#else
            for (u16_t Delay2 = 1000; Delay2 > 0; Delay2--);
#endif
    }
    else { // no scheduler available: fake msleep
        for (;Delay > 0; Delay-=10)
            for (u16_t Delay2 = 1000; Delay2 > 0; Delay2--);
    }
}
            void _ttc_task_msleep(Base_t Delay);
#define mSleep(Delay) ttc_task_msleep(Delay)

#ifdef DRIVER_ttc_task_begin_criticalsection
            void ttc_task_begin_criticalsection_owner(void (*Owner)()) {
    static void (*LastOwner)() = NULL;
    static TaskHandle_t LockedBy = NULL;
    (void) LastOwner;  // avoid warning "unused variable"

    if (ttc_task_CountCriticalSection) {
        Assert_Task(ttc_task_CountCriticalSection < 1000, ec_InvalidImplementation);
        Assert_Task(LockedBy == ttc_task_current_handle(), ec_InvalidImplementation); // already locked by another task!
    }
    else {
        DRIVER_ttc_task_begin_criticalsection();
        LockedBy = ttc_task_current_handle();
    }

    ttc_task_CountCriticalSection++;
    LastOwner = Owner;
}
#endif
#ifdef DRIVER_ttc_task_end_criticalsection
            void ttc_task_end_criticalsection_owner() {
    ttc_task_CountCriticalSection--;
    Assert(ttc_task_CountCriticalSection >= 0, ec_InvalidImplementation);

    if (ttc_task_CountCriticalSection==0)
        DRIVER_ttc_task_end_criticalsection();
}
#endif
   Base_signed_t ttc_task_check_stack() {
#ifdef INCLUDE_uxTaskGetStackHighWaterMark
    volatile int StackBytesFree = DRIVER_ttc_task_check_stack();
    Assert(StackBytesFree > 0, ec_Malloc); // stack is full: stop immediately

    return StackBytesFree; // <-- place your breakpoint right here!
#else
    return 1;
#endif
}
ttc_task_info_t* ttc_task_info(TaskHandle_t Handle) {
    if (!Handle)
        Handle = ttc_task_current_handle();

    ttc_task_info_t* Info = ttc_task_find_info(Handle);
    DRIVER_ttc_task_update_info(Handle, Info);

    return Info;
}
ttc_task_info_t* ttc_task_find_info(TaskHandle_t Handle) {
    ttc_task_info_t* TaskInfo = ttc_task_TaskInfos;

    // get entry containing given handle
    while (TaskInfo && (TaskInfo->Handle != Handle)) {
#ifdef TTC_REGRESSION
        Assert_Task(ttc_memory_is_writable(TaskInfo), ec_InvalidImplementation);
#endif
        TaskInfo = TaskInfo->Next;
    }

    return TaskInfo;
}
 ttc_task_all_t* ttc_task_get_all(ttc_task_all_t* TaskInfos) {
    static ttc_task_all_t* MyInfo = NULL;
    static u8_t Size_MyInfo = 0;
    if (!TaskInfos) {
        u8_t AmountOfTasks = DRIVER_ttc_task_get_amount();
        if (Size_MyInfo < AmountOfTasks)
            AmountOfTasks++; // allocating one entry ahead
            MyInfo = ttc_memory_alloc_zeroed( sizeof(ttc_task_all_t) +
                                              (sizeof(ttc_task_info_t) * AmountOfTasks)
                                            );
        TaskInfos = MyInfo;
        Size_MyInfo = AmountOfTasks;
    }
    DRIVER_ttc_task_get_all(TaskInfos);

    return TaskInfos;
}
            void ttc_task_start_scheduler() {

#ifdef DRIVER_ttc_task_start_scheduler
    tt_SchedulerRunning = TRUE;
    DRIVER_ttc_task_start_scheduler();
#endif
}
            BOOL ttc_task_is_scheduler_running() {
    return tt_SchedulerRunning;
}
            void ttc_task_suspend(TaskHandle_t Handle) {

#ifdef DRIVER_ttc_task_suspend
    DRIVER_ttc_task_suspend(Handle);
#else
#   warning ttc_task_suspend(Handle) - missing low-level implementation
#endif
}
            void _ttc_task_yield() {
    DRIVER_ttc_task_yield();
}

/** Dummy implementations for missing low-level drivers
   *
   * Functions below try to provide a single-task implementation for missing scheduler functionality.
   *
   */

 void _ttc_task_fallback_usleep(Base_t Delay) {
     for (; Delay > 0; Delay--);
 }
