#ifndef ttc_task_types_H
#define ttc_task_types_H

/*{ ttc_task.h ***********************************************
 
 * Written by Gregor Rebel 2010-2012
 *
 * Architecture independent support for multitasking.
 * 
 * Currently implemented architectures: stm32 + FreeRTOS
 * 
}*/

//{ includes

#ifdef EXTENSION_300_scheduler_free_rtos
#  include "scheduler/freertos_task_types.h"
#endif

#ifndef TaskHandle_t
#  error Missing type definition in low-level driver!
#endif

//}includes
//{ constants

#ifndef TTC_ASSERT_TASK    // any previous definition set (Makefile)?
#define TTC_ASSERT_TASK 1  // string asserts are enabled by default
#endif
#if (TTC_ASSERT_TASK == 1)  // use Assert()s in TASK code (somewhat slower but alot easier to debug)
  #define Assert_Task(Condition, ErrorCode) Assert(Condition, ErrorCode)
#else  // us no Assert()s in TASK code (somewhat smaller + faster, but crashes are hard to debug)
  #define Assert_Task(Condition, ErrorCode)
#endif

#ifndef TTC_TASK_MINIMAL_STACK_SIZE
  #define TTC_TASK_MINIMAL_STACK_SIZE 128
#endif

//}

//X typedef void* TaskHandle_t;
typedef struct ttc_task_info_s {
    const char* Name;
    TaskHandle_t Handle;                // handle used to identify this task
    u8_t Priority;                      // tasks of higher priority will block those of lower priority
    void (*Function)();                 // function being started as task
    void*   Argument;                   // argument passed to task function
    Base_t* StackTop;                   // address of last item push on stack
    Base_t* StackStart;                 // start address of stack
    Base_t* StackEnd;                   // end address of stack (only available if stack grows upwards)
    Base_t  StackUsed;                  // current amount of bytes used from stack
    Base_t  StackSize_Bytes;            // size of stack
    Base_t  StackFree_HighWatermark;    // calculated free bytes in stack (according to high-watermark)
    struct ttc_task_info_s* Next;       // points to next item in single linked list
} ttc_task_info_t;

typedef struct {
  u8_t AmountAll;                     // total amount of tasks
  u8_t AmountRunning;                 // total amount of running tasks
  u8_t AmountDelayed;                 // total amount of delayed tasks
  u8_t AmountOverflowDelayed;         // total amount of OverflowDelayed tasks
  u8_t AmountWaitTermination;         // total amount of tasks waiting for termination
  u8_t AmountSuspended;               // total amount of tasks being suspended
  ttc_task_info_t** Running;          // list of pointers to task-infos of running tasks
  ttc_task_info_t** Delayed;          // list of pointers to task-infos of delayed tasks
  ttc_task_info_t** OverflowDelayed;  // list of pointers to task-infos of OverflowDelayed tasks
  ttc_task_info_t** WaitTermination;  // list of task-infos of tasks waiting for termination
  ttc_task_info_t** Suspended;        // list of task-infos of tasks being suspended
  ttc_task_info_t* All[];             // list of task-infos of all tasks
} ttc_task_all_t;

#endif
