/** { ttc_timer.c *******************************************************

                           The ToolChain
                           
   High-Level interface for TIMER device.

   Implementation of high-level interface.

   Authors:
   
}*/
//{ Defines/ TypDefs *****************************************************
//} Defines

#include "ttc_timer.h"

//{ Global Variables *****************************************************

#if TTC_TIMER_AMOUNT == 0
  #warning No devices defined, check your makefile!
#endif

// for each initialized device, a pointer to its generic and stm32f1 definitions is stored
ttc_array_define(ttc_timer_config_t*, ttc_timer_configs, TTC_TIMER_AMOUNT);

// Set to TRUE on first call of ttc_timer_init()
BOOL Driver_Initialized = 0;

//} Global Variables
//{ Function definitions *************************************************

void ttc_timer_prepare() {
  // add your startup code here (Singletasking!)
}
u8_t ttc_timer_get_max_index() {
    return TTC_TIMER_AMOUNT;
}
ttc_timer_config_t* ttc_timer_get_configuration(u8_t LogicalIndex) {
    Assert_TIMER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_timer_config_t* Config = A(ttc_timer_configs, LogicalIndex-1); // will assert if outside array bounds

    if (!Config) {
        Config = A(ttc_timer_configs, LogicalIndex-1) = ttc_memory_alloc_zeroed(sizeof(ttc_timer_config_t));
        ttc_timer_reset(LogicalIndex);
    }

    return Config;
}
void ttc_timer_init(u8_t LogicalIndex) {
    Assert_TIMER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_timer_config_t* Config = ttc_timer_get_configuration(LogicalIndex);

    ttc_timer_errorcode_e Result = Driver_timer_init(Config);
    if (Result == ec_timer_OK)
      Config->Flags.Bits.Initialized = 1;
}
void ttc_timer_deinit(u8_t LogicalIndex) {
    Assert_TIMER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_timer_config_t* Config = ttc_timer_get_configuration(LogicalIndex);
  
    ttc_timer_errorcode_e Result = Driver_timer_deinit(Config);
    if (Result == ec_timer_OK)
      Config->Flags.Bits.Initialized = 0;
}
void ttc_timer_reset(u8_t LogicalIndex) {
    Assert_TIMER(LogicalIndex, ec_InvalidArgument); // logical index starts at 1
    ttc_timer_config_t* Config = ttc_timer_get_configuration(LogicalIndex);

    ttc_memory_set( Config, 0, sizeof(ttc_timer_config_t) );

    // load generic default values
    Config->LogicalIndex = LogicalIndex;

    Driver_timer_load_defaults(Config);    
}

// ToDo: additional functions go here...

//} Function definitions
