/** { ttc_timer.h *******************************************************

                           The ToolChain
                           
   High-Level interface for TIMER device.

   Structures, Enums and Defines being required by high-level timer and application.

   Authors:
   
}*/

#ifndef TTC_TIMER_H
#define TTC_TIMER_H

//{ Includes *************************************************************

#include "ttc_basic.h"
#include "ttc_memory.h"

#ifdef EXTENSION_450_timer_stm32f1
#  include "stm32f1/stm32f1_timer.h"
#endif

#include "ttc_timer_types.h"

//} Includes
//{ Check for missing low-level implementations **************************

// Note: functions declared below must be redefined to implementations provided by low-level Driver!

/** Prepares timer Driver for basic operation.
 *
 * Note: This function is called automatically at system start before the scheduler has been started.
 */
void ttc_timer_prepare();

/** loads configuration of indexed TIMER interface with default values
 * @param Config        pointer to struct ttc_timer_config_t (must have valid value for LogicalIndex)
 * @return  == 0:       configuration was loaded successfully
 */
ttc_timer_errorcode_e Driver_timer_load_defaults(ttc_timer_config_t* Config);
#ifndef Driver_timer_load_defaults
#  error: missing low-level implementation!
#endif

/** shutdown single TIMER device
 * @param Config        pointer to struct ttc_timer_config_t (must have valid value for LogicalIndex)
 * @return              == 0: TIMER has been shutdown successfully; != 0: error-code
 */
ttc_timer_errorcode_e Driver_timer_deinit(ttc_timer_config_t* Config);
#ifndef Driver_timer_deinit
#  error: missing low-level implementation!
#endif

/** initializes single TIMER
 * @param Config        pointer to struct ttc_timer_config_t (must have valid value for LogicalIndex)
 * @return              == 0: TIMER has been initialized successfully; != 0: error-code
 */
ttc_timer_errorcode_e Driver_timer_init(ttc_timer_config_t* Config);
#ifndef Driver_timer_init
#  error: missing low-level implementation!
#endif

//}
//{ Defines/ TypeDefs ****************************************************

//} Defines
//{ Structures/ Enums ****************************************************
// Structures and Enums required by High-Level timer only go here...

//} Structures/ Enums
//{ Function prototypes **************************************************

/** returns amount of devices available in current configuration
 *
 * @return amount of available devices
 */
u8_t ttc_timer_get_max_index();

/** returns configuration of indexed device (asserts if no valid configuration was found)
 *
 * @param LogicalIndex    index of device to init (1..ttc_TIMER_get_max_LogicalIndex())
 * @return                configuration of indexed device
 */
ttc_timer_config_t* ttc_timer_get_configuration(u8_t LogicalIndex);

/** initialize indexed device for use
 *
 * @param LogicalIndex    device index of device to init (1..ttc_TIMER_get_max_LogicalIndex())
 */
void ttc_timer_init(u8_t LogicalIndex);

/** shutdown indexed device
 *
 * @param LogicalIndex    device index of device to shutdown (1..ttc_TIMER_get_max_LogicalIndex())
 */
void ttc_timer_deinit(u8_t LogicalIndex);

/** reset configuration of indexed device back to default
 *
 * @param LogicalIndex    device index of device to init (1..ttc_TIMER_get_max_LogicalIndex())
 */
void ttc_timer_reset(u8_t LogicalIndex);

// additional prototypes go here...

//} Function prototypes
//{ private Function prototypes ******************************************

/** 
 *
 * Note: This function is marked private and should be called from outside!
 *
 * @param 
 * @return
 */
void _ttc_timer_();


//}private functions
//{ Macros ***************************************************************

//}Macros

#endif //TTC_TIMER_H
