/** { ttc_timer_types.h ************************************************

                           The ToolChain

   High-Level interface for TIMER device.
   
   Structures, Enums and Defines being required by both, high- and low-level timer.
   

   Authors:
   
}*/

#ifndef TTC_TIMER_TYPES_H
#define TTC_TIMER_TYPES_H

#include "ttc_basic.h"
#include "stm32f1/stm32f1_timer_types.h"

//{ Defines ***************************************************


//} Defines
//{ Static Configuration **************************************

// TTC_TIMERn has to be defined as constant by makefile.100_board_*
#ifdef TTC_TIMER5
  #ifndef TTC_TIMER4
    #error TTC_TIMER5 is defined, but not TTC_TIMER4 - all lower TTC_TIMERn must be defined!
  #endif
  #ifndef TTC_TIMER3
    #error TTC_TIMER5 is defined, but not TTC_TIMER3 - all lower TTC_TIMERn must be defined!
  #endif
  #ifndef TTC_TIMER2
    #error TTC_TIMER5 is defined, but not TTC_TIMER2 - all lower TTC_TIMERn must be defined!
  #endif
  #ifndef TTC_TIMER1
    #error TTC_TIMER5 is defined, but not TTC_TIMER1 - all lower TTC_TIMERn must be defined!
  #endif

  #define TTC_TIMER_AMOUNT 5
#else
  #ifdef TTC_TIMER4
    #define TTC_TIMER_AMOUNT 4

    #ifndef TTC_TIMER3
      #error TTC_TIMER5 is defined, but not TTC_TIMER3 - all lower TTC_TIMERn must be defined!
    #endif
    #ifndef TTC_TIMER2
      #error TTC_TIMER5 is defined, but not TTC_TIMER2 - all lower TTC_TIMERn must be defined!
    #endif
    #ifndef TTC_TIMER1
      #error TTC_TIMER5 is defined, but not TTC_TIMER1 - all lower TTC_TIMERn must be defined!
    #endif
  #else
    #ifdef TTC_TIMER3

      #ifndef TTC_TIMER2
        #error TTC_TIMER5 is defined, but not TTC_TIMER2 - all lower TTC_TIMERn must be defined!
      #endif
      #ifndef TTC_TIMER1
        #error TTC_TIMER5 is defined, but not TTC_TIMER1 - all lower TTC_TIMERn must be defined!
      #endif

      #define TTC_TIMER_AMOUNT 3
    #else
      #ifdef TTC_TIMER2

        #ifndef TTC_TIMER1
          #error TTC_TIMER5 is defined, but not TTC_TIMER1 - all lower TTC_TIMERn must be defined!
        #endif

        #define TTC_TIMER_AMOUNT 2
      #else
        #ifdef TTC_TIMER1
          #define TTC_TIMER_AMOUNT 1
        #else
          #define TTC_TIMER_AMOUNT 0
        #endif
      #endif
    #endif
  #endif
#endif

#ifndef TTC_ASSERT_TIMER    // any previous definition set (Makefile)?
#define TTC_ASSERT_TIMER 1  // string asserts are enabled by default
#endif
#if (TTC_ASSERT_TIMER == 1)  // use Assert()s in TIMER code (somewhat slower but alot easier to debug)
  #define Assert_TIMER(Condition, ErrorCode) Assert(Condition, ErrorCode)
#else  // us no Assert()s in TIMER code (somewhat smaller + faster, but crashes are hard to debug)
  #define Assert_TIMER(Condition, ErrorCode)
#endif

//}Static Configuration
//{ Enums/ Structures *****************************************

typedef enum {    // return codes of TIMER devices
  ec_timer_OK = 0,
  
  // other warnings go here..

  ec_timer_ERROR,           // general failure
  ec_timer_DeviceNotFound,
  ec_timer_InvalidImplementation
  
  // other failures go here..
} ttc_timer_errorcode_e;
typedef struct ttc_timer_config_s { // stm32f1 independent configuration data

    // Note: Write-access to this structure is only allowed before first ttc_timer_init() call!
    //       After initialization, only read accesses are allowed for high-level code.

                 u8_t  LogicalIndex; // automatically set: logical index of device to use (1 = TTC_TIMER1, ...)
    ttc_timer_arch_t* Architecture; // low-level configuration
    
    union  { // generic configuration bits common for all low-level Drivers
        u16_t All;
        struct {
          unsigned Initialized        : 1;  // ==1: device has been initialized successfully
          // ToDo: additional high-level flags go here..
            
            unsigned Reserved1        : 16; // pad to 16 bits
        } Bits;
    } Flags;

    // Additional high-level attributes go here..
    
} __attribute__((__packed__)) ttc_timer_config_t;

//}Structures

#endif // TTC_TIMER_TYPES_H
