/** regression_list.c ************************************************{
 
  Regression test for ttc_list.c/.h

  written by Gregor Rebel 2013
}*/

#include "regression_list.h"


//{ Function definitions *************************************************
//{ RMA-Implementation ****************************************************

// structures

//} RMA-Implementation
//{ Global Variables *****************************************************

typedef struct {
    // must be first entry
    ttc_list_item_t Item;

    // value stored in this item
    u32_t Value;
} ValueItem_t;

/*
#undef TTC_LIST_PIN_PUSH_BACK_SINGLE
#define TTC_LIST_PIN_PUSH_BACK_SINGLE PIN_PE1
#undef TTC_LIST_PIN_PULL_FRONT_SINGLE
#define TTC_LIST_PIN_PULL_FRONT_SINGLE PIN_PE3
*/

#define AMOUNT_VALUE_ITEMS 100
ValueItem_t ValueItems[AMOUNT_VALUE_ITEMS];
ValueItem_t* NextValueItem = ValueItems;

Base_t ElapsedTime1 = 0;
Base_t ElapsedTime2 = 0;

// transports items taskProducer() -> taskConsumer()
ttc_list_t ListProducer2Consumer;

// transports items taskProducer() <- taskConsumer()
ttc_list_t ListConsumer2Producer;

//} Global Variables

void list_push_back_single(ttc_list_t* List, ttc_list_item_t* Item) {

    Base_t Time = ttc_task_get_tick_count() / 100000;
    if (ElapsedTime1 != Time) { // another second: signal activity
        ElapsedTime1 = Time;
#ifdef TTC_LIST_PIN_PUSH_BACK_SINGLE
        if (ElapsedTime1 & 0x1)
            ttc_gpio_set(TTC_LIST_PIN_PUSH_BACK_SINGLE);
        else
            ttc_gpio_clr(TTC_LIST_PIN_PUSH_BACK_SINGLE);
#endif
    }
    ttc_list_push_back_single(List, Item);
}
ttc_list_item_t* list_pop_front_single(ttc_list_t* List) {

    Base_t Time = ttc_task_get_tick_count() / 100000;
    if (ElapsedTime2 != Time) { // another second: signal activity
        ElapsedTime2 = Time;
#ifdef TTC_LIST_PIN_PULL_FRONT_SINGLE
        if (ElapsedTime2 & 0x1)
            ttc_gpio_set(TTC_LIST_PIN_PULL_FRONT_SINGLE);
        else
            ttc_gpio_clr(TTC_LIST_PIN_PULL_FRONT_SINGLE);
#endif
    }

    return ttc_list_pop_front_single(List);
}

void regression_list_start() {

#ifdef TTC_LIST_PIN_PUSH_BACK_SINGLE
    ttc_gpio_init(TTC_LIST_PIN_PUSH_BACK_SINGLE,  tgm_output_push_pull);
#endif
#ifdef TTC_LIST_PIN_PULL_FRONT_SINGLE
    ttc_gpio_init(TTC_LIST_PIN_PULL_FRONT_SINGLE, tgm_output_push_pull);
#endif

    // TestRMA
    ttc_list_init(&ListProducer2Consumer, "ListProducer2Consumer");
    ttc_list_init(&ListConsumer2Producer, "ListConsumer2Producer");

    ValueItem_t* ValueItem = ValueItems;
    for (u8_t Count = 0; Count < AMOUNT_VALUE_ITEMS; Count++) {
        ValueItem->Value = Count;
        list_push_back_single(&ListProducer2Consumer, &(ValueItem->Item));
        ValueItem++;
    }

    Base_t SleepMax = 88;
    for (u8_t Amount = 8; Amount > 0; Amount--)
        ttc_task_create(taskProducer, "taskProducer", 128, (void*) SleepMax, 0, NULL);

    SleepMax = 20;
    for (u8_t Amount = 8; Amount > 0; Amount--)
        ttc_task_create(taskConsumer, "taskConsumer", 128, (void*) SleepMax, 0, NULL);
}
void taskProducer(void* Arg) {
    // Base_t MaxSleepTime = (Base_t) Arg;
    // Base_t SleepTotal = 0;
    // Base_t TimeSecs = ttc_task_get_tick_count() / 1000000;
    Base_t ItemsProcessed = 1;

    while (1) {
        ValueItem_t* ValueItem = (ValueItem_t*) list_pop_front_single(&ListConsumer2Producer);
        if (ValueItem) {
            ItemsProcessed++;
            list_push_back_single(&ListProducer2Consumer, &(ValueItem->Item));
        }
        if (ItemsProcessed == 10000)
            ttc_testpoint();
    }
}
void taskConsumer(void* Arg) {
    // Base_t MaxSleepTime = (Base_t) Arg;
    // Base_t SleepTotal = 0;
    // Base_t TimeSecs = ttc_task_get_tick_count() / 1000000;
    Base_t ItemsProcessed = 0;

    while (1) {
        ValueItem_t* ValueItem = (ValueItem_t*) list_pop_front_single(&ListProducer2Consumer);
        if (ValueItem) {
            ItemsProcessed++;
            list_push_back_single(&ListConsumer2Producer, &(ValueItem->Item));
        }
        if (ItemsProcessed == 10000)
            ttc_testpoint();
    }
}


//} Function definitions
