/*{ regression_queue.c ************************************************
 
  regression_queue_start() spawns thread taskBrightness()

  taskBrightness()
      Defines brightness of led as Ton / (Ton + Toff)
      spawns thread taskLEDs()
      while (1) {
        while (Ton < 100) { Ton++; Toff--; }
        while (Ton >   0) { Ton--; Toff++; }
      }

  taskLEDs()
      while (1) {
        switch on LED
        wait for Ton
        switch off LED
        wait for Toff
      }

}*/

#include "regression_queue.h"

#define REGRESSION_PIN_QUEUE_PUSH     PIN_PE0
#define REGRESSION_PIN_QUEUE_PULL     PIN_PE5
#define REGRESSION_PIN_QUEUE_PUSH_ISR PIN_PC6
#define REGRESSION_PIN_QUEUE_PULL_ISR PIN_PC13

// read-only configuration of current display (width, height, ...)
ttc_gfx_generic_t* DisplayConfig = NULL;

//{ Function definitions *************************************************
//{ FCTMA-Implementation ****************************************************

#define random(MAX)      ttc_rand() % MAX

// structures

// global variables

// amount of blocks allocated on heap
extern Base_t ttc_memory_Amount_Blocks_Allocated;

// amount of bytes occupied on heap by dynamic allocations
extern Base_t ttc_memory_Bytes_Allocated_Total;

// usable amount of allocated bytes
extern Base_t ttc_memory_Bytes_Allocated_Usable;

//} FCTMA-Implementation
//{ Global Variables *****************************************************

void taskProducer(void* Arg);
void taskConsumer(void* Arg);
u8_t DisplayY = 2;

// size of each queue
#define RQ_QUEUE_SIZE 200

ttc_queue_bytes_t Queue_Producer2Consumer;
u8_t Queue_Producer2Consumer_Data[RQ_QUEUE_SIZE];

Base_t AmountHeapFreeAtStart = 0;

Base_t P_AmountSent      = 0;
Base_t P_SleepTime_Total = 0;
Base_t C_BlocksReceived  = 0;
Base_t C_SleepTime_Total = 0;

// amount of producer tasks to spawn
#ifndef Amount_Producer
#define Amount_Producer 40
#endif

// amount of consumer tasks to spawn
#ifndef Amount_Consumer
#define Amount_Consumer 40
#endif

// amount of blocks to allocate in each turn
#define Amount_BytesPerTurn  1

#define Amount_Blocks_Total Amount_Producer * Amount_BytesPerTurn * 4

// each producer will halt after producing this amount of blocks
#define Amount_BlocksProduced_Per_Producer 100000

#ifndef ENABLED_GFX
#define ENABLED_GFX     0  // ==1: gfx display enabled (inaccurate timings due to display time)
#endif
#ifndef ENABLED_ENDLESS
#define ENABLED_ENDLESS 1  // ==1: run in endless loop/ ==0: use Amount_BlocksProduced_Per_Producer as limit
#endif

//} Global Variables

TaskStatistics_t  ProducerTasks[Amount_Producer + 2];
TaskStatistics_t* EndOfProducerTasks = ProducerTasks;
TaskStatistics_t  ConsumerTasks[Amount_Consumer + 2];
TaskStatistics_t* EndOfConsumerTasks = ConsumerTasks;

// statistics for producer and consumer interrupt service routines
TaskStatistics_t* ISR_P_Stats = ProducerTasks + Amount_Producer + 1;
TaskStatistics_t* ISR_C_Stats = ConsumerTasks + Amount_Producer + 1;

// we reserve 1 index for ISR_P()
u8_t MaxSenderIndex = Amount_Producer + 1;

const char* Text_TestQueue = "regress: Queue";

void regression_queue_start() {
    u8_t MaxDisplay = ttc_gfx_get_max_display_index();
    Assert(MaxDisplay > 0, ec_DeviceNotFound); // no displays defined by board makefile!
    DisplayConfig = ttc_gfx_get_configuration(1);
    Assert(DisplayConfig, ec_DeviceNotFound);
    // set color to use for first screen clear
    ttc_gfx_set_color_bg24(GFX_COLOR24_AQUA);
    ttc_gfx_init(1);

    // initialize gfx for use with printf()
    ttc_gfx_stdout_set(1);
    ttc_string_register_stdout(ttc_gfx_stdout_print_block, ttc_gfx_stdout_print_string);

    // allocate memories
    ttc_queue_byte_init(&Queue_Producer2Consumer, Queue_Producer2Consumer_Data, RQ_QUEUE_SIZE);

    static u8_t Temp3[20];
    Base_t SleepMax;
    SleepMax = 10;
    for (u8_t Index = 0; Index < Amount_Consumer; Index++) {
        snprintf(Temp3, 20, "C%02i", Index);
        EndOfConsumerTasks->SleepTime_Max = SleepMax;
        EndOfConsumerTasks->Index = Index + 101;
        EndOfConsumerTasks->AmountReceived = ttc_memory_alloc_zeroed(sizeof(Base_t) * (Amount_Producer+1));
        ttc_task_create(taskConsumer, (char*) Temp3, 90, (void*) EndOfConsumerTasks++, 0, NULL);
        ttc_task_check_stack();
    }

    SleepMax = 15;
    for (u8_t Index = 0; Index < Amount_Producer; Index++) {
        snprintf(Temp3, 20, "P%02i", Index);
        EndOfProducerTasks->SleepTime_Max = SleepMax;
        EndOfProducerTasks->Index = Index + 1;
        ttc_task_create(taskProducer, (char*) Temp3, 90, (void*) EndOfProducerTasks++, 0, NULL);
        ttc_task_check_stack();
    }

    ttc_task_create(taskRegression_Queue, "Queue", 200, NULL, 1, NULL);


#ifdef REGRESSION_PIN_QUEUE_PUSH
    ttc_gpio_init(REGRESSION_PIN_QUEUE_PUSH,     tgm_output_push_pull);
#endif
#ifdef REGRESSION_PIN_QUEUE_PULL
    ttc_gpio_init(REGRESSION_PIN_QUEUE_PULL,     tgm_output_push_pull);
#endif
#ifdef REGRESSION_PIN_QUEUE_PUSH_ISR
    ttc_gpio_init(REGRESSION_PIN_QUEUE_PUSH_ISR, tgm_output_push_pull);
#endif
#ifdef REGRESSION_PIN_QUEUE_PULL_ISR
    ttc_gpio_init(REGRESSION_PIN_QUEUE_PULL_ISR, tgm_output_push_pull);
#endif

}
void taskRegression_Queue(void* Arg) {
    (void) Arg;
    AmountHeapFreeAtStart = ttc_memory_get_free_size() + sizeof(Queue_Producer2Consumer) + sizeof(Queue_Producer2Consumer_Data[RQ_QUEUE_SIZE]);

    printAt(0,0, Text_TestQueue, 0);
    printAt(2, DisplayConfig->TextRows - 1, "used ", 0);
    printAt(0, DisplayConfig->TextRows - 2, "free: %iB", ttc_memory_get_free_size());

    TaskStatistics_t* TaskInfo = NULL;
    Base_t SleepTime_Average;

    while (1) {
        ttc_task_check_stack();
        u8_t Row = 1;

        P_AmountSent   = 0;
        P_SleepTime_Total   = 0;
        TaskInfo = ProducerTasks;
        while (TaskInfo < EndOfProducerTasks) { // collect statistic data of all producer tasks
            P_AmountSent += TaskInfo->AmountSent;
            P_SleepTime_Total += TaskInfo->SleepTime_Total;
            TaskInfo++;
        }
        if (!P_AmountSent) P_AmountSent = 1; // avoid divide by zero

        C_BlocksReceived   = 0;
        C_SleepTime_Total  = 0;
        TaskInfo = ConsumerTasks;
        while (TaskInfo < EndOfConsumerTasks) { // collect statistic data of all consumer tasks
            for (u8_t I = 1; I <= Amount_Producer; I++)
                C_BlocksReceived += TaskInfo->AmountReceived[I];
            C_SleepTime_Total += TaskInfo->SleepTime_Total;
            TaskInfo++;
        }
        if (!C_BlocksReceived) C_BlocksReceived = 1; // avoid divide by zero

        printAt(0, Row, "P(%2i)", Amount_Producer);
        printAt(5, Row, "#%09i",  P_AmountSent);
        Row++;
        SleepTime_Average = DIV(P_SleepTime_Total, P_AmountSent);
        printAt( 0, Row++, "SleepAvg %3i ", SleepTime_Average);
        printAt(0, Row, "C(%2i)", Amount_Consumer);
        printAt(5, Row, "#%09i",  C_BlocksReceived);
        Row++;
        SleepTime_Average = DIV(C_SleepTime_Total, C_BlocksReceived);
        printAt( 0, Row++, "SleepAvg %3i ", SleepTime_Average);

        if (C_BlocksReceived == Amount_BlocksProduced_Per_Producer * Amount_Producer) {
            Assert(P_AmountSent == C_BlocksReceived, ec_InvalidImplementation);
            printAt(0, DisplayConfig->TextRows - 3, "Successfull!", 0);
            ttc_testpoint();
        }
        Assert(C_BlocksReceived <= Amount_BlocksProduced_Per_Producer * Amount_Producer, ec_InvalidImplementation); // too many blocks received!

        if (1) printAt(8, DisplayConfig->TextRows - 1, "%iB", AmountHeapFreeAtStart - ttc_memory_get_free_size());
        static u8_t Count = 0;
        if (Count++ & 1)
            printAt(0, DisplayConfig->TextRows - 1, "+", 0);
        else
            printAt(0, DisplayConfig->TextRows - 1, "-", 0);
        mSleep(1000);
        while (!ENABLED_GFX) { // GFX output disabled: no further update
            mSleep(1000);
        }
    }
}
void taskProducer(void* Arg) {
    TaskStatistics_t* MyTaskInfo = (TaskStatistics_t*) Arg;
    Assert(ttc_memory_is_writable(MyTaskInfo), ec_InvalidImplementation);

    while (1) {
        ttc_task_check_stack();

        if (ENABLED_ENDLESS || (MyTaskInfo->AmountSent < Amount_BlocksProduced_Per_Producer) ) {
            Base_t Blocks2Send = Amount_BytesPerTurn;
            while (Blocks2Send) {
                ttc_task_check_stack();
#ifdef REGRESSION_PIN_QUEUE_PUSH
                if (MyTaskInfo->Index == 1) { // we only measure timing in one producer task
                    ttc_gpio_set(REGRESSION_PIN_QUEUE_PUSH);
                }
#endif
                ttc_queue_byte_push_back(&Queue_Producer2Consumer, MyTaskInfo->Index);

#ifdef REGRESSION_PIN_QUEUE_PUSH
                if (MyTaskInfo->Index == 1) { // we only measure timing in one producer task
                    ttc_gpio_clr(REGRESSION_PIN_QUEUE_PUSH);
                }
#endif
                MyTaskInfo->AmountSent++;
                Blocks2Send--;
            }
            if (MyTaskInfo->AmountSent)
                MyTaskInfo->SleepTime_Average = DIV(MyTaskInfo->SleepTime_Total, MyTaskInfo->AmountSent / Amount_BytesPerTurn);

            // simulate random time required to fill Data[]
            if (0) {
                Base_t SleepTime = 1 + random(MyTaskInfo->SleepTime_Max);
                MyTaskInfo->SleepTime_Total += SleepTime;
                mSleep(SleepTime);
            }
            else   ttc_task_yield();
        }
        else
            mSleep(1000); // finished producing blocks
    }
}
void taskConsumer(void* Arg) {
    TaskStatistics_t* MyTaskInfo = (TaskStatistics_t*) Arg;
    Assert(ttc_memory_is_writable(MyTaskInfo), ec_InvalidImplementation);
    Base_t AmountReceived = 0;

    while (1) {
        ttc_task_check_stack();

        if (0) { // simulate random processing time of Data[]
            Base_t SleepTime = 1 + random(MyTaskInfo->SleepTime_Max);
            MyTaskInfo->SleepTime_Total += SleepTime;
            mSleep(SleepTime);
        }
        else ttc_task_yield();

        for (u8_t I = Amount_BytesPerTurn; I > 0;) {
            ttc_task_check_stack();
            u8_t SenderTaskIndex = 0;
#ifdef REGRESSION_PIN_QUEUE_PULL
            if (MyTaskInfo->Index == 101) { // we only measure timing in one consumer task
                ttc_gpio_set(REGRESSION_PIN_QUEUE_PULL);
            }
#endif
            if (ttc_queue_byte_pull_front(&Queue_Producer2Consumer, &SenderTaskIndex) == tqe_OK) {
#ifdef REGRESSION_PIN_QUEUE_PULL
                if (MyTaskInfo->Index == 101) { // we only measure timing in one consumer task
                    ttc_gpio_clr(REGRESSION_PIN_QUEUE_PULL);
                }
#endif
                check_ReceivedByte(SenderTaskIndex, MyTaskInfo);
                I--;
                AmountReceived++;
            }
            else
                ttc_task_yield();
        }
        if (AmountReceived)
            MyTaskInfo->SleepTime_Average = DIV(MyTaskInfo->SleepTime_Total, AmountReceived / Amount_BytesPerTurn);
    }
}
void check_ReceivedByte(u8_t SenderTaskIndex, TaskStatistics_t* MyTaskInfo) {
    Assert( (SenderTaskIndex > 0) && (SenderTaskIndex <= MaxSenderIndex), ec_InvalidImplementation); // invalid producer id received!
    TaskStatistics_t* SenderTask = &(ProducerTasks[SenderTaskIndex-1]);
    if (SenderTask) {
        Assert(ttc_memory_is_writable(SenderTask), ec_InvalidImplementation);
        Assert(SenderTask->Index == SenderTaskIndex, ec_InvalidImplementation);
        MyTaskInfo->AmountReceived[SenderTask->Index]++;
    }
}
void ISR_P() {
    ttc_task_begin_criticalsection(ISR_P);

#ifdef REGRESSION_PIN_QUEUE_PUSH_ISR
    ttc_gpio_set(REGRESSION_PIN_QUEUE_PUSH_ISR);
#endif
    if (ttc_queue_byte_push_back_isr(&Queue_Producer2Consumer, MaxSenderIndex) == tqe_OK) {
#ifdef REGRESSION_PIN_QUEUE_PUSH_ISR
        ttc_gpio_clr(REGRESSION_PIN_QUEUE_PUSH_ISR);
#endif
        ISR_P_Stats->AmountSent++;
    }

    ttc_task_end_criticalsection();
}
void ISR_C() {
    ttc_task_begin_criticalsection(ISR_C);

    u8_t SenderTaskIndex = 0;
#ifdef REGRESSION_PIN_QUEUE_PULL_ISR
    ttc_gpio_set(REGRESSION_PIN_QUEUE_PULL_ISR);
#endif
    if (ttc_queue_byte_pull_front_isr(&Queue_Producer2Consumer, &SenderTaskIndex) == tqe_OK) {
#ifdef REGRESSION_PIN_QUEUE_PULL_ISR
        ttc_gpio_clr(REGRESSION_PIN_QUEUE_PULL_ISR);
#endif
        check_ReceivedByte(SenderTaskIndex, ISR_C_Stats);
    }

    ttc_task_end_criticalsection();
}
void printAt(u8_t X, u8_t Y, const char* Format, u32_t Value) {
    static TaskHandle_t MyTask = NULL;

    if (MyTask) { // check if called from invalid task
        Assert(MyTask == ttc_task_current_handle(), ec_InvalidImplementation); // this function may only be used from one task
    }
    else {        // called first time: store owner task
        MyTask = ttc_task_current_handle();
        Assert(ttc_memory_is_writable(MyTask), ec_InvalidImplementation);
    }

    static u8_t Temp1[100]; // static because there can be only one taskPrint() instance
    ttc_gfx_text_cursor_set(X, Y);
    ttc_string_snprintf(Temp1, 100, Format, Value);
    ttc_gfx_print_solid( (char*) Temp1, 100);
}

//} Function definitions
