/*{ regression_queue.h ************************************************
 
  Regression test for simple queues
  written by Gregor Rebel 2013
 
}*/
#ifndef regression_queue_H
#define regression_queue_H

//{ Includes *************************************************************

#include "ttc_basic.h"
#include "ttc_task.h"
#include "ttc_string.h"
#include "ttc_memory.h"
#include "ttc_gfx.h"
#include "ttc_random.h"
#include "ttc_queue.h"
#include "math_basic.h"
//? #include "ttc_list.h"

//} Includes
//{ Defines/ TypeDefs ****************************************************

#ifndef EXTENSION_500_ttc_gfx
  #error Missing EXTENSION_500_ttc_gfx !
#endif
#ifdef EXTENSION_500_ttc_memory_heap_sdma
#include "heap_sdma.h"
#endif

//} Defines

typedef struct {
    // each task has a different index number
    u8_t Index;

    // read by task
    Base_t SleepTime_Max;

    // written by task
    Base_t  AmountSent;         // total amount of memory blocks sent to consumer from this task
    Base_t* AmountReceived;     // != NULL: AmountReceived[I] = amount of bytes of value I Received

    Base_t SleepTime_Total;    // total amount of sleeps of this task so far
    Base_t SleepTime_Average;  // = SleepTime_Total / BlocksProcessed
} TaskStatistics_t;

//{ Function prototypes **************************************************

// enqueues print job
void printAt(u8_t X, u8_t Y, const char* Format, u32_t Value);

// main entry: initializes hardware + spawns all threads
// returns immediately
void regression_queue_start();

// main task running regression test
void taskRegression_Queue(void* Arg);

// checks received byte + updates given task info data
void check_ReceivedByte(u8_t SenderTaskIndex, TaskStatistics_t* MyTaskInfo);

//} Function prototypes
#endif //regression_queue_H
