/*{ Template_FreeRTOS::.c ************************************************

     Several different boxes run over the screen.
     Each box is controlled by one task.
     
     written 2012 by Ioanna Tsekoura, Gregor Rebel
     
}*/

#include "example_gfx_boxes.h"

//{ Function definitions *************************************************
const char* TaskNames[10] = { "tMoveBox1", "tMoveBox2", "tMoveBox3", "tMoveBox4", "tMoveBox5", "tMoveBox6", "tMoveBox7", "tMoveBox8", "tMoveBox9", "tMoveBox10" };
const char** LastTaskName = NULL;
ttc_gfx_generic_t* DisplayConfig = NULL;
u32_t ColorBg = GFX_COLOR24_WHITE;

// size of area in which to draw boxes (start at (0,0))
u32_t DrawAreaWidth  = 0;
u32_t DrawAreaHeight = 0;
u32_t TimeBase_usecs = 20000; // box speed is pixel/TimeBase_usecs
const u16_t MathMultiplier = 1; // coordinates, speed are multiplied by this factor for increased mathematical accuracy

#define NumberofBoxes 6
DrawData_t *AllPolygons[NumberofBoxes];
DrawData_t **LastPolygon = AllPolygons;

void example_gfx_boxes_start() {

    LastTaskName = TaskNames;

    u8_t MaxDisplay = ttc_gfx_get_max_display_index();
    Assert(MaxDisplay > 0, ec_DeviceNotFound); // no displays defined by board makefile!
    DisplayConfig = ttc_gfx_get_configuration(1);
    Assert(DisplayConfig, ec_DeviceNotFound);

    // set color to use for first screen clear
    ttc_gfx_set_color_bg24(ColorBg);

    ttc_gfx_init(1);

    // allow text display only in last line of display
    DisplayConfig->TextMinY = DisplayConfig->Height - DisplayConfig->Font->CharHeight;

    DrawAreaWidth  = DisplayConfig->Width    * MathMultiplier;
    DrawAreaHeight = DisplayConfig->TextMinY * MathMultiplier;

    // print text in last line
    ttc_gfx_text_cursor_set(0,0);
    ttc_gfx_print("Boxes/s:", -1);

    xQueueHandle DrawQueue = xQueueCreate(10, sizeof(DrawData_t));
    Assert(NULL != DrawQueue, ec_Malloc);

    ttc_srand(32);

    for (s16_t I = 1; I <= NumberofBoxes; I++) {
        DrawData_t* Box = newRandomBox(DrawQueue);
        Assert(Box != NULL, ec_Malloc);
    }

    ttc_task_create(tDrawIt,                       // function to start as thread
                    "tDrawIt",                             // thread name (just for debugging)
                    1 * TTC_TASK_MINIMAL_STACK_SIZE,          // stack size
                    (void *) DrawQueue,                    // passed as argument to taskLEDs()
                    1,                                     // task priority (higher values mean more process time)
                    (void*) NULL                           // can return a handle to created task
                    );

}
DrawData_t* newRandomBox(xQueueHandle DrawingQueue) {
    ttc_gfx_set_color_fg24( ((ttc_rand() & 0xff) << 16) | ((ttc_rand() & 0xff) << 8) | (ttc_rand() & 0xff) );

    return newBox( DrawingQueue,
                   DisplayConfig->ColorFg,
                   0 + ttc_rand() % (DrawAreaWidth  / MathMultiplier),
                   0 + ttc_rand() % (DrawAreaHeight / MathMultiplier),
                   3 + ttc_rand() % 20,
                   3 + ttc_rand() % 20,
                   (ttc_rand() % 20) - 10,
                   (ttc_rand() % 20) - 10
                 );
}
DrawData_t* newBox(xQueueHandle DrawingQueue, u16_t Color, s32_t X, s32_t Y, u16_t Width, u16_t Height, s8_t SpeedX, s8_t SpeedY) {
    drawTaskArgs_t* NewBox = (drawTaskArgs_t*) ttc_memory_alloc( sizeof(drawTaskArgs_t) );

    if (NewBox != NULL) {
        NewBox->DrawQueue = DrawingQueue;
        NewBox->Box2Draw.Type = ddt_MovingBox;
        NewBox->Box2Draw.Polygon.MovingBox.Color  = Color;
        NewBox->Box2Draw.Polygon.MovingBox.PosX   = X      * MathMultiplier;
        NewBox->Box2Draw.Polygon.MovingBox.PosY   = Y      * MathMultiplier;
        NewBox->Box2Draw.Polygon.MovingBox.Width  = Width  * MathMultiplier;
        NewBox->Box2Draw.Polygon.MovingBox.Height = Height * MathMultiplier;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedX = SpeedX * MathMultiplier;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedY = SpeedY * MathMultiplier;

        if ( ttc_task_create(tMoveBox,                      // function to start as thread
                        (const char*) *LastTaskName++,      // thread name (just for debugging)
                        (size_t) 2 * TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                        (void *) NewBox,                       // passed as argument to task-function
                        (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                        (xTaskHandle*) NULL                // can return a handle to created task
                        )
             ) {
            *LastPolygon++ = &(NewBox->Box2Draw);
            return &(NewBox->Box2Draw);
        }
    }

    // Error: could not create task (perhaps out of memory)
    return NULL;
}

/** calculate effect of collision of two moving boxes on their SpeedX,SpeedY
 *
 * @param Box1  first box colliding
 * @param Box2  second box colliding
 */
void collide(elb_MovingBox_t* Box1, elb_MovingBox_t* Box2) {
    // reference: http://de.wikipedia.org/wiki/Sto%C3%9F_(Physik)


    if (0) {

        // calculate line slope Moving-Target (using top-left corner instead of center points for simplicity)
        s16_t CentralLineX = Box2->PosX - Box1->PosX;
        s16_t CentralLineY = Box2->PosY - Box1->PosY;

        // calculate line being tangent to CentralLine
        //X s16_t CentralTangentX = - CentralLineY;
        //X s16_t CentralTangentY = CentralLineX;

        double dX = 1.0 * CentralLineX;
        double dY = 1.0 * CentralLineY;
        double CentralLineAngle, CentralLineLength;
        math_cartesian2polar(dX, dY, &CentralLineAngle, &CentralLineLength);

        dX = 1.0 * Box1->SpeedX;
        dY = 1.0 * Box1->SpeedY;
        double SpeedAngle, SpeedLength;
        math_cartesian2polar(dX, dY, &SpeedAngle, &SpeedLength);

        // rotate speedangle
        SpeedAngle -= CentralLineAngle;

        double Speed2X, Speed2Y;
        math_polar2cartesian(SpeedAngle, SpeedLength, &Speed2X, &Speed2Y);

        double ImpulseOnTarget  = Speed2X / Speed2Y; // ==0.0: no impact on Target; ==1.0: full impact on target
        double ImpulseRemaining = 1.0 - ImpulseOnTarget;
        (void) ImpulseRemaining;
    }

    s32_t Mass1 = Box1->Width * Box1->Height;
    s32_t Mass2 = Box2->Width * Box2->Height;
    s32_t CommonX = (2 * (Mass1 * Box1->SpeedX + Mass2 * Box2->SpeedX)) / (Mass1 + Mass2);
    s32_t CommonY = (2 * (Mass1 * Box1->SpeedY + Mass2 * Box2->SpeedY)) / (Mass1 + Mass2);

    if (0) {
        Assert(CommonX != Box1->SpeedX, ec_UNKNOWN); // resulting speed == 0
        Assert(CommonY != Box1->SpeedY, ec_UNKNOWN); // resulting speed == 0
        Assert(CommonX != Box2->SpeedX, ec_UNKNOWN); // resulting speed == 0
        Assert(CommonY != Box2->SpeedY, ec_UNKNOWN); // resulting speed == 0
    }

    Box1->SpeedX = CommonX - Box1->SpeedX;
    Box1->SpeedY = CommonY - Box1->SpeedY;
    Box2->SpeedX = CommonX - Box2->SpeedX;
    Box2->SpeedY = CommonY - Box2->SpeedY;
}
#define pointIsInBox(X, Y, Left, Right, Top, Bottom) ( (X >= Left) && (X <= Right) && (Y >= Top) && (Y <= Bottom) )

void tMoveBox(void *pvParameters) {

    drawTaskArgs_t* Arguments = (drawTaskArgs_t*) pvParameters;
    xQueueHandle DrawQueue = Arguments->DrawQueue;

    Assert(NULL != DrawQueue, ec_InvalidArgument);

    DrawData_t* ThisPolygon = &(Arguments->Box2Draw);
    elb_MovingBox_t* ThisBox = &(ThisPolygon->Polygon.MovingBox);

    DrawData_t DrawClearBox;
    DrawClearBox.Type = ddt_MovingBox;
    elb_MovingBox_t* ClearBox = &(DrawClearBox.Polygon.MovingBox);
    ClearBox->Color  = ColorBg;
    ClearBox->SpeedX = 0;
    ClearBox->SpeedY = 0;

    DrawData_t DrawClearLPolygon;
    DrawClearLPolygon.Type = ddt_L_Polygon;
    elb_L_Polygon_t* ClearLPolygon = &(DrawClearLPolygon.Polygon.L_Polygon);
    ClearLPolygon->Color  = ColorBg;

    DrawData_t DrawLPolygon;
    DrawLPolygon.Type = ddt_L_Polygon;
    elb_L_Polygon_t* LPolygon = &(DrawLPolygon.Polygon.L_Polygon);

    queueInsert(DrawQueue, ThisPolygon); // Drawing of the Box(es) in the initial position(s).
    BOOL DrawFullBoxes = FALSE; // select update implementation to use

    Base_t LastUpdate = ttc_task_get_tick_count();

    // shrink screen borders according to box size
    s32_t Border_Right = (DrawAreaWidth  - ThisBox->Width);
    s32_t Border_Lower = (DrawAreaHeight - ThisBox->Height);

    s32_t PosX = ThisBox->PosX;
    s32_t PosY = ThisBox->PosY;
    s32_t NewPosX = PosX;
    s32_t NewPosY = PosY;

    while (1) {
        Base_t Now = ttc_task_get_tick_count();
        if ((Now - LastUpdate) > TimeBase_usecs) { // delay passed: calculate new position + collisions
            LastUpdate = Now;
            if (1) ttc_task_begin_criticalsection(tMoveBox);
            PosX = ThisBox->PosX;
            PosY = ThisBox->PosY;
            NewPosX = PosX + ThisBox->SpeedX;
            NewPosY = PosY + ThisBox->SpeedY;

            if (1) { // collision with other boxes
                s16_t MovementLeft   = MIN (PosX, NewPosX) ;
                s16_t MovementRight  = MAX(NewPosX + ThisBox->Width, PosX + ThisBox->Width);
                s16_t MovementTop    = MIN(PosY,NewPosY);
                s16_t MovementBottom = MAX( NewPosY + ThisBox->Height, PosY + ThisBox->Height);

                DrawData_t** Reader = AllPolygons;
                while (Reader != LastPolygon) {
                    DrawData_t* OtherPolygon = *Reader++;
                    if (ThisPolygon != OtherPolygon) {
                        //X Assert(ttc_memory_is_writable(OtherPolygon), ec_UNKNOWN);
                        elb_MovingBox_t* OtherBox = &(OtherPolygon->Polygon.MovingBox);

                        s16_t OtherBoxLeft   = OtherBox->PosX;
                        s16_t OtherBoxRight  = OtherBox->PosX + OtherBox->Width;
                        s16_t OtherBoxTop    = OtherBox->PosY;
                        s16_t OtherBoxBottom = OtherBox->PosY + OtherBox->Height;

                        BOOL OtherBoxSeen = FALSE;

                        OtherBoxSeen = pointIsInBox(OtherBoxLeft,  OtherBoxTop,    MovementLeft, MovementRight, MovementTop, MovementBottom) ||
                                       pointIsInBox(OtherBoxLeft,  OtherBoxBottom, MovementLeft, MovementRight, MovementTop, MovementBottom) ||
                                       pointIsInBox(OtherBoxRight, OtherBoxTop,    MovementLeft, MovementRight, MovementTop, MovementBottom) ||
                                       pointIsInBox(OtherBoxRight, OtherBoxBottom, MovementLeft, MovementRight, MovementTop, MovementBottom);

                        if (OtherBoxSeen) { // found other box within movement box
                            if (
                                    (
                                        ((OtherBox->PosX < ThisBox->PosX)  && (ThisBox->SpeedX < 0)) ||
                                        ((OtherBox->PosX > ThisBox->PosX)  && (ThisBox->SpeedX > 0))
                                        ) ||
                                    (
                                        ((OtherBox->PosY < ThisBox->PosY)  && (ThisBox->SpeedY < 0)) ||
                                        ((OtherBox->PosY > ThisBox->PosY)  && (ThisBox->SpeedY > 0))
                                        )
                                    )
                            { // this box is moving towards other box: collide
                                if (1) collide(ThisBox, OtherBox);
                                NewPosX += ThisBox->SpeedX;
                                NewPosY += ThisBox->SpeedY;
                            }
                        }
                    }
                }
            }
            if (1) { // collision with borders

                if (NewPosX > Border_Right) {
                    ThisBox->SpeedX = -ThisBox->SpeedX;
                    s32_t Excess = NewPosX - Border_Right;
                    NewPosX -= Excess;
                }
                else if (NewPosX < 0) {
                    ThisBox->SpeedX = -ThisBox->SpeedX;
                    NewPosX = -NewPosX;
                }
                if (NewPosY > Border_Lower) {
                    ThisBox->SpeedY = -ThisBox->SpeedY;
                    s32_t Excess = NewPosY - Border_Lower;
                    NewPosY -= Excess;
                }
                else if (NewPosY < 0) {
                    ThisBox->SpeedY = -ThisBox->SpeedY;
                    NewPosY = -NewPosY;
                }
            }
            if (1) ttc_task_end_criticalsection();
        }

        // create draw instructions to move box
        if (DrawFullBoxes) { // 1st implementation, clear the previous box and redraw all of it
            ClearBox->PosX = PosX;
            ClearBox->PosY = PosY;
            ClearBox->Width  = ThisBox->Width;
            ClearBox->Height = ThisBox->Height;
            queueInsert(DrawQueue, &DrawClearBox);

            ThisBox->PosX = NewPosX;
            ThisBox->PosY = NewPosY;
            queueInsert(DrawQueue, ThisPolygon); // draw box as indicator of implementation switch
        }
        else {               // 2nd implementation: clear as few pixels as possible

            if ((ThisBox->PosX==NewPosX)&&(ThisBox->PosY==NewPosY)) {
                // clear nothing
                ThisBox->PosX=NewPosX;
                ThisBox->PosY=NewPosY;
                queueInsert(DrawQueue, ThisPolygon);
            }
            else if ((ThisBox->PosX==NewPosX)&&(ThisBox->PosY<NewPosY)) {
                if ((ThisBox->PosY+ThisBox->Height)<NewPosY) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = NewPosY - PosY;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
            }
            else if ((ThisBox->PosX==NewPosX)&&(ThisBox->PosY>NewPosY)) {
                if ((NewPosY+ThisBox->Height)<ThisBox->PosY) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = NewPosY+ThisBox->Height;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = PosY-NewPosY;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
            }
            else if ((ThisBox->PosX<NewPosX)&&(ThisBox->PosY==NewPosY)) {
                if ((ThisBox->PosX+ThisBox->Width)<NewPosX) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = NewPosX-PosX;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
            }
            else if ((ThisBox->PosX>NewPosX)&&(ThisBox->PosY==NewPosY)) {
                if ((NewPosX+ThisBox->Width)<ThisBox->PosX) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearBox->PosX   = NewPosX+ThisBox->Width;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = PosX-NewPosX;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
            }

            else if ((ThisBox->PosX<NewPosX)&&(ThisBox->PosY<NewPosY)) { /*Left to Right. Top to Bottom.*/
                if ((NewPosX>ThisBox->PosX+ThisBox->Width)||(NewPosY>ThisBox->PosY+ThisBox->Height)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = ThisBox->Width;
                    ClearLPolygon->Height    = ThisBox->Height;
                    ClearLPolygon->InternalX = NewPosX;
                    ClearLPolygon->InternalY = NewPosY;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color = ThisBox->Color;
                    LPolygon->ExternalX = NewPosX+ThisBox->Width;
                    LPolygon->ExternalY = NewPosY+ThisBox->Height;
                    LPolygon->Width     = ThisBox->Width;
                    LPolygon->Height    = ThisBox->Height;
                    LPolygon->InternalX = PosX+ThisBox->Width;
                    LPolygon->InternalY = PosY+ThisBox->Height;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                }
            }
            else if ((ThisBox->PosX<NewPosX)&&(ThisBox->PosY>NewPosY)) { /*Left to Right. Bottom to Top*/
                if ((NewPosX>ThisBox->PosX+ThisBox->Width)||((NewPosY+ThisBox->Height)<ThisBox->PosY)) {

                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY+ThisBox->Height;
                    ClearLPolygon->Width     = ThisBox->Width;
                    ClearLPolygon->Height    = ThisBox->Height;
                    ClearLPolygon->InternalX = NewPosX;
                    ClearLPolygon->InternalY = NewPosY+ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color = ThisBox->Color;
                    LPolygon->ExternalX = NewPosX+ThisBox->Width;
                    LPolygon->ExternalY = NewPosY;
                    LPolygon->Width     = ThisBox->Width;
                    LPolygon->Height    = ThisBox->Height;
                    LPolygon->InternalX = PosX+ThisBox->Width;
                    LPolygon->InternalY = PosY;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                }
            }
            else if ((ThisBox->PosX>NewPosX)&&(ThisBox->PosY<NewPosY)) { /*Right to Left. Top to Bottom*/
                if (((NewPosX+ThisBox->Width)<ThisBox->PosX)||(NewPosY>ThisBox->PosY+ThisBox->Height)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearLPolygon->ExternalX = PosX+ThisBox->Width;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = ThisBox->Width;
                    ClearLPolygon->Height    = ThisBox->Height;
                    ClearLPolygon->InternalX = NewPosX+ThisBox->Width;
                    ClearLPolygon->InternalY = NewPosY;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color     = ThisBox->Color;
                    LPolygon->ExternalX = NewPosX;
                    LPolygon->ExternalY = NewPosY+ThisBox->Height;
                    LPolygon->Width     = ThisBox->Width;
                    LPolygon->Height    = ThisBox->Height;
                    LPolygon->InternalX = PosX;
                    LPolygon->InternalY = PosY+ThisBox->Height;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                }
            }
            else if ((ThisBox->PosX>NewPosX)&&(ThisBox->PosY>NewPosY)) {
                if (((NewPosX+ThisBox->Width)<ThisBox->PosX)||((NewPosY+ThisBox->Height)<ThisBox->PosY)) {
                    ClearBox->PosX   = PosX;
                    ClearBox->PosY   = PosY;
                    ClearBox->Width  = ThisBox->Width;
                    ClearBox->Height = ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearBox);

                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                    queueInsert(DrawQueue, ThisPolygon);
                }
                else {
                    ClearLPolygon->ExternalX = PosX+ThisBox->Width;
                    ClearLPolygon->ExternalY = PosY+ThisBox->Height;
                    ClearLPolygon->Width     = ThisBox->Width;
                    ClearLPolygon->Height    = ThisBox->Height;
                    ClearLPolygon->InternalX = NewPosX+ThisBox->Width;
                    ClearLPolygon->InternalY = NewPosY+ThisBox->Height;
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    LPolygon->Color     = ThisBox->Color;
                    LPolygon->ExternalX = NewPosX;
                    LPolygon->ExternalY = NewPosY;
                    LPolygon->Width     = ThisBox->Width;
                    LPolygon->Height    = ThisBox->Height;
                    LPolygon->InternalX = PosX;
                    LPolygon->InternalY = PosY;
                    queueInsert(DrawQueue, &DrawLPolygon);
                    ThisBox->PosX=NewPosX;
                    ThisBox->PosY=NewPosY;
                }
            }
        }


        if (0) mSleep(30);
        else ttc_task_yield();
    }
}
void queueInsert(xQueueHandle DrawQueue, DrawData_t *Box) {

    while (xQueueSend(DrawQueue, Box, (portTickType) -1) != pdTRUE) 
      ttc_task_yield();

}
void tDrawIt(void *pvParameters) {
    xQueueHandle DrawQueue= (xQueueHandle) pvParameters;
    Assert(NULL != DrawQueue, ec_InvalidArgument);
    u16_t TimeSecs1 = ttc_task_get_tick_count() / 1000000;
    u16_t TimeSecs2 = TimeSecs1;
    u16_t ElapsedSecs = -1;
    Base_t PolygonCount = 0;

#define SIZE_STRING 20
    static u8_t String[SIZE_STRING];

    while (1) {
        DrawData_t Box2Draw;
        while ( xQueueReceive(DrawQueue, &Box2Draw, (portTickType) -1) ) {
            PolygonCount++;
            if (Box2Draw.Type == ddt_MovingBox) {
                ttc_gfx_set_color_fg16(Box2Draw.Polygon.MovingBox.Color);

                ttc_gfx_rect_fill(Box2Draw.Polygon.MovingBox.PosX   / MathMultiplier,
                                  Box2Draw.Polygon.MovingBox.PosY   / MathMultiplier,
                                  Box2Draw.Polygon.MovingBox.Width  / MathMultiplier,
                                  Box2Draw.Polygon.MovingBox.Height / MathMultiplier
                                  );
            }
            else {
                ttc_gfx_set_color_fg16(Box2Draw.Polygon.L_Polygon.Color);

                LCD_DrawLpolygon(Box2Draw.Polygon.L_Polygon.ExternalX / MathMultiplier,
                                 Box2Draw.Polygon.L_Polygon.ExternalY / MathMultiplier,
                                 Box2Draw.Polygon.L_Polygon.Width     / MathMultiplier,
                                 Box2Draw.Polygon.L_Polygon.Height    / MathMultiplier,
                                 Box2Draw.Polygon.L_Polygon.InternalX / MathMultiplier,
                                 Box2Draw.Polygon.L_Polygon.InternalY / MathMultiplier
                                );
            }
            TimeSecs2 = ttc_task_get_tick_count() / 1000000;
            if (TimeSecs1 != TimeSecs2) {
                if (ElapsedSecs == -1) { // first second passed: reset statistic
                    PolygonCount = 0;
                    ElapsedSecs  = 0;
                }
                else {
                    if (TimeSecs2 > TimeSecs1)
                        ElapsedSecs += TimeSecs2 - TimeSecs1;
                    else
                        ElapsedSecs += TimeSecs2; // ToDo: use correct calculation for tick-counter overflow

                    TimeSecs1 = TimeSecs2;
                    ttc_gfx_text_cursor_set(9, 0);
                    ttc_gfx_set_color_fg16(0);
                    if (ElapsedSecs) {
                        ttc_string_snprintf(String, SIZE_STRING, "%i ", PolygonCount / ElapsedSecs);
                        ttc_gfx_print_solid( (char*) String, SIZE_STRING);
                    }
                }
            }
        }
        ttc_task_yield();
    }
}
void LCD_DrawLpolygon(s32_t ExternalX, s32_t ExternalY, s32_t Width, s32_t Height, s32_t InternalX, s32_t InternalY)
{
    if ((ExternalX<InternalX)&&(ExternalY<InternalY)) {
        ttc_gfx_rect_fill(ExternalX, ExternalY, InternalX-ExternalX, Height);
        ttc_gfx_rect_fill(InternalX, ExternalY, ExternalX + Width - InternalX, InternalY-ExternalY);
    }
    else if ((ExternalX<InternalX)&&(ExternalY>InternalY)) {
        ttc_gfx_rect_fill(ExternalX, ExternalY - Height, InternalX-ExternalX, Height);
        ttc_gfx_rect_fill(InternalX, InternalY, ExternalX + Width - InternalX, ExternalY - InternalY);
    }
    else if ((ExternalX>InternalX)&&(ExternalY<InternalY)) {
        ttc_gfx_rect_fill(ExternalX - Width, ExternalY, Width, InternalY-ExternalY);
        ttc_gfx_rect_fill(InternalX, InternalY, ExternalX-InternalX, ExternalY + Height - InternalY);
    }
    else {
        ttc_gfx_rect_fill(ExternalX - Width, InternalY, Width, ExternalY-InternalY);
        ttc_gfx_rect_fill(InternalX, ExternalY - Height, ExternalX-InternalX, InternalY + Height - ExternalY);
    }
}

//} Function definitions
