/*{ Template_FreeRTOS::.h ************************************************
 
  Example code for FreeRTOS + LCD display on Olimex prototype board STM32-LCD.
 
  written 2012 by Ioanna Tsekoura, Gregor Rebel
     
}*/

//{ Includes *************************************************************

// Basic set of helper functions
#include "../common/ttc_basic.h"
#include "../common/ttc_task.h"
#include "../common/ttc_random.h"
#include "../common/ttc_gfx.h"
#include "../common/ttc_mutex.h"
#include "math/math_basic.h"

//#include <stdlib.h>
//#define USE_STDPERIPH_DRIVER
//#include "stm32f10x.h"
//#include "stm32f10x_rcc.h"
//#include "stm32f10x_conf.h"
//#include "core_cm3.h"        // StdPeripheralsLibrary/CMSIS/CM3/CoreSupport/
//#include "bits.h"
//#include <string.h>
//#include "FreeRTOS.h"
//#include "task.h"
//#include "queue.h"
//#include "lcd-320x240.h"

//} Includes
//{ Defines/ TypeDefs ****************************************************

typedef struct {
    u32_t Color;
    s32_t PosX;
    s32_t PosY;
    s16_t SpeedX;
    s16_t SpeedY;
    u16_t  Width;
    u16_t  Height;
} __attribute__((__packed__)) elb_MovingBox_t;

typedef struct {
    u32_t Color;      //             <- Width ->
    s32_t ExternalX;  // External--> X##########       A
    s32_t ExternalY;  //             ###########       |
    s32_t InternalX;  //             ###X <-- Internal |Height
    s32_t InternalY;  //             ####              |
    u16_t  Width;     //             ####              |
    u16_t  Height;    //             ####              V
} __attribute__((__packed__)) elb_L_Polygon_t;

typedef union {
    elb_MovingBox_t MovingBox;
    elb_L_Polygon_t L_Polygon;
} elb_Polygon_t;

typedef enum  {
    ddt_None,
    ddt_MovingBox,
    ddt_L_Polygon,
    ddt_Unknown
} elb_DrawDataType_e;

typedef struct {
    elb_DrawDataType_e Type;
    elb_Polygon_t Polygon;
} __attribute__((__packed__)) DrawData_t;

typedef struct {
    xQueueHandle DrawQueue;
    DrawData_t      Box2Draw;
} __attribute__((__packed__)) drawTaskArgs_t;


//} Defines
//{ Structures/ Enums ****************************************************


//} Structures/ Enums
//{ Structures/ Enums ****************************************************

//} Structures/ Enums
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************

// main entry: initializes hardware + spawns all threads
// returns immediately
void example_gfx_boxes_start();

// spawns a new box with randomly chosen parameters
// return: != NULL: settings struct of moving box
DrawData_t* newRandomBox(xQueueHandle DrawingQueue);

// spawns a new box with given parameters
// return: != NULL: settings struct of moving box
DrawData_t* newBox(xQueueHandle DrawingQueue, u16_t Color, s32_t X, s32_t Y, u16_t Width, u16_t Height, s8_t SpeedX, s8_t SpeedY);

void tMoveBox(void *pvParameters);
void queueInsert (xQueueHandle DrawQueue, DrawData_t *Polygon2Draw);
void tDrawIt (void *pvParameters);
void LCD_DrawLpolygon(s32_t ExternalX, s32_t ExternalY, s32_t Width, s32_t Height, s32_t InternalX, s32_t InternalY);

// main task controlling LCD-panel
//} Function prototypes
