/*{ Template_FreeRTOS::.c ************************************************

  example_gfx_printf_start() spawns thread taskBrightness()

  taskBrightness()
      Defines brightness of led as Ton / (Ton + Toff)
      spawns thread taskLEDs()
      while (1) {
        while (Ton < 100) { Ton++; Toff--; }
        while (Ton >   0) { Ton--; Toff++; }
      }

  taskLEDs()
      while (1) {
        switch on LED
        wait for Ton
        switch off LED
        wait for Toff
      }

}*/

#include "example_gui.h"

ttc_gfx_generic_t* DisplayConfig = NULL;

u16_t value=50,vorschub,radius=10,X,Y;

void example_gui_start(){
    example_gui_init();

    ttc_task_create(task_gui,       // function to start as thread
                    "INPUT",              // thread name (just for debugging)
                    128,                // stack size
                    (void *) NULL,      // passed as argument to taskGUIs()
                    1,                  // task priority (higher values mean more process time)
                    NULL                // can return a handle to created task
                    );




}

void example_gui_init(){

    u8_t MaxDisplay = ttc_gfx_get_max_display_index();
    Assert(MaxDisplay > 0, ec_DeviceNotFound); // no displays defined by board makefile!
    DisplayConfig = ttc_gfx_get_configuration(1);
    Assert(DisplayConfig, ec_DeviceNotFound);

    // set color to use for first screen clear
    ttc_gfx_set_color_bg24(GFX_COLOR24_WHITE);
    ttc_gfx_set_color_fg24(GFX_COLOR24_RED);
    ttc_gfx_init(1); //initialisation of the lcd
    ttc_input_start(1); //starting input-area-check,
    ttc_gfx_stdout_set(1);




}

void task_gui(void *pvParameters){
    #define SIZE_STRING 5
    static u8_t String[SIZE_STRING];

    pvParameters = pvParameters; // avoids warning: unused parameter 'pvParameters'
    ttc_gfx_text_cursor_set(0,0);
    ttc_gfx_print_boxed("Vorschub",14,0);
    ttc_gfx_text_cursor_set(4,1);
    ttc_gfx_print(" mm",6);
    ttc_gui_plus_button(190,0,239,50,&vorschub);
    ttc_gui_minus_button(139,0,190,50,&vorschub);
    ttc_gfx_text_cursor_set(0,2);
    ttc_gfx_print_boxed("Radius",14,0);
    ttc_gfx_text_cursor_set(3,3);
    ttc_gfx_print(" grad",5);
    ttc_gui_plus_button(190,50,239,100,&radius);
    ttc_gui_minus_button(139,50,190,100,&radius);
    ttc_gui_create_push_button(button_event_clear,NULL,9,260,240,320,"CLEAR");

    while(1){
        function_read_position(&X,&Y);
        ttc_gfx_text_cursor_set(1,1);
        ttc_string_snprintf(String, SIZE_STRING, "%i ", vorschub);
        ttc_gfx_print_solid( (char*) String, SIZE_STRING);
        ttc_gfx_text_cursor_set(1,3);
        ttc_string_snprintf(String, SIZE_STRING, "%i ", radius);
        ttc_gfx_print_solid( (char*) String, SIZE_STRING);
        ttc_gfx_text_cursor_set(0,6);
        ttc_string_snprintf(String, SIZE_STRING, "%i ", X);
        ttc_gfx_print_solid( (char*) String, SIZE_STRING);
        ttc_gfx_text_cursor_set(0,7);
        ttc_string_snprintf(String, SIZE_STRING, "%i ", Y);
        ttc_gfx_print_solid( (char*) String, SIZE_STRING);
        mSleep(100);
    }
    return;
}

void button_event_clear(void *pvParameters){


    ttc_gfx_set_color_bg24(GFX_COLOR24_BLACK);
    ttc_gui_clear();
    ttc_gfx_set_color_bg24(GFX_COLOR24_WHITE);
    ttc_gui_clear();
    ttc_gfx_text_cursor_set(0,0);
    ttc_gfx_print_boxed("Vorschub",14,0);
    ttc_gfx_text_cursor_set(4,1);
    ttc_gfx_print(" mm",6);
    ttc_gui_plus_button(190,0,239,50,&vorschub);
    ttc_gui_minus_button(139,0,190,50,&vorschub);
    ttc_gfx_text_cursor_set(0,2);
    ttc_gfx_print_boxed("Radius",14,0);
    ttc_gfx_text_cursor_set(3,3);
    ttc_gfx_print(" grad",5);
    ttc_gui_plus_button(190,50,239,100,&radius);
    ttc_gui_minus_button(139,50,190,100,&radius);
    ttc_gui_create_push_button(button_event_clear,NULL,9,260,240,320,"CLEAR");


}


