/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_semaphores.h"

#define AmountProducers 1
#define AmountConsumers 6


#ifdef EXTENSION_500_ttc_gfx
  const char* Example    = "Example";
  const char* Threading  = "Threading";
  const char* Semaphores = "Semaphores";

  // configuration data of connected display
  ttc_gfx_generic_t* DisplayConfig = NULL;
#endif

//{ Function definitions *************************************************

  void example_threading_semaphores_start() {

      // Note: This function is called with disabled task scheduler!

#ifdef TTC_LED1
          ttc_gpio_init(TTC_LED1, tgm_output_push_pull);
#endif
#ifdef TTC_LED2
          ttc_gpio_init(TTC_LED2, tgm_output_push_pull);
#endif

  #ifdef EXTENSION_500_ttc_gfx

      u8_t MaxDisplay = ttc_gfx_get_max_display_index();
      Assert(MaxDisplay > 0, ec_DeviceNotFound); // no displays defined by board makefile!
      DisplayConfig = ttc_gfx_get_configuration(1);
      Assert(DisplayConfig, ec_DeviceNotFound);

      // set color to use for first screen clear
      ttc_gfx_set_color_bg24(GFX_COLOR24_BLUE);
      ttc_gfx_set_color_fg24(GFX_COLOR24_YELLOW);
      ttc_gfx_init(1);

      // initialize gfx for use with printf()
      ttc_gfx_stdout_set(1);
      ttc_string_register_stdout(ttc_gfx_stdout_print_block, ttc_gfx_stdout_print_string);

      ttc_gfx_print_solid_at(ttc_gfx_text_center(Example, -1),   0, Example,   -1);
      ttc_gfx_print_solid_at(ttc_gfx_text_center(Threading, -1), 1, Threading, -1);
      ttc_gfx_print_solid_at(ttc_gfx_text_center(Semaphores, -1), 2, Semaphores, -1);

  #endif

      static ets_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

      // create counting semaphore Producer -> Consumer
      ThreadInfo.SemaphoreProducer2Consumer = ttc_semaphore_create();
      Assert(NULL != ThreadInfo.SemaphoreProducer2Consumer, ec_Malloc);

      // create queue Producer/Consumer -> Print
      ThreadInfo.QueuePrint = ttc_queue_create(5, sizeof(et_PrintJob_t));
      Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);

      ThreadInfo.DisplayRow = 4;

  #ifdef EXTENSION_500_ttc_gfx
      Assert(0 != ttc_task_create( taskPrint,                    //{ function to start as thread
                                   "Print",                      // thread name (just for debugging)
                                   TTC_TASK_MINIMAL_STACK_SIZE,  // stack size
                                   (void *) &ThreadInfo,         // passed as argument to taskValue()
                                   1,                            // task priority (higher values mean more process time)
                                   (xTaskHandle*) NULL           // can return a handle to created task
                                   ),
             ec_Malloc); //}
  #endif

      for (int Amount = 0; Amount < AmountProducers; Amount++) {
          Assert(0 != ttc_task_create( taskProducer,                          //{ function to start as thread
                                       "Producer",       // thread name (just for debugging)
                                       TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                                       (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                       1,            // task priority (higher values mean more process time)
                                       (xTaskHandle*) NULL                    // can return a handle to created task
                                       ),
                 ec_Malloc); //}

      }

      for (int Amount = 0; Amount < AmountConsumers; Amount++) {
          Assert(0 != ttc_task_create( taskConsumer,                          //{ function to start as thread
                                       "Consumer",       // thread name (just for debugging)
                                       TTC_TASK_MINIMAL_STACK_SIZE, // stack size
                                       (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                       1,            // task priority (higher values mean more process time)
                                       (xTaskHandle*) NULL                    // can return a handle to created task
                                       ),
                 ec_Malloc); //}
      }
  }
void taskProducer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);

#ifdef EXTENSION_500_ttc_gfx
    ttc_task_begin_criticalsection(taskProducer);
    u8_t DisplayRow = ThreadInfoPtr->DisplayRow++;
    ttc_task_end_criticalsection();

    queuePrintJob(ThreadInfoPtr->QueuePrint, 0, 3, "Q=");
    Base_t BytesFree = ttc_memory_get_free_size();
    queuePrintJobV(ThreadInfoPtr->QueuePrint, 2, DisplayConfig->TextRows - 1, "free: %iB", BytesFree);
    queuePrintJob(ThreadInfoPtr->QueuePrint, 2, DisplayRow, "P>");
#endif

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        Value++;
        if (Value == 100) Value = 0;
        for (int I = 0; I < Value; I+=10) {
            while ( ttc_semaphore_give(ThreadInfoPtr->SemaphoreProducer2Consumer, 1) );
        }
        mSleep(800);

#ifdef EXTENSION_500_ttc_gfx
        queuePrintJobV(ThreadInfoPtr->QueuePrint, 6, DisplayRow, "+%i ", Value/10 + 1);
        displayQueue(ThreadInfoPtr->SemaphoreProducer2Consumer, ThreadInfoPtr->QueuePrint);
#endif
    }
}
void taskConsumer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);

    u8_t DisplayRow = ThreadInfoPtr->DisplayRow++;
    (void) DisplayRow;
#ifdef EXTENSION_500_ttc_gfx
    ttc_task_begin_criticalsection(taskProducer);
    ttc_task_end_criticalsection();

    queuePrintJob(ThreadInfoPtr->QueuePrint, 2, DisplayRow, "C>");
#endif
    u32_t Counter = 0;
    while (1) {
        if (! ttc_semaphore_take(ThreadInfoPtr->SemaphoreProducer2Consumer, 1, -1, taskConsumer) ) { // ==0: take was successfull
            Counter++;
#ifdef TTC_LED1
            if (DisplayRow == 4) {
              for (int I = Counter; I >= 0; I--) {
                  ttc_gpio_set(TTC_LED1);
                  mSleep(400);
                  ttc_gpio_clr(TTC_LED1);
                  mSleep(400);
              }
            }
#endif
#ifdef TTC_LED2
            if (DisplayRow == 5) {
              for (int I = Counter; I >= 0; I--) {
                  ttc_gpio_set(TTC_LED2);
                  mSleep(400);
                  ttc_gpio_clr(TTC_LED2);
                  mSleep(400);
              }
            }
#endif
#ifdef EXTENSION_500_ttc_gfx
            queuePrintJob(ThreadInfoPtr->QueuePrint, 5, DisplayRow, "|");
            mSleep(500);
            queuePrintJob(ThreadInfoPtr->QueuePrint, 5, DisplayRow, "-");
            mSleep(500);
            queuePrintJobV(ThreadInfoPtr->QueuePrint, 7, DisplayRow, "%d", Counter);
            displayQueue(ThreadInfoPtr->SemaphoreProducer2Consumer, ThreadInfoPtr->QueuePrint);
#endif
        }
    }
}

#ifdef EXTENSION_500_ttc_gfx

void displayQueue(ttc_semaphore_smart_t* Semaphore, ttc_QueueHandle QueuePrint) {
    char QueueText[11];
    char* Writer = QueueText;
    Base_t QueueSize = ttc_queue_amount_waiting(Semaphore);
    u8_t I = 0;
    for (; (I < QueueSize/3) && (I < 10); I++)
        *Writer++ = '*';
    for (; I < 10; I++)
        *Writer++ = ' ';
    *Writer++ = 0;
    queuePrintJob(QueuePrint, 2, 3, QueueText);
}
void queuePrintJobV(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s32_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (ttc_queue_push_back(PrintQueue, &PrintJob, -1) != tqe_OK);
}
void queuePrintJob(ttc_QueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text) {
    queuePrintJobV(PrintQueue, Column, Row, Text, 0);
}
void taskPrint(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);

    const u8_t tP_BufferSize = 100;
    u8_t Buffer[tP_BufferSize];

    while (1) {
        et_PrintJob_t PrintJob;
        if ( ttc_queue_pop_front(ThreadInfoPtr->QueuePrint, &PrintJob,  -1) == tqe_OK ) {
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                ttc_gfx_text_cursor_set(PrintJob.Column, PrintJob.Row);

            // create fixed string
            ttc_string_snprintf(Buffer, tP_BufferSize, PrintJob.Text, PrintJob.Value);
            ttc_gfx_print_solid( (char*) Buffer, tP_BufferSize);
        }
    }
}

#endif

//} Function definitions
