#!/bin/bash

#{  Meta install script for STM32-toolchain.
#  written by Gregor Rebel 2011-04-11.
#  
#  This script will automatically run all install_* scripts in current folder.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64
#}

source scripts/installFuncs.sh

function extractFilename() {
  Path="$1"

  export FileName=`perl -e "print substr('$Path', rindex('$Path', '/')+1);"`  
}

Option="$1"
if [ "$Option" == "" ]; then #{ print help + error + exit
  cat <<END_OF_HELP 
$0 BASIC|ALL|MODULE [USERNAME[ GROUP]]

The Toolchain
Starts installation of extension packages.

BASIC     will run all install_XXX.sh scripts in current folder.
ALL       will also run install_XXX.sh scripts in folder optional/

USERNAME  if given, all files will be chowned to this user
          Note: Script must be run by user root for this to work!
GROUP     if given, all files will be chowned to this group

Examples:

$0 BASIC
sudo $0 ALL user1 users

$Error
END_OF_HELP

  if [ "$Error" != "" ]; then
    echo ""
    exit 10
  fi
  exit 0
fi #}

checkUserRoot "$2" "$3"

if [ "$Option" == "BASIC" ]; then #{
  Install_Basic="1"
fi #}
if [ "$Option" == "ALL" ]; then #{
  Install_Basic="1"
  Install_Optional="1"
fi #}


if [ "$Install_Basic" == "1" ]; then #{ run all install-files
  Files=`ls install_*.sh`
  
  if [ "$Install_Optional" == "1" ]; then # add optional files
    Files="$Files `ls optional/install_*.sh`"
  fi

  #{ prepare for installation
  #X rm -f   2>/dev/null ErrorOccured OK.AllInstalls
  rm -f   2>/dev/null ErrorOccured
  rm -Rf 2>/dev/null extensions/*
  #X echo "" >cleanup.sh
  if [ ! -d Logs ]; then
    mkdir Logs
  fi
  for Script in $Files; do
    extractFilename $Script
    if [ ! -e Logs/$FileName.log ]; then
      echo "" >Logs/$FileName.log
    fi
  done
  
  #}prepare for installation
  identifyOS
  if [ "$OS_TYPE" == "opensuse" ]; then #{ add some repositories for openSUSE
    PackManAvailable=`sudo zypper lr | grep -i packman`
    if [ "$PackManAvailable" == "" ]; then
      echo "adding PackMan repository for openSuSE v$OS_VERSION"
      sudo zypper addrepo -f http://packman.inode.at/suse/$OS_VERSION/ PackMan
    fi
  fi #}

  InstallAllDir=`pwd`
  if [ ! -e OK.AllInstalls ]; then #{ get system updates + install software packages
    echo "missing file ${InstallAllDir}/OK.AllInstalls"
    if [ -e /usr/bin/apt-get ]; then
      echo "getting required system updates..."
      sudo apt-get update -y
    fi
    if [ ! -e OK.Packages ]; then #{ install some general software packages
      FailedInstalls="" 
      installPackageSafe "which git"   git gitosis
      installPackageSafe "which svn"   svn subversion
      installPackageSafe "which xterm" xterm
      installPackageSafe "which aterm" aterm
      if [ "$FailedInstalls" == "" ]; then
        touch OK.Packages
      else
        echo "$0 - ERROR: Cannot install required software packages: $FailedInstalls"
        exit 10
      fi
    fi #}
  fi #}
  #{ createLinks
  cat <<END_OF_LINKS >scripts/createLinks.sh #{
#!/bin/bash
# createLinks.sh created `date` by $0
#
# This script is part of ToolChain_STM32

if [ "\$Source" == "" ]; then
  Source="\$HOME/Source"
fi
if [ ! -d extensions.active ]; then
  mkdir extensions.active
fi
cd extensions.active
if [ ! -e readme.txt ]; then
  echo <<"END_OF_README" >readme.txt

                               ToolChain_STM32

                         Extension to central makefile

                       written by Gregor Rebel 2010-`date +%Y`


All makefile.* files in this folder are automatically being included by ../makefile

The idea of this folder is to activate certain extensions by creating symbolic
links to files inside ../extensions

These links get created by several activate-scripts from ../activate_project.sh
You can find these activate scripts inside ../extensions/

END_OF_README
fi

MakefilesPresent=\`ls | grep makefile.\`
cd ..

rm -f 2>/dev/null extensions;  ln -sv \$Source/ToolChain_STM32/ToolChain/extensions  extensions

cd additionals

END_OF_LINKS
#}
  chmod +x scripts/createLinks.sh  
  if [ -d ../Template/additionals/ ]; then
    rm -f 2>/dev/null ../Template/additionals/*
  else
    mkdir ../Template/additionals
  fi
  if [ -d extensions/ ]; then
    rm -f 2>/dev/null extensions/*
  else
    mkdir extensions
  fi
  
  # make some scripts globally available
  rm -f bin/findNewestDownload.pl
  ln -s ../scripts/findNewestDownload.pl bin/
  
  #} createLinks
  #{ copy QtCreator.ini
  
  if [ ! -d ~/.config/ ]; then
    mkdir ~/.config
  fi
  if [ ! -d ~/.config/Nokia/ ]; then
    mkdir ~/.config/Nokia/
  fi
  if [ ! -e ~/.config/Nokia/QtCreator.ini ]; then
    rm -f ~/.config/Nokia/QtCreator.ini
    cp ../Template/QtCreator/QtCreator.ini ~/.config/Nokia/QtCreator.ini
  fi
  
  #}QtCreator.ini
  #{ run all install scripts
  CurrentVersion=`cat ../Version`

  LogFolder="`pwd`/Logs/*"

  # spawn xterm to show current log messages..
  Title="Installing ToolChain_STM32 v$CurrentVersion"
  XtermIsRunning=`ps -Af | grep "$Title" | grep -v grep`
  if [ "$XtermIsRunning" == "" ]; then
    Available=`which aterm`
    if [ "$Available" != "" ]; then
      aterm -T "$Title" -geometry 1920 -e /usr/bin/tail -f Logs/* &
    else
      Available=`which gnome-terminal`
      if [ "$Available" != "" ]; then
        gnome-terminal -x /usr/bin/tail -f Logs/* &
      else    
        xterm -T "$Title" -geometry 1920 -e "tail -f $LogFolder" &
      fi
    fi
  fi

  echo "Runing all install scripts."
  echo "You may check progress by inspecting log files via"
  echo "  tail -f $LogFolder"
  echo ""

  if [ -d extensions/ ]; then
    rm -f -R extensions/*
  fi
  echo -n "clearing log-files"
  for Script in $Files; do
    echo "" >Logs/$Script.log 
    echo "" >Logs/$Script.err
    echo -n "."
  done
  echo "OK"
  
  for Script in $Files; do
    if [ ! -e ErrorOccured ]; then
      Error=""
      $0 $Script "$TargetUser" "$TargetGroup" || Error="1"
      if [ "$Error" != "" ]; then
        echo ""
        echo "ERROR occured during $Script !"
        echo ""
        exit 6
      fi
    fi
  done
  #} run all install scripts
  
  cp ../Version scripts/
  cd scripts
  for TTC_Script in `ls ttc_*`; do
    createLink ../scripts/$TTC_Script $InstallAllDir/bin/$TTC_Script
  done

  #{ remove temporary files
  #X chmod +x cleanup.sh
  #X ./cleanup.sh
  #}remove temporary files
  
  #{ installation finished successfully
  
  cd "$InstallAllDir"
  cd ..
  createLink ToolChain_STM32_$CurrentVersion ../ToolChain_STM32 # enable this ToolChain as current version
  if [ "$TargetUser" != "" ]; then
    chown $TargetUser:$TargetGroup ../ToolChain_STM32
  fi

  cd "$InstallAllDir"
  echo "" >OK.AllInstalls


  addLine scripts/SourceMe.sh  '# ensure that $PATH contains no . (brings find into trouble)'
  AlreadyPresent=`grep "export PATH=" scripts/SourceMe.sh | grep split | grep join`
  if [ "$AlreadyPresent" == "" ]; then
  cat <<'END_OF_TEXT' >>scripts/SourceMe.sh
export PATH=`perl -e "my @A=split(':', '\$PATH');  print join(':', grep { \\\$_ ne '.' } @A );"`
END_OF_TEXT
  fi
  
  addLine scripts/SourceMe.sh "alias commit='git pull && git commit' #$0"
  addLine scripts/SourceMe.sh "export PATH=\"./extensions.local/:\$HOME/Source/ToolChain_STM32/ToolChain/extensions/:\$HOME/Source/ToolChain_STM32/ToolChain/bin/:\$PATH\" #$0"
  source scripts/SourceMe.sh
  OldDir="`pwd`"
  cd ../Template
  ./createLinks.sh
  cd "$OldDir"
  cd ..
  addLine $HOME/.bashrc "source `pwd`/ToolChain/scripts/SourceMe.sh"

  if [ "$TargetUser" != "" ]; then
    echo "chowning `pwd` to $TargetUser:$TargetGroup .."
    chown -R $TargetUser:$TargetGroup .
  fi
  
  cd ..
  echo ""
  echo "All tools have been installed successfully."
  echo "Note: Your .bashrc has been updated."
  echo ""
  echo "Next steps:"
  echo "  (1) Connect JTAG programmer (e.g. ARM-USB-TINY-H) to your computer"
  echo "  (2) Connect your ARM prototype board (e.g. STM32-P107) to JTAG programmer"
  echo "  (3) Start a new shell!"
  echo "  (4) cd '`pwd`/ToolChain_STM32_$CurrentVersion'"
  echo "  (5) ./createNewProject.pl Test ItsMe"
  echo ""
  echo "  Have fun! :-)"

  
  #} installation finished successfully
#} run all install-files
else                                 #{ run single install-file
  Script="$1"

  echo -n "running `pwd`/$Script .."
  dir Logs
  extractFilename $Script
  ./$Script "$TargetUser" "$TargetGroup" >>Logs/$FileName.log 2>>Logs/$FileName.err && OK="1"
  if [ "$OK" == "1" ]; then
    echo "OK"
  else
    echo "ERROR"
    echo ""
    echo "Check logfiles:"
    find `pwd`/Logs/ -name *.log -exec echo "less {}" \; | sort
    touch ErrorOccured
    exit 5
  fi
fi
#}run single install-file
