#!/bin/bash
#            ToolChain_STM32
#
#  Install script for basic extensions.
#
#  Script written by Gregor Rebel 2012
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64
# Restaurant, Tanzen, Kneipe, Kino, Bowling

source scripts/installFuncs.sh
checkUserRoot "$1" "$2"

StartPath="`pwd`"
INSTALLPATH="000_basic_extensions"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  
  if [ ! -e OK.Documentation ]; then #{
    Dir=`pwd`

    # nothing to document (yet)

    if [ "$Error" == "" ]; then
      #echo "Documentation complete: $INSTALLPATH."
      touch OK.Documentation
    fi
  fi #}
  if [ ! -e OK.Install ]; then #{
    
    if [ ! -e OK.Packages ]; then #{
      # nothing to install (yet)
      Arch=`uname -i`
      if [ "$Arch" == "x86_64" ]; then
        installPackage "ia32-libs"
      fi
      touch OK.Install
      touch OK.Packages
    fi #}
  fi #}
  if [ -e OK.Install ]; then #{
    
    Name="${INSTALLPATH}_basic_setup"
    createExtensionMakefileHead ${Name} #{         create makefile for extension
    File="../extensions/makefile.${Name}" 
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

COMPILE_OPTS += -DEXTENSION_${Name}=1

# basic software interface of TheToolChain (ttc_XXX() functions)
INCLUDE_DIRS += -I common/
INCLUDE_DIRS += -I common/scheduler/
INCLUDE_DIRS += -I examples/
INCLUDE_DIRS += -I extensions.active/

vpath %.c common/ common/scheduler/ examples/ extensions.active/

# add some basic object files which are always available
MAIN_OBJS += ttc_extensions.o ttc_extensions_active.o ttc_basic.o

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    createActivateScriptHead $Name ../extensions/ "$0" "Basic toolchain settings required by all architectures and boards." #{ create activate script
    if [ "$Delete" != "" ]; then
      echo "rm 2>/dev/null \$Dir_ExtensionsActive/${INSTALLPATH}" >>$ActivateScriptFile
    fi
    ../scripts/printBaseExtensions.sh >>$ActivateScriptFile
    cat <<END_OF_ACTTIVATE >>$ActivateScriptFile #{

createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

END_OF_ACTTIVATE
#}
    createActivateScriptTail $Name ../extensions/ 
#}
    
    Name="${INSTALLPATH}_optimize_1_debug"
    createExtensionMakefileHead ${Name}
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile #{
  
  DEBUG=1
  OPTIMIZE=-O0 -g3

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name}
    #createActivateScript "../extensions/activate.${Name}.sh" $Name "makefile.${INSTALLPATH}_optimize_1_*" "$0" "compile option: do not optimize at all (best visibility during debugging)"
    createActivateScript $Name ../extensions/ "$0" "makefile.${INSTALLPATH}_optimize_1_*" "compile option: do not optimize at all (best visibility during debugging)"

    Name="${INSTALLPATH}_optimize_1_smallest"
    createExtensionMakefileHead ${Name}
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile #{
    
OPTIMIZE=-Os 

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name}
    createActivateScript $Name ../extensions/ "$0" "makefile.${INSTALLPATH}_optimize_1_*" "compiler option: create small code (can degrade execution speed)"

    Name="${INSTALLPATH}_optimize_1_fastest"
    createExtensionMakefileHead ${Name}
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile #{

  OPTIMIZE=-O9

END_OF_MAKEFILE
#}
    createExtensionMakefileTail ${Name}
    createActivateScript $Name ../extensions/ "$0" "makefile.${INSTALLPATH}_optimize_1_*" "compiler option: create fast code (can increase code- and memory-usage)"
    
    Name="${INSTALLPATH}_optimize_2_inline"
    createExtensionMakefileHead ${Name} #{
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

COMPILE_OPTS += -finline -finline-functions-called-once

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    createActivateScript $Name ../extensions/ "$0" "" "compiler option: try to inline as many functions as possible (can increase code- and memory-usage)"

    Name="${INSTALLPATH}_enable_no_argument_checks"
    createExtensionMakefileHead ${Name} #{
    cat <<END_OF_MAKEFILE >>$ExtensionMakeFile 

COMPILE_OPTS += -DTTC_NO_ARGUMENT_CHECKS

END_OF_MAKEFILE
    createExtensionMakefileTail ${Name} #}
    createActivateScript $Name ../extensions/ "$0" "" "Disables argument checking in ttc_XXX() functions. Use with caution!"

    echo "Installed successfully: $INSTALLPATH"
  fi #}
  cd ..
  rmdir 2>/dev/null $INSTALLPATH  # removes only empty install dir
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

cd "$StartPath"
applyTargetUserAndGroup "$INSTALLPATH"

exit 0
