#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh
checkUserRoot "$1" "$2"

StartPath="`pwd`"
INSTALLPATH="100_board_olimex_lcd"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e $Dir/OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards
    cd ../../Documentation/Boards
    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-LCD.pdf Board_STM32-LCD.pdf
    getFile http://www.olimex.com/dev/pdf/PIC/ "FS-K320QVB-V1-01-3T%20(3).pdf" LCD_FS-K320QVB-V1-01-3T.pdf
    getFile http://olimex.com/dev/ stm32-lcd.html
    if [ "$Get_Fails" == "" ]; then
      touch $Dir/OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}
  cd "$Dir"

  Driver1="450_ttc_gfx_driver_$INSTALLPATH"             #{ gfx driver for this board
  createExtensionMakefileHead ${Driver1} #{
  File="../extensions/makefile.${Driver1}" 
  cat <<END_OF_MAKEFILE >>$File 

# ttc_gfx driver settings for board Olimex-LCD
# reference: http://olimex.com/dev/ARM/ST/STM32-LCD/STM32-LCD-schematic.pdf

# LCD-Display 320x240x24 - define settings required by low-level driver to run on current board

COMPILE_OPTS += -DTTC_GFX1_WIDTH=240
COMPILE_OPTS += -DTTC_GFX1_HEIGHT=320
COMPILE_OPTS += -DTTC_GFX1_DEPTH=24
COMPILE_OPTS += -DTTC_GFX1_DRIVER=tgd_ILI9320    # one from ttc_gfx_types.h:ttc_gfx_driver_e

END_OF_MAKEFILE
  createExtensionMakefileTail ${Driver1} #}
  createActivateScriptHead $Driver1 ../extensions/ $0 "gfx driver for Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile #{

  #rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Driver1 \$Dir_ExtensionsActive/makefile.$Driver1 '' QUIET
  
  activate.400_lcd_320x240_ili9320.sh QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $Driver1 ../extensions/
  #}Driver1
  Driver2="450_ttc_input_driver_$INSTALLPATH"           #{ touch-handler driver for this board
  createExtensionMakefileHead ${Driver2} #{
  File="../extensions/makefile.${Driver2}" 
  cat <<END_OF_MAKEFILE >>$File 

# ttc_input driver settings for board Olimex-LCD
# reference: http://olimex.com/dev/ARM/ST/STM32-LCD/STM32-LCD-schematic.pdf

# Touch-Display - define settings required by low-level driver to run on current board

COMPILE_OPTS += -DTTC_INPUT1_DRIVER=tid_Analog # one from ttc_input_types.h:ttc_input_driver_e
COMPILE_OPTS += -DTTC_INPUT1_WIDTH=240
COMPILE_OPTS += -DTTC_INPUT1_HEIGHT=320
COMPILE_OPTS += -DTTC_INPUT1_DEPTH=24
COMPILE_OPTS += -DTTC_INPUT_DIGITAL_XR=PIN_PC3
COMPILE_OPTS += -DTTC_INPUT_DIGITAL_XL=PIN_PC2
COMPILE_OPTS += -DTTC_INPUT_DIGITAL_YU=PIN_PC1
COMPILE_OPTS += -DTTC_INPUT_DIGITAL_YD=PIN_PC0
COMPILE_OPTS += -DTTC_INPUT_ANALOG_XR=ADC_Channel_13
COMPILE_OPTS += -DTTC_INPUT_ANALOG_XL=ADC_Channel_12
COMPILE_OPTS += -DTTC_INPUT_ANALOG_YU=ADC_Channel_11
COMPILE_OPTS += -DTTC_INPUT_ANALOG_YD=ADC_Channel_10
COMPILE_OPTS += -DTTC_INPUT_ANALOG_ADC=ADC1

END_OF_MAKEFILE
  createExtensionMakefileTail ${Driver2} #}
  createActivateScriptHead $Driver2 ../extensions/ $0 "touch-handler driver for Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile #{

  #rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Driver2 \$Dir_ExtensionsActive/makefile.$Driver2 '' QUIET
  
  activate.400_input_analog.sh QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $Driver2 ../extensions/
  #}Driver2
  Driver3="450_ttc_accelerometer_driver_$INSTALLPATH"   #{ accelerometer driver for this board
  createExtensionMakefileHead ${Driver3} #{
  File="../extensions/makefile.${Driver3}" 
  cat <<END_OF_MAKEFILE >>$File 

# ttc_input driver settings for board Olimex-LCD
# reference: http://olimex.com/dev/ARM/ST/STM32-LCD/STM32-LCD-schematic.pdf

# Accelerometer - define settings required by low-level driver to run on current board



END_OF_MAKEFILE
  createExtensionMakefileTail ${Driver3} #}
  createActivateScriptHead $Driver3 ../extensions/ $0 "accelerometer driver for Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile #{

  #rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Driver3 \$Dir_ExtensionsActive/makefile.$Driver3 '' QUIET
  
  activate.400_sensor_lis3lv02dl.sh QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $Driver3 ../extensions/
  #}Driver3
  Name="${INSTALLPATH}"                          #{ board definitions
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

COMPILE_OPTS += -DBOARD=$Name
BOARD=$Name

# define microcontroller used on board
STM32F10X_HD  =1
COMPILE_OPTS += -DUC_DEVICE=STM32F103ZE
COMPILE_OPTS += -DUC_PACKAGE_PINS=144

# Note: Make sure to set correct sizes for RAM and Flash
#
# updateProjectFiles.pl will automatically create memory regions in configs/memory.ld 
# for every define of this format (xxx = name of region, n = numeric value)
# COMPILE_OPTS += -DTTC_MEMORY_REGION_xxx_SIZE_K=n
# COMPILE_OPTS += -DTTC_MEMORY_REGION_xxx_START=n
# COMPILE_OPTS += -DTTC_MEMORY_REGION_xxx_ACCESS=rxwa
#
COMPILE_OPTS += -DTTC_MEMORY_REGION_RAM_SIZE_K=64
COMPILE_OPTS += -DTTC_MEMORY_REGION_RAM_START=0x20000000
COMPILE_OPTS += -DTTC_MEMORY_REGION_RAM_ACCESS=rxw
COMPILE_OPTS += -DTTC_MEMORY_REGION_ROM_SIZE_K=512
COMPILE_OPTS += -DTTC_MEMORY_REGION_ROM_START=0x08000000
COMPILE_OPTS += -DTTC_MEMORY_REGION_ROM_ACCESS=rx

# Define port pins
# reference: http://olimex.com/dev/ARM/ST/STM32-LCD/STM32-LCD-schematic.pdf
#{ parallel ports (8 or 16 consecutive pins in same bank)
#
# Every 8 available, consecutive bits in same GPIO bank can form an 8-bit wide port.
# A 16-bit wide port requires 16 available, consecutive bits (obviously).
# A pin is available if it not driven externally or used for something else.
# Check circuit of your board for details.

# 16-bit wide parallel port #1 using whole bank F (16-bit always starts a pin 0)
COMPILE_OPTS += -DTTC_PARALLEL16_1_BANK=GPIO_BANK_F

# 16-bit wide parallel port #2 using whole bank G (16-bit always starts a pin 0)
COMPILE_OPTS += -DTTC_PARALLEL16_2_BANK=GPIO_BANK_G


# 8-bit wide parallel port #1 starting at pin 0 of bank F
COMPILE_OPTS += -DTTC_PARALLEL08_1_BANK=GPIO_BANK_F
COMPILE_OPTS += -DTTC_PARALLEL08_1_FIRSTPIN=0

# 8-bit wide parallel port #2 starting at pin 8 of bank F
COMPILE_OPTS += -DTTC_PARALLEL08_2_BANK=GPIO_BANK_F
COMPILE_OPTS += -DTTC_PARALLEL08_2_FIRSTPIN=8

# 8-bit wide parallel port #3 starting at pin 0 of bank G
COMPILE_OPTS += -DTTC_PARALLEL08_3_BANK=GPIO_BANK_G
COMPILE_OPTS += -DTTC_PARALLEL08_3_FIRSTPIN=0

# 8-bit wide parallel port #4 starting at pin 8 of bank G
COMPILE_OPTS += -DTTC_PARALLEL08_4_BANK=GPIO_BANK_G
COMPILE_OPTS += -DTTC_PARALLEL08_4_FIRSTPIN=8

#}parallel ports
#{ USART1  - define universal synchronous asynchronous receiver transmitter #1
COMPILE_OPTS += -DTTC_USART1=INDEX_USART1       # RS232 on 10-pin male header connector UEXT1 connected USART
COMPILE_OPTS += -DTTC_USART1_TX=PIN_PA9   # UEXT1-3, UEXT40-3
COMPILE_OPTS += -DTTC_USART1_RX=PIN_PA10  # UEXT1-4, UEXT40-4
COMPILE_OPTS += -DTTC_USART1_CK=PIN_PA8   # EXT-40
#}USART1
#{ USART2  - define universal synchronous asynchronous receiver transmitter #2
  COMPILE_OPTS += -DTTC_USART2=INDEX_USART2      # RS232 on 10-pin male header connector UEXT2 connected USART
  COMPILE_OPTS += -DTTC_USART2_TX=PIN_PA2  # UEXT2-3
  COMPILE_OPTS += -DTTC_USART2_RX=PIN_PA3  # UEXT2-4
  COMPILE_OPTS += -DTTC_USART2_CK=PIN_PA4  # UEXT1-10
#}USART2
#{ SPI1    - define serial peripheral interface #1
  COMPILE_OPTS += -DTTC_SPI1=SPI1          # 
  COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PA7  # UEXT1-8,  UEXT40-8
  COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PA6  # UEXT1-7,  UEXT40-7
  COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PA5   # UEXT1-9,  UEXT40-9
  COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PA4   # UEXT1-10, UEXT40-10
#}SPI1
#{ SPI2    - define serial peripheral interface #2
  COMPILE_OPTS += -DTTC_SPI2=SPI2          # 
  COMPILE_OPTS += -DTTC_SPI2_MOSI=PIN_PB15 # UEXT1-8
  COMPILE_OPTS += -DTTC_SPI2_MISO=PIN_PB14 # UEXT1-7
  COMPILE_OPTS += -DTTC_SPI2_SCK=PIN_PB13  # UEXT1-9
  COMPILE_OPTS += -DTTC_SPI2_NSS=PIN_PB12  # UEXT2-10
#}SPI2
#{ I2C1    - define inter-integrated circuit interface #1
  COMPILE_OPTS += -DTTC_I2C1=I2C1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB7    # UEXT2-5/ UEXT40-6
  COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB6    # UEXT2-6/ UEXT40-5
  COMPILE_OPTS += -DTTC_I2C1_SMBAL=PIN_PB5  # EXT-10
#}I2C1
#{ I2C2    - define inter-integrated circuit interface #2
  COMPILE_OPTS += -DTTC_I2C2=I2C2           # internal device being used for this port
  COMPILE_OPTS += -DTTC_I2C2_SDA=PIN_PB11   # connected to accelerometer
  COMPILE_OPTS += -DTTC_I2C2_SCL=PIN_PB10   # connected to accelerometer
#}I2C2
#{ define Controller Area Network (CAN) devices
  COMPILE_OPTS += -DTTC_CAN1=0               # physical device index
  COMPILE_OPTS += -DTTC_CAN1_PIN_RX=PIN_PA11  # pin used for receive
  COMPILE_OPTS += -DTTC_CAN1_PIN_TX=PIN_PA12  # pin used for transmit
#}CAN
#{ define pin configuration of GPIOs on the UEXT1 Conncector
  COMPILE_OPTS += -DUEXT1_USART_TX=PIN_PA9
  COMPILE_OPTS += -DUEXT1_USART_RX=PIN_PA10
  COMPILE_OPTS += -DUEXT1_I2C_SCL=PIN_PB6
  COMPILE_OPTS += -DUEXT1_I2C_SDA=PIN_PB7
  COMPILE_OPTS += -DUEXT1_SPI_MISO=PIN_PA6
  COMPILE_OPTS += -DUEXT1_SPI_MOSI=PIN_PA7
  COMPILE_OPTS += -DUEXT1_SPI_SCK=PIN_PA5
  COMPILE_OPTS += -DUEXT1_SPI_NSS=PIN_PA4
  
  COMPILE_OPTS += -DUEXT1_SPI_INDEX=1
  COMPILE_OPTS += -DUEXT1_USART_INDEX=1
  COMPILE_OPTS += -DUEXT1_I2C_INDEX=1
#}
#{ define pin configuration of GPIOs on the UEXT2 Conncector
  COMPILE_OPTS += -DUEXT2_USART_TX=PIN_PA3
  COMPILE_OPTS += -DUEXT2_USART_RX=PIN_PA2
  COMPILE_OPTS += -DUEXT2_I2C_SCL=PIN_PB8
  COMPILE_OPTS += -DUEXT2_I2C_SDA=PIN_PB9
  COMPILE_OPTS += -DUEXT2_SPI_MISO=PIN_PB14
  COMPILE_OPTS += -DUEXT2_SPI_MOSI=PIN_PB15
  COMPILE_OPTS += -DUEXT2_SPI_SCK=PIN_PA13
  COMPILE_OPTS += -DUEXT2_SPI_NSS=PIN_PA12
  
  COMPILE_OPTS += -DUEXT2_SPI_INDEX=2
  COMPILE_OPTS += -DUEXT2_USART_INDEX=2
  COMPILE_OPTS += -DUEXT2_I2C_INDEX=1
  
#}

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScriptHead $Name ../extensions/ $0 "touch-handler driver for Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32f10x.sh QUIET "\$0"
  #X activate.400_lcd_320x240_K320QVB.sh QUIET "\$0"
  #X activate.400_input_analog.sh QUIET "\$0"
  #X activate.400_sensor_lis3lv02dl.sh QUIET "\$0"
  
  # activate drivers for board components
  activate.${Driver1}.sh QUIET "\$0"
  activate.${Driver2}.sh QUIET "\$0"
  activate.${Driver3}.sh QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $Name ../extensions/
  #}board definitions

#}
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

cd "$StartPath"
applyTargetUserAndGroup "$INSTALLPATH"

exit 0
