#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh
checkUserRoot "$1" "$2"

StartPath="`pwd`"
INSTALLPATH="100_board_wsn3_stm32w108"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------
  Dir=`pwd`
  if [ ! -e $Dir/OK.Docs ]; then #{
    Get_Fails=""
    dir ../../Documentation/Boards
    cd ../../Documentation/Boards
    getFile http://olimex.com/dev/pdf/ARM/ST/ STM32-LCD.pdf Board_STM32-LCD.pdf
    getFile http://www.olimex.com/dev/pdf/PIC/ "FS-K320QVB-V1-01-3T%20(3).pdf" LCD_FS-K320QVB-V1-01-3T.pdf
    getFile http://olimex.com/dev/ stm32-lcd.html
    if [ "$Get_Fails" == "" ]; then
      touch $Dir/OK.Docs
    else
      echo "$0 - ERROR: missing files: $Get_Fails"
    fi
  fi #}
  cd "$Dir"

  Name="${INSTALLPATH}"
  createExtensionMakefileHead ${Name} #{
  File="../extensions/makefile.${Name}" 
  cat <<END_OF_MAKEFILE >>$File 

COMPILE_OPTS += -DBOARD=$Name
BOARD=$Name

# uController to use
STM32W108   = 1
COMPILE_OPTS += -DUC_DEVICE=STM32W108
COMPILE_OPTS += -DUC_PACKAGE_PINS=48

# set define required by stm32w108xx.h in STM32W108xx_SimpleMAC_V2.0.1/
COMPILE_OPTS += -DSTM32W108CB

# set define required by hal.h in STM32W108xx_SimpleMAC_V2.0.1/
COMPILE_OPTS += -DUSE_MB851_REVA_REVB

COMPILE_OPTS += -DuCONTROLLER=STM32W108

# Note: Make sure to set correct sizes for RAM and Flash
#
# updateProjectFiles.pl will automatically create memory regions in configs/memory.ld 
# for every define of this format (xxx = name of region, n = numeric value)
# COMPILE_OPTS += -DTTC_MEMORY_REGION_xxx_SIZE_K=n
# COMPILE_OPTS += -DTTC_MEMORY_REGION_xxx_START=n
# COMPILE_OPTS += -DTTC_MEMORY_REGION_xxx_ACCESS=rxwa
#
COMPILE_OPTS += -DTTC_MEMORY_REGION_RAM_SIZE_K=8
COMPILE_OPTS += -DTTC_MEMORY_REGION_RAM_START=0x20000000    # all em357 have same start address  
COMPILE_OPTS += -DTTC_MEMORY_REGION_RAM_ACCESS=rxw
COMPILE_OPTS += -DTTC_MEMORY_REGION_ROM_SIZE_K=128
COMPILE_OPTS += -DTTC_MEMORY_REGION_ROM_START=0x08000000  # all em357 have same start address
COMPILE_OPTS += -DTTC_MEMORY_REGION_ROM_ACCESS=rx
COMPILE_OPTS += -DTTC_MEMORY_REGION_FIB_SIZE_K=2
COMPILE_OPTS += -DTTC_MEMORY_REGION_FIB_START=0x08040000  # all em357 have same start address
COMPILE_OPTS += -DTTC_MEMORY_REGION_FIB_ACCESS=ra

# set some well known port defines
# Note: Using the Constant names listed here makes it easier to reuse example code! 


#BoardRev10=1
#BoardRev11=1
BoardRev12=1


ifdef BoardRev10  #{
  # LED-pins (push pull output)
  COMPILE_OPTS += -DTTC_LED1=PIN_PC6
  COMPILE_OPTS += -DTTC_LED2=PIN_PC7

  # Switches (input with pull up resistor)
  COMPILE_OPTS += -DTTC_SWITCH1=PIN_PB0   
  COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH2=PIN_PB6
  COMPILE_OPTS += -DTTC_SWITCH2_TYPE=tgm_input_floating # -> ttc_gpio_types.h

  COMPILE_OPTS += -DTTC_SWITCH3=PIN_PA1
  COMPILE_OPTS += -DTTC_SWITCH3_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH4=PIN_PA2
  COMPILE_OPTS += -DTTC_SWITCH4_TYPE=tgm_input_floating # -> ttc_gpio_types.h
endif
#}
ifdef BoardRev11  #{
  # LED-pins (push pull output)
  COMPILE_OPTS += -DTTC_LED1=PIN_PC6
  COMPILE_OPTS += -DTTC_LED2=PIN_PC7

  # Switches (input with pull up resistor)
  COMPILE_OPTS += -DTTC_SWITCH1=PIN_PB0   
  COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH2=PIN_PB6
  COMPILE_OPTS += -DTTC_SWITCH2_TYPE=tgm_input_floating # -> ttc_gpio_types.h

  COMPILE_OPTS += -DTTC_SWITCH3=PIN_PA1
  COMPILE_OPTS += -DTTC_SWITCH3_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH4=PIN_PA2
  COMPILE_OPTS += -DTTC_SWITCH4_TYPE=tgm_input_floating # -> ttc_gpio_types.h
endif
#}
ifdef BoardRev12  #{
  # LED-pins (push pull output)
  COMPILE_OPTS += -DTTC_LED1=PIN_PC5
  COMPILE_OPTS += -DTTC_LED2=PIN_PA7

  # Switches (input with pull up resistor)
  COMPILE_OPTS += -DTTC_SWITCH1=PIN_PC1
  COMPILE_OPTS += -DTTC_SWITCH1_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH2=PIN_PB0
  COMPILE_OPTS += -DTTC_SWITCH2_TYPE=tgm_input_floating # -> ttc_gpio_types.h

  COMPILE_OPTS += -DTTC_SWITCH3=PIN_PA1
  COMPILE_OPTS += -DTTC_SWITCH3_TYPE=tgm_input_floating # -> ttc_gpio_types.h
  COMPILE_OPTS += -DTTC_SWITCH4=PIN_PA2
  COMPILE_OPTS += -DTTC_SWITCH4_TYPE=tgm_input_floating # -> ttc_gpio_types.h
endif
#}

#{ USART1 - define serial port #1
  COMPILE_OPTS += -DTTC_USART1=UART1
  COMPILE_OPTS += -DTTC_USART1_TX=PIN_PB1
  COMPILE_OPTS += -DTTC_USART1_RX=PIN_PB2
  COMPILE_OPTS += -DTTC_USART1_RTS=PIN_PB4 
  COMPILE_OPTS += -DTTC_USART1_CTS=PIN_PB3
#}USART1
#{ SPI1   - define serial peripheral interface port #1
  COMPILE_OPTS += -DTTC_SPI1=SPI1           # internal device being used for this port
  COMPILE_OPTS += -DTTC_SPI1_MOSI=PIN_PB1
  COMPILE_OPTS += -DTTC_SPI1_MISO=PIN_PB2
  COMPILE_OPTS += -DTTC_SPI1_SCK=PIN_PB3
  COMPILE_OPTS += -DTTC_SPI1_NSS=PIN_PB4
#}SPI1
#{ SPI2   - define serial peripheral interface port #2
  COMPILE_OPTS += -DTTC_SPI2=SPI2
  COMPILE_OPTS += -DTTC_SPI2_MOSI=PIN_PA0
  COMPILE_OPTS += -DTTC_SPI2_MISO=PIN_PA1
  COMPILE_OPTS += -DTTC_SPI2_SCK=PIN_PA2
  COMPILE_OPTS += -DTTC_SPI2_NSS=PIN_PA3
#}SPI2
#{ I2C1   - (Inter-Integrated Circuit-) Interface #1
  COMPILE_OPTS += -DTTC_I2C1=I2C1
  COMPILE_OPTS += -DTTC_I2C1_SDA=PIN_PB1  # CONN_30-16
  COMPILE_OPTS += -DTTC_I2C1_SCL=PIN_PB2  # CONN_30-15
#}I2C1
#{ I2C2   - (Inter-Integrated Circuit-) Interface #2
  COMPILE_OPTS += -DTTC_I2C2=I2C2
  COMPILE_OPTS += -DTTC_I2C2_SDA=PIN_PA1  # CONN_30-22
  COMPILE_OPTS += -DTTC_I2C2_SCL=PIN_PA2  # CONN_30-21
#}I2C1

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScriptHead $Name ../extensions/ $0 "Protoboard LCD from Olimex -> http://olimex.com/dev/stm32-lcd.html"  #{
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile #{

  rm 2>/dev/null \$Dir_ExtensionsActive/makefile.100_board_*
  
  # create links into extensions.active/
  createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET
  
  activate.200_cpu_stm32w1xx.sh                                  QUIET "\$0"
  activate.251_cpu_stm32w1xx_simple_mac.sh                       QUIET "\$0"
  activate.253_cpu_stm32w1xx_hal.sh                              QUIET "\$0"
  activate.255_cpu_stm32w1xx_std_peripherals_stm32w108xx_sc.sh   QUIET "\$0"
  activate.255_cpu_stm32w1xx_std_peripherals_stm32w108xx_clk.sh  QUIET "\$0"
  activate.255_cpu_stm32w1xx_std_peripherals_stm32w108xx_gpio.sh QUIET "\$0"
  activate.255_cpu_stm32w1xx_std_peripherals_stm32w108xx_adc.sh  QUIET "\$0"

END_OF_ACTIVATE
#}
  createActivateScriptTail $Name ../extensions/
  #}

    
  if [ "1" == "0" ]; then #{ moved installation of kernel module + rules.d entry to install_03_OpenOCD.sh

  BootLocal=""
  for File in /etc/rc.local /etc/init.d/boot.local; do 
    if [ -e "$File" ]; then
      BootLocal="$File"
    fi
  done
  echo "BootLocal='$BootLocal'"
  
  if [ "$BootLocal" != "" ]; then
    Line="rmmod ftdi_sio; modprobe ftdi_sio product=0xbafa vendor=0x0403 # added by $0 for ToolChainSTM32"
    AlreadyPatched=`grep "$Line" $BootLocal`
    if [ "$AlreadyPatched" == "" ]; then #{
      cp $BootLocal boot.local
      replaceInFile boot.local "exit 0" "#exit 0"
      addLine boot.local $Line
      echo "patching file ${BootLocal}..."
      sudo cp boot.local $BootLocal
      sudo chown :root $BootLocal
    fi #}
  fi 
    Rules="45-openlink-permissions.rules"
    if [ ! -e "/etc/udev/rules.d/$Rules" ]; then #{ create udev-rule
      cat <<"END_OF_RULES" >$Rules #{

# Note: read info of current udev device
# udevadm info -a -p $(udevadm info -q path -n ttyUSB0)

SUBSYSTEMS=="usb-serial", DRIVERS=="ftdi_sio", SYMLINK+="ttyUSB_FTDI"

# OpenLink Programming Adapter v1.1 (-> http://thetoolchain.com)
SUBSYSTEM=="usb", ATTRS{idVendor}=="0403", ATTRS{idProduct}=="bafa", MODE="0664", GROUP="dialout", DRIVERS=="ftdi_sio", NAME="ttyWSN"

END_OF_RULES
#}
  
      sudo cp $Rules /etc/udev/rules.d/
      sudo udevadm control --reload-rules
    fi #}udev-rule
  fi #}
  
  echo "Installed successfully: $INSTALLPATH"

cd ..
rmdir 2>/dev/null $INSTALLPATH
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

cd "$StartPath"
applyTargetUserAndGroup "$INSTALLPATH"

exit 0
