#!/bin/bash

#
#  Install script for Fixpoint Mathematical Library 
#  URL: http://code.google.com/p/fixpointlib/
#
#  Initial Script written by Gregor Rebel 2010-2012.
# 
#  Published under GNU Lesser General Public License
#  Check LEGAL.txt for details.
#
# Tested successfully on:
# openSuSE 12.1 x86
# XUbuntu 11.10 x64

INSTALLPATH="050_compiler_gcc_arm_binary"

source scripts/installFuncs.sh
checkUserRoot "$1" "$2"

StartPath="`pwd`"
dir "$INSTALLPATH"
cd "$INSTALLPATH"

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

# look for newest versions on download pages
NewestDownloads="../${0}_downloads"
if [ "$1" == "FINDNEWEST" ] || [ ! -e "$NewestDownloads" ]; then #{ find newest downloads
  echo -n "$0: looking for new GCC... "
  updateNewest GCC https://launchpad.net/gcc-arm-embedded/+download 'gcc-arm-none-eabi-.+-linux\.tar\.bz2' $NewestDownloads
  if [ "$return" != "" ]; then
    # prepare folder for new version
    rm -Rf *
  else
    exit 0
  fi
fi #}
source $NewestDownloads
if [ "$GCC_URL" == "" ] || [ "$GCC_File" == "" ]; then #{
  echo "$0 - ERROR: Cannot find newest GCC version!"
  exit 10
fi #}

if [ ! -e OK.Docs ];    then    #{ download documentation

  Get_Fails=""
  
  # download doc-file + move to Documentation/ + create symbolic link into Documentation/ right here 
  # File="STM32F10x_Standard_Peripherals_Library.html"
  # getFile http://www.mikrocontroller.net/articles/ STM32F10x_Standard_Peripherals_Library $File
  # addDocumentationFile $File STM
  
  if [ "$Get_Fails" == "" ]; then #{ check get() inside scripts/installFuncs.sh 
    touch OK.Docs
  else
    echo "$0 - ERROR: missing files: $Get_Fails"
  fi #}
fi #}
if [ ! -e OK.Install ]; then    #{ download + install software
  if [ ! -e OK.Packages ]; then #{ install packages
    # installPackage libgmp3-dev
    
    touch OK.Packages
  fi #}
  if [ -e OK.Packages ];   then #{ install software
    echo "installing in $INSTALLPATH ..."
    InstallDir=`pwd`
    
    getFile $GCC_URL $GCC_File $GCC_File
    untgj "current" $GCC_File
    Dir=`find ./ -maxdepth 1 -mindepth 1 -type d -name "gcc-arm-none-eabi*"`
    createLink "$Dir" "current"
    
    if [ -e "current/bin/arm-none-eabi-gcc" ]; then
      echo "" >OK.Install
    fi
  else
    echo "$0 - ERROR: Packages were not installed correctly!"
  fi #}OK.Packages
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then    #{ create makefiles and activate scripts

  BinFolder="`pwd`/current/bin"
  ToolChainBinFolder="Source/ToolChain_STM32/ToolChain/bin"

  Name="${INSTALLPATH}"  
  createExtensionSourcefileHead ${Name}    #{ create initializer code for your extension
  
  # Below you may write C-code that will automatically being placed inside extensions.active/ttc_extensions_active.c during _/compile.sh
  # Every #include directive will automatically being added to extensions.active/ttc_extensions_active.h
  #
  cat <<END_OF_SOURCEFILE >>$ExtensionSourceFile

//  #include "YOUR_SOURCE_HERE.h"
//  YOUR_START_HERE();

END_OF_SOURCEFILE
  createExtensionSourcefileTail ${Name} #}    
  createExtensionMakefileHead ${Name}      #{ create makefile
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile # variable set by createExtensionMakefileHead() 

COMPILE_OPTS += -DSOURCE_COMPILER=$Name

INCLUDE_DIRS += -I additionals/$Name/arm-none-eabi/include/ \\
                -I additionals/$Name/lib/gcc/arm-none-eabi/4.4.1/include/

# binaries from CodeSourcery must be somewhere in $PATH
CC =arm-none-eabi-gcc
AR =arm-none-eabi-ar
CP =arm-none-eabi-objcopy
OD =arm-none-eabi-objdump
#LD =arm-none-eabi-ld -v
LD =arm-none-eabi-gcc                                       # workaround: floating-point bug (http://fun-tech.se/stm32/OlimexBlinky/ld_float.php)
#AS =arm-none-eabi-as
AS =\$(CC) \$(AS_OPTS) -x assembler-with-cpp -c \$(TARGET)  # workaround: .o-files compiled by as cannot be linked by ld

CFLAGS =-std=gnu99 -c -fno-common -fno-builtin -ffreestanding -funsigned-char \$(COMPILE_OPTS) \$(TARGET) 
LDFLAGS=-Wl,-Map=\$(MAIN_MAP) -nostartfiles \$(TARGET)
#X LDFLAGS += -Wl,-\\( \${filter %.a,\$^} \$(LD_LIBS) -Wl,-\\)      # workaround: make pre-compiled libraries available 
CPFLAGS=-Obinary
ODFLAGS	= -S
ASFLAGS=\$(COMPILE_OPTS)
# ASFLAGS=-gdwarf2 -mcpu=\$(ARM_CPU) -mthumb-interwork -o\$@

END_OF_MAKEFILE
  createExtensionMakefileTail ${Name} #}
  createActivateScriptHead $Name ../extensions/ "$0 2" "binary distribution of recent GCC for CortexM3 published by ARM Corp." #{ create activate script
  cat <<END_OF_ACTIVATE >>$ActivateScriptFile
# available variables (-> scripts/installFuncs.sh:createActivateScriptHead())
#
# Paths to certain directories in current project folder
#   \$Dir_Extensions        -> extensions/
#   \$Dir_ExtensionsLocal   -> extensions.local/
#   \$Dir_ExtensionsActive  -> extensions.active/
#   \$Dir_Additionals       -> additionals/

# ACTIVATE_SECTION_A remove activated variants of same type
rm 2>/dev/null \$Dir_ExtensionsActive/*.050_compiler_*

# ACTIVATE_SECTION_B create link to your makefile into extensions.active/ to add makefile to your project
createLink \$Dir_Extensions/makefile.$Name \$Dir_ExtensionsActive/makefile.$Name '' QUIET

# ACTIVATE_SECTION_C activate initialization source-code for this extension 
#createLink \$Dir_Extensions/$ExtensionSourceFile \$Dir_ExtensionsActive/ '' QUIET

rm \$HOME/$ToolChainBinFolder/arm-none-eabi-*
ln -s $BinFolder/arm-none-eabi-* \$HOME/$ToolChainBinFolder/

END_OF_ACTIVATE
  createActivateScriptTail $Name ../extensions/
  #}
  # create symbolic link from extension folder into every PROJECT_FOLDER/additionals/ to provide extra source code and headers
  addLine ../scripts/createLinks.sh "rm 2>/dev/null $INSTALLPATH; ln -sv \$Source/ToolChain_STM32/ToolChain/${INSTALLPATH}/current/ $INSTALLPATH"
    
  cd "$OldPWD"
  echo "Installed successfully: $INSTALLPATH"
#}
else                            #{ Install failed
  echo "failed to install $INSTALLPATH"
  exit 10
fi #}

cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

cd "$StartPath"
applyTargetUserAndGroup "$INSTALLPATH"

exit 0
