#!/bin/bash
#
# (c) Gregor Rebel 2010-2011

FILE="$1"
shift

killall arm-none-eabi-gdb

if [ "$FILE" == "" ]; then
  echo "$0 ELF_BINARY"
  echo "Possible binaries: `ls *.elf`"
  exit 10
fi

if [ ! -e "$FILE" ]; then
  echo "$0 - ERROR: Cannot read binary-file '$FILE' in `pwd`!"
  exit 11
fi

Script1="configs/startscript1_${FILE}.gdb"
if [ ! -e $Script1 ]; then
  echo "creating missing gdb start-script '`pwd`/$Script1'"
  cat >>$Script1 <<END_OF_SCRIPT
# GNU Debugger Startscript #1 for symbol file '${FILE}'
#
# This script is automatically given to GDB on every start of $0.
# If deleted, this file will be reconstructed automatically.
#

symbol-file $FILE

END_OF_SCRIPT
fi #}
Script2="configs/startscript2_general.gdb"
if [ ! -e $Script2 ]; then
  echo "creating missing gdb start-script '`pwd`/$Script2'"
  cat >>$Script2 <<END_OF_SCRIPT
# GNU Debugger startscript #2 for general debugger setup 
#
# This script is executed by $0 for every symbol file.
# It may contain all break points and other setting to be activated at 
# start of each gdb session-
# If deleted, this file will be reconstructed automatically.
#

# CHANGE LINE BELOW ACCORDING TO YOUR SETUP!
target remote 127.0.0.1:3333  # connect to openocd runnning on local machine
#target remote 192.169.0.1:3333 # connect to openocd running on another machine (here 192.169.0.1)

# prevents single steping via next/ step: 
#monitor gdb_breakpoint_override hard

monitor reset init
monitor soft_reset_halt
set debug arm

break Assert_Halt_EC
break WWDG_IRQHandler

END_OF_SCRIPT
fi #}

arm-none-eabi-gdb -x $Script1 -x $Script2 $@
