#!/bin/bash

# Set of helpfull functions for install-scripts
#
# written 2003 - 2013 by Gregor Rebel (gregor HT thetoolchain DT com)
# Feel free do distribute and change to your own needs!
#

# ensure that $PATH contains no . (brings find into trouble)
export PATH=`perl -e "my @A=split(':', '$PATH');  print join(':', grep { substr(\\\$_,0,1) eq '/' } @A );"`

#{ InstallFuncs v1.3 --------------------------------------------------------------

function add2CleanUp() {     # File, Line              adds given line to cleanup file (if not already done)
    CleanUpFile="$1"
    Line="$2"
    
    if [ ! -e "$CleanUpFile" ]; then
      echo "#!/bin/bash"  >$CleanUpFile
      echo ""            >>$CleanUpFile
      chmod +x $CleanUpFile
    fi
    addLine "$CleanUpFile" "$Line"
}
function add2Script() {      # Script,Command,Execute  adds Command to Script; executes Command before if Execute==1  
  ScriptFile=$1
  Command=$2
  Execute=$3
  
  grep "$Command" $ScriptFile >tmp.grep
  AlreadyAdded=`cat tmp.grep`
  if [ "$AlreadyAdded" == "" ]; then
    Error=""
    if [ "$Execute" == "1" ]; then
      $Command || Error="1"
    fi
    if [ "$Error" == "" ]; then
      echo "$ScriptFile: $Command"
      echo "$Command" >>$ScriptFile
    fi
  fi
}
function addLine() {         # File,Line               adds given line to File if not already done
  al_File="$1"
  al_Line="$2"
  
  if [ ! -e "$al_File" ]; then
    touch "$al_File"
  fi
  AlreadyPresent=`grep "$al_Line" "$al_File"`
  if [ "$AlreadyPresent" == "" ]; then
    echo "$al_Line" >>"$al_File"
#  else
#    echo "file already up2date: $File2Add2"
  fi
}
function addLineIf() {       # CheckCMD, File, Line    if (`$CheckCMD`) then addLine(File, Line)
  ali_CheckCMD="$1"

  if [ "`$ali_CheckCMD`" ]; then
    addLine "$2" "$3"      
  fi  
}
function cmd() {             # CMD,[STDOUT]            display + execute given command; records log + errors
  CMD="$1"
  StdOut="$2"
  
  if [ "$StdOut" != "" ]; then
    echo "`pwd`> $CMD >>$StdOut"
    $CMD >>$StdOut 2>&1
  else
    echo "`pwd`> $CMD"
    $CMD 2>&1
    EC="$?"
    if [ "$EC" != "0" ]; then
      CMD_ERROR="RC=$EC from $CMD"
    else
      CMD_ERROR=""
    fi
  fi
}
function get() {             # URL_Prefix,File,OutFile,DestDir retrieves file from URL if not already done
  URL_Prefix="$1"
  gFile="$2"
  gOutFile="$3"
  gDownloadDir="$4"
  
  if [ "$gOutFile" == "" ]; then
    gOutFile=$gFile
  fi
  if [ "$gDownloadDir" != "" ]; then
    dir "$gDownloadDir"
    gOutFile="$gDownloadDir$gOutFile"
  fi

  if [ ! -e "$gOutFile" ]; then
    # CMD="wget $1$2"
    # echo "`pwd`> $CMD"
    # $CMD
    CMD_ERROR=""
    #cmd "wget --progress=bar:force -O ${gOutFile}.tmp $URL_Prefix$gFile" "log.wget"
    cmd "wget --progress=bar:force --timeout=60 -O ${gOutFile}.tmp $URL_Prefix$gFile"
    if [ "$CMD_ERROR" == "" ]; then
      mv "${gOutFile}.tmp" "${gOutFile}"
    else
      echo "get($URL_Prefix$gFile) - ERROR: '$CMD_ERROR'"
    fi
  else
    echo "`pwd`> get($URL_Prefix$gFile) - file already exists: '$gOutFile"
  fi
  if [ ! -e "$gOutFile" ]; then
    echo "missing file '`pwd`/$gOutFile"
    Get_Fails="$Get_Fails `pwd`/$gFile"
  fi
}
function findNewest() {      # URL_Prefix,Wildcard     $NewestFile = matching file with highest version number on given website
  fnURL_Prefix="$1"  # website that lists file downloads on an index page
  fnWildCard="$2"    # regular expression identifying interesting files
  
  # Note: make sure that findNewestDownload.pl is in search path!
  $HOME/Source/ToolChain_STM32/ToolChain/scripts/findNewestDownload.pl >/dev/null "$fnURL_Prefix" "$fnWildCard" tmp.GetNewest_URL tmp.GetNewest_File tmp.GetNewest_Dir
  
  if [ -e tmp.GetNewest_URL ]; then
    export NewestURL=`cat tmp.GetNewest_URL`
  else
    export NewestURL=""
  fi
  if [ -e tmp.GetNewest_File ]; then
    export NewestFile=`cat tmp.GetNewest_File`
  else
    export NewestFile=""
  fi
  if [ -e tmp.GetNewest_Dir ]; then
    export NewestDir=`cat tmp.GetNewest_Dir`
  else
    export NewestDir=""
  fi
}
function getFileSize() {     # FilePath                $FileSize = size of local file found under $FilePath
  gFS_FilePath="$1"
  
  FileSize=`ls -al "$gFS_FilePath" | awk '{ print $5 }'`
}
function getNewest() {       # URL_Prefix,Wildcard,OutFile,DestDir downloads newest version of a file from given website 
  gnURL_Prefix="$1"     # website that lists file downloads on an index page
  gnWildCard="$2"       # regular expression identifying interesting files
  gnOutFile="$3"        # filename to use for download file
  gnDownloadDir="$4"    # download into this directory
  
  #D echo "findNewestDownload.pl $gnURL_Prefix '$gnWildCard' tmp.GetNewest_URL tmp.GetNewest_File" #D
  
  # Note: make sure that findNewestDownload.pl is in search path!
  $HOME/Source/ToolChain_STM32/ToolChain/scripts/findNewestDownload.pl $gnURL_Prefix "$gnWildCard" tmp.GetNewest_URL tmp.GetNewest_File
  URL_Prefix=`cat tmp.GetNewest_URL`
  File=`cat tmp.GetNewest_File`
  Candidate="${URL_Prefix}${File}"
  echo "found newest download: '$Candidate'"
  
  if [ "$Candidate" != "" ]; then
    get "$gnURL_Prefix" "$File" "$gnOutFile" "$gnDownloadDir"
  fi
}
function getScriptPath() {   #                         ScriptPath = absolute path of directory in which script is located
  MyPWD=`pwd`
  FirstChar=`perl -e "print substr('$0', 0, 1);"`
  if [ "$FirstChar" == "/" ]; then
    MySelf="$0"
  else
    MySelf="$MyPWD/$0"
  fi
  ScriptPath=`perl -e "print substr('$MySelf', 0, rindex('$MySelf', '/'));"`
  cd "$ScriptPath"
  ScriptPath=`pwd`
  cd "$MyPWD"
  ##X echo "MySelf= $MySelf"
  ##X echo "ScriptPath= $ScriptPath"
}
function gitClone() {        # URL, Directory          clones/ updates git repository from URL into Directory/
  URL="$1"
  Directory="$2"
  
  if [ -d "$Directory" ]; then
    cd "$Directory"
    echo -n "`pwd`> git pull: "
    git pull
    cd ..
  else
    echo "`pwd`> git clone $URL"
    git clone $URL
  fi
}
function dir() {             # Directory               creates directory if missing
  Directory="$1"
  
  if [ ! -d $Directory ]; then
    mkdir "$Directory"
  fi
}
function findPrefix() {      #                         loads variable Prefix either with /usr or /usr/local according to system
  if [ -e "/usr/include/math.h" ]; then
    Prefix="/usr"
  else
    Prefix="/usr/local"
  fi
}
function createLink() {      # Source,Link,User,QUIET  creates symbolic link (removes old one before)
  cl_Source="$1"
  cl_Link="$2"
  cl_User="$3"
  cl_Quiet="$4"
  
  if [ "$cl_Quiet" = "" ]; then
    Verbose="-v"
  else
    Verbose=""
  fi
  if [ ! -e "$cl_Source" ]; then
    MissingFiles="$MissingFiles $cl_Source"
  fi
  
  rm -f "$cl_Link"
  if [ "$cl_User" != "" ]; then
    sudo -u$cl_User ln $Verbose -sf "$cl_Source" "$cl_Link"
  else
    ln $Verbose -sf "$cl_Source" "$cl_Link"
  fi
}
function identifyOS() {      #                         loads $OS_TYPE, $OS_SUBTYPE, $OS_VERSION, $OS_ARCHITECTURE according to current operating system

  OS_TYPE=""
  OS_SUBTYPE=""
  OS_VERSION=""
  OS_ARCHITECTURE=""
  
  if [ -e /etc/SuSE-release ]; then #{ checking repositories on openSUSE
    OS_TYPE="opensuse"
    OS_VERSION=`grep VERSION /etc/SuSE-release | awk '{ print $3 }'`
  fi #}
  if [ -e /etc/lsb-release ]; then #{ checking repositories on openSUSE
    IsUbuntu=`grep DISTRIB_ID /etc/lsb-release | grep -i ubuntu`
    if [ "$IsUbuntu" != "" ]; then
      OS_TYPE="ubuntu"
    fi
  fi
  if [ "$OS_TYPE" == "" ]; then
    IsUbuntu=`uname -a | grep -i ubuntu`
    if [ "$IsUbuntu" != "" ]; then
      OS_TYPE="ubuntu"
    fi
  fi
  if [ "$OS_TYPE" == "ubuntu" ]; then
    OS_VERSION=`grep RELEASE /etc/lsb-release | awk '{ split($0,a,"="); print a[2] }'`
    
    IsXUbuntu=`grep -i xubuntu /etc/apt/sources.list`
    if [ "$IsXUbuntu" != "" ]; then
      OS_SUBTYPE="xubuntu"
    fi
    IsKUbuntu=`grep -i kubuntu /etc/apt/sources.list`
    if [ "$IsKUbuntu" != "" ]; then
      OS_SUBTYPE="kubuntu"
    fi
  fi #}
  IsX86_64=`uname -a | grep -i x86_64`
  if [ "$IsX86_64" != "" ]; then
    OS_ARCHITECTURE="x86_64"
  fi
  IsX86_32=`uname -a | grep -i i686`
  if [ "$IsX86_32" != "" ]; then
    OS_ARCHITECTURE="x86_32"
  fi

  echo "identifyOS() - found $OS_TYPE ($OS_SUBTYPE) v$OS_VERSION $OS_ARCHITECTURE"
}
function install() {         #                         downloads, compiles + installs software from source tarball
  TestIfInstalled=$1; shift 
  UrlPrefix=$1;       shift
  FileName=$1;        shift
  Suffix=$1;          shift
  UrlSuffix=$1;       shift
  Directory=$1;       shift
  UnpackCommand=$1;   shift
  Configure=$1;       shift
  Make=$1;            shift
  Install=$1;         shift

  # Default-values:  
  # $Suffix          .tar.gz
  # $UrlSuffix       ""
  # $Directory       $FileName
  # $UnpackCommand   "tar xzf", "tar xjf" (for suffix .tar.bz2) or "tar xf" (for suffix .tar) 
  # $Configure       "./configure --prefix='/usr/'"
  # $Make            "make"
  # $Install         "make install"
  
  # Examples:
  # install " " "ftp://ftp.mars.org/pub/mpeg/" "libmad-0.15.1b"
  # install " " "http://downloads.xiph.org/releases/theora/" "libtheora-1.0alpha7" ".tar.bz2"
  # install " " "http://downloads.sourceforge.net/project/inkscape/inkscape/${Version}/" "inkscape-${Version}" ".tar.bz2" "?use_mirror=mesh" ".tar.bz" "inkscape-${Version}"
  # install " " "http://www2.mplayerhq.hu/MPlayer/releases/" "MPlayer-1.0rc1" ".tar.bz2" "MPlayer-1.0rc1" "tar xjf" \
  #             "./configure --cc=gcc410 --host-cc=gcc410 --prefix=/usr --mandir=/usr/share/man --confdir=/etc/mplayer/ --enable-gui --enable-largefiles --enable-menu --enable-mga --enable-xmga --enable-xv --enable-x11 --enable-fbdev --enable-vm  --enable-xvmc  --language=de,en  --enable-mmx  --enable-3dnow  --enable-sse  --enable-sse2  --enable-shm  --with-glib-config=/opt/gnome/bin/glib-config --with-gtk-config=/opt/gnome/bin/gtk-config"

  OriginDirectory=`pwd`
  
  if [ "$ReinstallAll" == "1" ]; then
    TestIfInstalled=""
  fi
  
  if [ $TestIfInstalled ]; then
    echo "skipping already installed '$FileName'"
  else
    if [ "$Suffix" == "" ]; then
      Suffix=".tar.gz"
    fi
    if [ "$Directory" == "" ]; then
      Directory=$FileName
    fi
    if [ "$UnpackCommand" == "" ]; then
      UnpackCommand="tar xzf"
      if [ "$Suffix" == ".tar.bz2" ]; then
        UnpackCommand="tar xjf"
      fi
      if [ "$Suffix" == ".tar.bz" ]; then
        UnpackCommand="tar xjf"
      fi
      if [ "$Suffix" == ".tar" ]; then
        UnpackCommand="tar xf"
      fi
    fi
    if [ "$Configure" == "" ]; then
      Configure="./configure --prefix=/usr"
    fi
    if [ "$Make" == "" ]; then
      Make="make"
    fi
    if [ "$Install" == "" ]; then
      Install="make install"
    fi
    
    echo ""
    echo "installiere $FileName"
    
    if [ ! -e $FileName$Suffix ]; then
      echo "Datei '$FileName$Suffix' nicht gefunden"
      pwd
      echo "hole $UrlPrefix$FileName$Suffix$UrlSuffix"
      wget $UrlPrefix$FileName$Suffix$UrlSuffix
      echo ""
    fi
    
    if [ ! "$UnpackCommand" == " " ]; then
      echo "entpacke mit '$UnpackCommand'"
      $UnpackCommand $FileName$Suffix
      echo "rm -Rf $FileName" >>$CleanUpBatch
    fi
    
    if [ ! -d $Directory ]; then
      echo "Verzeichnis '$Directory' nicht gefunden!"
  pwd
      echo "Verzeichnis '$Directory' nicht gefunden!" >>$OriginDirectory/$FileName.err
    else
      cd "$Directory"
      LogFileBase="../$FileName"
      if [ "$Directory" == "." ]; then
        LogFileBase="$FileName"
      fi
      Error=""
      echo -n "kompiliere mit $Configure && $Make && $Install || Error='1'."
      $Configure >$OriginDirectory/$FileName.log  2>$OriginDirectory/$FileName.err && echo -n "." && \
      $Make     >>$OriginDirectory/$FileName.log 2>>$OriginDirectory/$FileName.err && echo -n "." && \
      $Install  >>$OriginDirectory/$FileName.log 2>>$OriginDirectory/$FileName.err || \
      Error='1' 
      
      if [ -n "$Error" ]; then
        echo "FEHLER!"
        echo "$FileName" >>$OriginDirectory/$Failed_Installs
        else
        echo "OK"
      fi
    fi
    cd "$OriginDirectory"
  fi
}
function installPackage() {  # Name1, Name2, ...       installs all named packages via local package manager (zypper or apt-get)
  if [ -x /usr/bin/zypper ]; then
    cmd "sudo zypper --no-gpg-checks install -n -l -y $@"
  else
    cmd "sudo dpkg --configure -a"    # some installations showed a problem requiring this command
    waitForProcess dpkg
    export DEBIAN_FRONTEND=noninteractive
    # cmd "sudo DEBIAN_FRONTEND=noninteractive apt-get --install-suggests --force-yes --fix-missing -y install $@"
    cmd "sudo DEBIAN_FRONTEND=noninteractive apt-get --force-yes --fix-missing -y install $@"
  fi
}
function installPackageSafe() { # CheckIfInstalled, Name, Alternative1, ...  installs named package, installs alternatives if test fails
#{ Example Usage:
#  if [ ! -e OK.Packages ]; then #{ install some general software packages
#      FailedInstalls="" 
#      installPackage "which git" git gitosis
#      installPackage "which svn" svn subversion
#      if [ "$FailedInstalls" == "" ]; then
#        touch OK.Packages
#      else
#        echo "$0 - ERROR: Cannot install required software packages: $FailedInstalls"
#        exit 10
#      fi
#    fi #}
#}
    
  CheckIfInstalled="$1"      # 

  for Name in $2 $3 $4 $5 $6; do
    echo "installPackageSafe($Name)..." #D
    if [ "`$CheckIfInstalled`" == "" ]; then #{ test failed: install package
      if [ -x /usr/bin/zypper ]; then
        cmd "sudo zypper --no-gpg-checks install -n -l -y $Name"
      else
        cmd "sudo dpkg --configure -a"    # some installations showed a problem requiring this command
        waitForProcess dpkg
        export DEBIAN_FRONTEND=noninteractive
        # cmd "sudo DEBIAN_FRONTEND=noninteractive apt-get --install-suggests --force-yes --fix-missing -y install $Name"
        cmd "sudo DEBIAN_FRONTEND=noninteractive apt-get --force-yes --fix-missing -y install $Name"
      fi
    fi #}
  done
  if [ "`$CheckIfInstalled`" == "" ]; then #{ test failed: package install error
    echo "$0 installPackageSafe() - ERROR: Could not install any package from of $2 $3 $4 $5 $6!"
    FailedInstalls="$Name $FailedInstalls"
  fi #}
}
function makeOrig() {        # File                    creates backup file XXX_orig if not present
  bFile="$1"
  
  BakFile="${bFile}_orig"
  if [ ! -e $BakFile ]; then
    cp $bFile $BakFile
  fi
}
function replaceInFile() {   # File,Search,Replace     replaces all occurences of "$Search" in file $File by "$Replace"
    File="$1"
    Search="$2"   # some characters have to be escape (e.g. * -> \*)
    Replace="$3"  # some characters have to be escape (e.g. * -> \*)
    
   if [ ! -e "${File}_orig" ]; then
      makeOrig "${File}"
    fi
    
    echo "replace $File: $Search"
    echo "     -> $File: $Replace"
    echo "sed -e \"s|$Search|$Replace|g\" $File"
    sed -e "s|$Search|$Replace|g" $File >${File}_2
    mv -f ${File}_2 ${File}
}
function startIfMissing() {  # File,Command,Name       starts Command if File is missing
  
  TestFile=$1
  Command=$2
  Name=$3
  
  if [ ! -e $TestFile ]; then
    echo "Datei nicht gefunden: '$TestFile'"
    if [ ! "$Name" == "" ]; then
      echo "installiere $Name"
    fi
    $Command
  else
    echo "Datei bereits vorhanden: '$TestFile'"
  fi
}
function untgz() {           # Dir,Archiv              unpacks .tar.gz / .tgz archive if Dir is missing
  Dir="$1"
  Archiv="$2"
  

  if [ ! -d $Dir ]; then
    cmd "tar xzf $Archiv"
  fi
}
function untgj() {           # Dir,Archiv              unpacks .tar.bz2 / .tgj archive if Dir is missing
  Dir="$1"
  Archiv="$2"
  

  if [ ! -d $Dir ]; then
    cmd "tar xjf $Archiv"
  fi
}
function unZIP() {           # Dir,Archiv              unpacks .zip archive if Dir is missing
  Dir="$1"
  Archiv="$2"
  
  if [ ! -d $Dir ]; then
    cmd "unzip -o $Archiv" "unzip.log"
  fi
}
function testZIP() {         # Archiv                  tests integrity of .zip archive (exports $Error)
  Archiv="$1"
  
    export Error=`unzip -qqt $Archiv 2>&1`
}
function updateNewest() {    # URL_Prefix,Wildcard,SourceFile looks for newest file via findNewest() + updates source-file
  unVariablePrefix="$1"  # Prefix to use for variable names when adding lines to $unSourceFile 
  unURL_Prefix="$2"      # website that lists file downloads on an index page
  unWildCard="$3"        # regular expression identifying interesting files
  unSourceFile="$4"      # shell-script where to store URL of newest file URL
  
  findNewest https://launchpad.net/gcc-arm-embedded/+download 'gcc-arm-none-eabi-.+-linux\.tar\.bz2'
  if [ ! -e "$unSourceFile" ]; then #{ create new file
  cat <<END_OF_SCRIPT >$unSourceFile
#!/bin/bash
#
# list of newest versions of file downloads.
# This file has been created/ updated by 
# $0
#

END_OF_SCRIPT
  fi #}    
  NotNew=`grep "$NewestFile" "$unSourceFile"`
  if [ "$NotNew" == "" ]; then #{
    echo "found new: '$NewestURL$NewestFile'"
    cat <<END_OF_SCRIPT >>$unSourceFile #{

# $0 FINDNEWEST (`date`)
${unVariablePrefix}_URL="$NewestURL"
${unVariablePrefix}_File="$NewestFile"
END_OF_SCRIPT

#}
    export return="$NewestURL$NewestFile"
  else
    echo "no update."
    export return=""
  fi #}

}
function waitForProcess() {  # Processname             waits until all processes matching given name do exit
  ProcessName="$1"
  echo "$0 - waiting for process '$ProcessName' to exit.."
  while killall -0 $ProcessName 2>/dev/null; do sleep 1; done
}
function countCPUs() {       #                         $AmountCPUs = amount of installed CPU-cores
  AmountCPUs=`cat /proc/cpuinfo | grep "processor" | wc -l`
}
countCPUs

#} InstallFuncs v1.3 - written by Gregor Rebel
#{ Extra functions for The ToolChain ----------------------------------------------

function addDocumentationFile() {           # File, Category1, [Category2, ...]     moves given file into documentation folder + creates symbolic link
  File="$1"  # must be found in current working directory!
  Cat1="$2"
  Cat2="$3"
  Cat3="$4"
  
  DocFolder="$HOME/Source/ToolChain_STM32/Documentation"
  
  if [ "$Cat1" != "" ]; then #{ create category folder (if required)
    DocFolder=$DocFolder/$Cat1  
    if [ ! -d $DocFolder ]; then
      mkdir "$DocFolder"
    fi
    DocFolder=$DocFolder/$Cat2  
    if [ ! -d $DocFolder ]; then
      mkdir "$DocFolder"
    fi
    DocFolder=$DocFolder/$Cat3  
    if [ ! -d $DocFolder ]; then
      mkdir "$DocFolder"
    fi
  fi #}
  if [ -e "$File" ]; then
    mv -v "$File" "$DocFolder/"
  fi
  
  echo "createLink $DocFolder/$File ."
  createLink $DocFolder/$File .
  
}
cASH_Activated="" # stores Name of last createActivateScriptHead() call
function createActivateScriptHead() {       # ScriptName, Directory, CreatorScript, Info  creates head of activate.XXXX script in given file
  cASH_ScriptName="$1"
  cASH_Directory="$2"
  cASH_CreatorScript="$3"
  cASH_InfoText="$4"

  if [ "$cASH_Activated" != "" ]; then #{ ERROR: function called twice
    Error="createActivateScriptHead() was called twice!"
  
    cat <<END_OF_ERROR
$0 - createActivateScriptHead() ERROR: $Error

It is mandatory to call createActivateScriptTail() after every createActivateScriptHead() for same script!

Previous call: createActivateScriptTail('$cASH_Activated')
Current call:  createActivateScriptTail('$cASH_ScriptName', 
                                        '$cASH_Directory', 
                                        '$cASH_CreatorScript'
                                        '$cASH_InfoText'
                                       )
                                       
Called from  ${BASH_SOURCE[*]}:${BASH_LINENO[*]}
END_OF_ERROR
    exit 10
  fi #}
  cASH_Activated="$cASH_ScriptName"
  
  ActivateScriptFile="${cASH_Directory}/activate.${cASH_ScriptName}.sh"

  if [ "$cASH_CreatorScript" == "" ]; then #{
    cASH_CreatorScript="$0"
  fi #}
  if [ "$cASH_InfoText" == "" ]; then #{
    cASH_InfoText="DEFAULT_INFO: $ActivateScriptFile created by $cASH_CreatorScript"
  fi #}
  
  cat <<END_OF_SCRIPT    >$ActivateScriptFile #{
#!/bin/bash
#
#                  ToolChain_STM32
#
#           written 2010-2012 by Gregor Rebel
#
# This script activates an extension for ToolChain_STM32.
# This script has been automatically created by $cASH_CreatorScript.
# Any changes made to this script will be lost!
#
# Usage: Run this script from your project folder where your main.c is.
#

if [ "\$1" == "INFO" ]; then
  cat <<'END_OF_INFO'
$cASH_InfoText
END_OF_INFO
  exit 0
fi

TEST=""
if [ "\$1" == "TEST" ]; then
  TEST="1"
fi

END_OF_SCRIPT
#}
  cat <<'END_OF_SCRIPT' >>$ActivateScriptFile #{

if [ "$1" == "QUIET" ]; then
  Quiet="1"
  shift
fi
if [ "$1" != "" ]; then
  Caller="$1"
  shift
fi

if [ "$Quiet" == "" ]; then
  echo ""
fi

# ensure that $PATH contains no . (brings find into trouble)
export PATH=`perl -e "my @A=split(':', '$PATH');  print join(':', grep { substr(\\\$_,0,1) eq '/' } @A );"`

function createLink() {      # Source,Link,User,QUIET  creates symbolic link (removes old one before)
  cl_Source="$1"
  cl_Link="$2"
  cl_User="$3"
  cl_Quiet="$4"
  
  if [ "$cl_Quiet" = "" ]; then
    Verbose="-v"
  else
    Verbose=""
  fi
  if [ ! -e "$cl_Source" ]; then
    MissingFiles="$MissingFiles $cl_Source"
  fi
  
#X   if [ "$cl_User" != "" ]; then
#X     sudo -u$cl_User rm -f $Verbose $cl_Link
#X   else
#X     rm -f $cl_Link
#X   fi
   
  if [ "$cl_User" != "" ]; then
    sudo -u$cl_User ln $Verbose -sf "$cl_Source" "$cl_Link"
  else
    ln $Verbose -sfv "$cl_Source" "$cl_Link"
  fi
}
function getScriptPath() {   #                         ScriptPath = absolute path of directory in which script is located
  MyPWD=`pwd`
  FirstChar=`perl -e "print substr('$0', 0, 1);"`
  if [ "$FirstChar" == "/" ]; then
    MySelf="$0"
  else
    MySelf="$MyPWD/$0"
  fi
  ScriptPath=`perl -e "print substr('$MySelf', 0, rindex('$MySelf', '/'));"`
  cd "$ScriptPath"
  ScriptPath=`pwd`
  cd "$MyPWD"
  #X echo "MySelf= $MySelf"
  #X echo "ScriptPath= $ScriptPath"
}
function addLine() {         # File,Line               adds given line to File if not already done
  File2Add2="$1"
  Line="$2"
  
  if [ ! -e $File2Add2 ]; then
    touch $File2Add2
  fi
  AlreadyPresent=`grep "$Line" $File2Add2`
  if [ "$AlreadyPresent" == "" ]; then
    echo "$Line" >>$File2Add2
#  else
#    echo "file already up2date: $File2Add2"
  fi
}

CurrentPath=`pwd`
getScriptPath
Dir_Extensions=$ScriptPath                    # this script is located inside extensions/
Dir_ExtensionsLocal=${Dir_Extensions}.local   # path to extensions.local/
Dir_Configs=`pwd`/configs                     # path to configs/
Dir_Additionals=`pwd`/additionals             # path to additionals/
cd "$CurrentPath"

if [ -d extensions.active ]; then #{
  Dir_ExtensionsActive="extensions.active"
  Dir_Additionals="additionals"
#}
else #{
  if [ -d ../extensions.active ]; then
    Dir_ExtensionsActive="../extensions.active"
    Dir_Additionals="../additionals"
  else
    Dir_ExtensionsActive=${ScriptPath}.active
    Dir_Additionals=""
  fi
fi #}

END_OF_SCRIPT
#}
  cat <<END_OF_SCRIPT   >>$ActivateScriptFile #{

if [ -e "\$Dir_ExtensionsActive/makefile.${cASH_ScriptName}" ]; then
  # echo "\$0 - already activated"
  exit 0
fi

END_OF_SCRIPT
#}
}
function createActivateScriptTail() {       # ScriptName, Directory, Name           creates tail of activate.XXXX script in given file
  cAST_ScriptName="$1"
  cAST_Directory="$2"
  #X cAST_Name="$3"
  
  if [ "$cASH_Activated" == "" ]; then #{ ERROR: 
    Error="createActivateScriptHead() not called before!"
  else
    if [ "$cASH_Activated" != "$cAST_ScriptName" ]; then
      Error="createActivateScriptHead() called for different script!"
    fi
  fi #}
  if [ "$Error" != "" ]; then #{
    cat <<END_OF_ERROR
$0 - createActivateScriptTail() ERROR: $Error

It is mandatory to call createActivateScriptHead() before every createActivateScriptTail() for same script!

Previous call: createActivateScriptHead('$cASH_Activated')
Current call:  createActivateScriptTail('$cAST_ScriptName', 
                                        '$cAST_Directory'
                                       )

Called from  ${BASH_SOURCE[*]}:${BASH_LINENO[*]}
END_OF_ERROR
    exit 10
  fi #}
  cASH_Activated=""

  ActivateScriptFile="${cAST_Directory}/activate.${cAST_ScriptName}.sh"

  cat <<END_OF_SCRIPT >>$ActivateScriptFile #{

# back to project folder
cd "\$CurrentPath"
if [ -e "\$Dir_ExtensionsActive/makefile.$cAST_ScriptName" ]; then
  echo "Extension activated: $cAST_ScriptName (\$Caller)"
  if [ "\$Quiet" == "" ]; then
    echo ""
    echo "\${CurrentPath}/extensions.active/"
    find "\$Dir_ExtensionsActive/" -name "makefile.*" -execdir echo '  {}' \; | sort
  fi
else
  echo "ERROR: Failed to activate extension $cAST_ScriptName in \$CurrentPath (missing '\`pwd\`/\$Dir_ExtensionsActive/makefile.$cAST_ScriptName')!"
fi
END_OF_SCRIPT
#}
  chmod +x $ActivateScriptFile
}
function createActivateScript() {           # ScriptName,Directory,CreatorName,Delete,CreatorScript,Info   creates complete activate.XXXX script in given file
    cAS_ScriptName="$1"    # used to build filename 
    cAS_Directory="$2"     # relative/ absolute directory path where to create activate-script
    cAS_CreatorName="$3"   # name of this script //??? used
    cAS_Delete="$4"        # != "": will add rm $cAS_Delete to header of activate script 
    cAS_CreatorScript="$5" # $0 of script calling createActivateScript() (makes debugging easier) 
    cAS_InfoText="$6"      # each activate script provides a short information text
    
    
    createActivateScriptHead "$cAS_ScriptName" "$cAS_Directory" "$cAS_CreatorScript" "$cAS_InfoText"
    if [ "$cAS_Delete" != "" ]; then
      echo "rm 2>/dev/null \$Dir_ExtensionsActive/$cAS_Delete" >>$ActivateScriptFile
    fi
    echo "createLink \$ScriptPath/makefile.$cAS_ScriptName \$Dir_ExtensionsActive/makefile.$cAS_ScriptName '' QUIET \"\$0\"" >>$ActivateScriptFile
    createActivateScriptTail "$cAS_ScriptName" "$cAS_Directory"
}
cEMH_Activated="" # stores Name of last createExtensionMakefileHead() call
function createExtensionMakefileHead() {    # Name,[Folder]                         creates head of makefile.XXXX in given file; return variable MakeFile
  cEMH_Name="$1"
  cEMH_Folder="$2"
  
  if [ "$cEMH_Activated" != "" ]; then #{ ERROR: function called twice
    Error="createExtensionMakefileHead() was called twice!"
  
    cat <<END_OF_ERROR
$0 - createExtensionMakefileHead() ERROR: $Error

It is mandatory to call createExtensionMakefileTail() after every createExtensionMakefileHead() for same script!

Previous call: createExtensionMakefileTail('$cEMH_Activated')
Current call:  createExtensionMakefileTail('$cEMH_Name', '$cEMH_Folder')
                                       
Called from  ${BASH_SOURCE[*]}:${BASH_LINENO[*]}
END_OF_ERROR
    exit 10
  fi #}
  cEMH_Activated="$cEMH_Name"

  if [ "$cEMH_Folder" == "" ]; then #{
    cEMH_Folder="../extensions"
  fi #}
  ExtensionMakeFile="$cEMH_Folder/makefile.${cEMH_Name}" #{
  cat <<END_OF_MAKEFILE >$ExtensionMakeFile 
#{ EXTENSION_$cEMH_Name

EXTENSION_$cEMH_Name = 1
COMPILE_OPTS += -DEXTENSION_$cEMH_Name=1

END_OF_MAKEFILE
#}
#}
}
function createExtensionMakefileTail() {    # Name,[Folder]                         creates tail of makefile.XXXX in given file
  cEMT_Name="$1"
  cEMT_Folder="$2"
  
    if [ "$cEMH_Activated" == "" ]; then #{ ERROR: 
    Error="createExtensionMakefileHead() not called before!"
  else
    if [ "$cEMH_Activated" != "$cEMT_Name" ]; then
      Error="createExtensionMakefileHead() called for different script!"
    fi
  fi #}
  if [ "$Error" != "" ]; then #{
    cat <<END_OF_ERROR
$0 - createExtensionMakefileTail() ERROR: $Error

It is mandatory to call createExtensionMakefileHead() before every createExtensionMakefileTail() for same script!

Previous call: createExtensionMakefileHead('$cEMH_Activated')
Current call:  createExtensionMakefileTail('$cEMT_Name', '$cEMT_Folder')

Called from  ${BASH_SOURCE[*]}:${BASH_LINENO[*]}
END_OF_ERROR
    exit 10
  fi #}
  cEMH_Activated=""

  if [ "$cEMT_Folder" == "" ]; then #{
    cEMT_Folder="../extensions"
  fi #}
  ExtensionMakeFile="$cEMT_Folder/makefile.${cEMT_Name}" #{
  cat <<END_OF_MAKEFILE >>$ExtensionMakeFile #{ 
#} EXTENSION_$cEMT_Name
END_OF_MAKEFILE
#}
}
function createExtensionSourcefileHead() {  # Name,[Folder]                         creates head of source.XXXX.c in given file; return variable MakeFile
  cESH_Name="$1"
  cESH_Folder="$2"
  
  if [ "$cESH_Folder" == "" ]; then
    cESH_Folder="../extensions"
  fi
  export ExtensionSourceFile="$cESH_Folder/source.${cESH_Name}.c" #{
  cat <<END_OF_SOURCEFILE >$ExtensionSourceFile 
//{ EXTENSION_$cESH_Name
END_OF_SOURCEFILE
#}
#}
}
function createExtensionSourcefileTail() {  # Name,[Folder]                         creates tail of source.XXXX.c in given file
  cEST_Name="$1"
  cEST_Folder="$2"
  
  if [ "$cEST_Folder" == "" ]; then
    cEST_Folder="../extensions"
  fi
  export ExtensionSourceFile="$cEST_Folder/source.${cEST_Name}.c" #{
  cat <<END_OF_SOURCEFILE >>$ExtensionSourceFile #{
//} EXTENSION_$cEST_Name
END_OF_SOURCEFILE
#}
}
function getFile() {                        # URL_Prefix,File,OutFile,DestDir       retrieves file from URL into common download folder + creates local symbolic link
  URL_Prefix="$1"
  URL_File="$2"
  OutFile="$3"
  DestDir="$4"
  
  if [ "$OutFile" == "" ]; then
    OutFile="$URL_File"
  fi
  MirrorURL="http://thetoolchain.com/mirror/"
  
  echo "getFile('$URL_Prefix' '$URL_File' '$OutFile')"
  DownloadDir="$HOME/Source/ToolChain_STM32.Contents"
  dir "$DownloadDir"
  
  TempFile="$DownloadDir/$URL_File"
  get "$URL_Prefix" "$URL_File" "$TempFile"
  getFileSize "$TempFile"
  if [ "$FileSize" == "0" ]; then
    echo "ignoring received empty file"
    rm "$TempFile"
  fi
  if [ ! -e "$TempFile" ]; then
    echo "Original URL not found: '$URL_Prefix/$URL_File'"
    echo "fetching from backup mirror..."
    get "$MirrorURL" "$URL_File" "$TempFile" 
  fi
  getFileSize "$TempFile"
  if [ "$FileSize" == "0" ]; then
    echo "ignoring received empty file"
    rm "$TempFile"
  fi

  if [ -e "$TempFile" ]; then
    if [ "$DestDir" != "" ]; then
      dir "$DestDir"
    fi
    rm -f  "$DestDir$OutFile"
    ln -sv "$TempFile" "$DestDir$OutFile"
    GetFile_Error=""
  else
    GetFile_Error="ERROR: Missing file '$TempFile'"
    echo "getFile($URL_Prefix$URL_Prefix) - ERROR: Missing file '$TempFile'"
    addLine "$DownloadDir/_FailedDownloads.text" "wget $URL_Prefix/$URL_File -O \"$OutFile\""
  fi
}
function getNewestFile() {                  # URL_Prefix,WildCard,OutFile,DestDir   retrieves newest file from URL into common download folder + creates local symbolic link
  URL_Prefix="$1"
  WildCard="$2"
  OutFile="$3"
  DestDir="$4"

  # Note: make sure that findNewestDownload.pl is in search path!
  $HOME/Source/ToolChain_STM32/ToolChain/scripts/findNewestDownload.pl $URL_Prefix "$WildCard" tmp.GetNewest_URL tmp.GetNewest_File
  URL_Prefix=`cat tmp.GetNewest_URL`
  NewestFile=`cat tmp.GetNewest_File`
  NewestURL="${URL_Prefix}${NewestFile}"
  echo "found newest download: '$NewestURL'"
  
  if [ "$NewestURL" != "" ]; then
    getFile "$URL_Prefix" "$NewestFile" "$OutFile" "$DestDir"
  fi
}
function checkUserRoot() {                  # checks given user- and group-name if script is run by user root
  if [ "$USER" == "root" ]; then #{ check for required arguments
    TargetUser="$1"
    TargetGroup="$2"
  
    if [ "$TargetUser" == "" ]; then #{ print help + error + exit
      cat <<END_OF_ERROR
$0 [USERNAME [GROUP]]

USERNAME  chown all created files to this user (requires to run as user root)
GROUP     chown all created files to this group

ERROR: This script should not been run as user root without argument USERNAME!
END_OF_ERROR
      exit 10
    fi #}
    if [ ! -e "/home/$TargetUser" ]; then
      echo "$0 - ERROR: Cannot find home directory '/home/$TargetUser' !"
      exit 10
    fi
    
    # set environment (other scripts might use them)
    export HOME="/home/$TargetUser"
    export USER="$TargetUser"
    
    if [ "$TargetGroup" != "" ]; then
      TargetUserAndGroup="$TargetUser:$TargetGroup"
    else
      TargetUserAndGroup="$TargetUser"
    fi
  fi #}
}
function applyTargetUserAndGroup() {        # chowns given directory to $TargetUserAndGroup
  aTUAG_DirPath="$1"
  
  if [ "$TargetUserAndGroup" != "" ]; then #{ chown install directory
    # TargetUserAndGroup is set inside installFuncs.sh
    if [ -d "$aTUAG_DirPath" ]; then
      chown -R $TargetUserAndGroup "$aTUAG_DirPath"
    fi
  fi #}

}

#}Extra functions
