#!/usr/bin/perl -w

use strict;
use File::Spec::Functions qw(rel2abs);
use File::Basename;
my $ToolChainDir = dirname(rel2abs($0));

my $ProjectName = shift(@ARGV);
my $AuthorName  = shift(@ARGV);

my $StartPWD=`pwd`;
chomp($StartPWD);
chdir("..");
my $ProjectDir = $StartPWD.'/../'.$ProjectName;

unless ($AuthorName) {
  print "$0 PROJECT_NAME AUTHOR_NAME\n";
  print "Creates a new copy of Template project. Automatically sets project name and author in all files.\n";
  print "\n";
  exit 5;
}

unless ( -d "$ToolChainDir/Template" ) {
  print STDERR "$0 ERROR: Cannot find folder Template in $ToolChainDir/ !\n";
  exit 10;
}
if (-d $ProjectDir) {
  print STDERR "$0 ERROR: Directory $ProjectDir already exists!\n";
  exit 11;
}
if (-e $ProjectName) {
  print STDERR "$0 ERROR: File $ProjectName is in the way in $StartPWD!\n";
  exit 12;
}

system("rm -f $ENV{HOME}/ToolChain_STM32");
system("ln -sv $ToolChainDir $ENV{HOME}/ToolChain_STM32");
print "Creating new project $ProjectName for $AuthorName in $ProjectDir/\n";
my $Error;
exitOnError(`cp 2>&1 -R '$ToolChainDir/Template' $ProjectDir/ `);
system("rm 2>/dev/null $ProjectDir/extensions.active/*");
exitOnError(`cp 2>&1 >/dev/null '$ToolChainDir/Version' $ProjectDir/required_version`);
unless (-d $ProjectDir) {
  print STDERR "$0 ERROR: Cannot create directory $ProjectDir in `pwd` !\n";
  exit 13;
}
chdir($ProjectDir);
foreach my $FileName (qw( TemplateName.config  TemplateName.creator  TemplateName.creator.user  TemplateName.files  TemplateName.includes ) ) {
  my $OrigFileNames = `find ./ -maxdepth 3 -name $FileName`;
  my @OrigFileNames = split("\n", $OrigFileNames);
  foreach my $OrigFileName (@OrigFileNames) {
    my $NewName = $OrigFileName;
    $NewName =~ s/TemplateName/$ProjectName/;
    print "$OrigFileName -> $NewName\n";
    rename($OrigFileName, $NewName);
  }
}

print "searching for files to be processed in ".`pwd`."...\n";
my @Files2Process;
my %Files2Process;
foreach my $Keyword ( qw(TemplateName TemplateAuthor TemplateProjectFolder) ) {
  my $FilesFound = `find ./ -maxdepth 3 -exec grep -l $Keyword {} \\;`;
  my @FilesFound = split("\n", $FilesFound);
  foreach my $FileFound (@FilesFound) {
    my @Parts = split(':', $FileFound);
    my $FileName = $Parts[0];
    if ($FileName && (!$Files2Process{$FileName}) ) {
      push @Files2Process, $FileName;
      $Files2Process{$FileName} = 1;
    }
  }
}

$Error = '';
my $PWD=`pwd`; chomp($PWD);
foreach my $File2Process (@Files2Process) { # replace Keywords in all files
  my $OldError = $Error;
  if (-l $File2Process) { print "skipping symbolic link $PWD/$File2Process\n"; }
  else {
    open(IN, "<$File2Process") or $Error .= "Cannot read from file '$File2Process' ($!)\n";
    my @Lines;
    if ($OldError eq $Error) {
      @Lines = <IN>;
      close(IN);
      print "modifying $PWD/$File2Process ..\n";
      foreach my $Line (@Lines) {
        $Line =~ s/TemplateName/$ProjectName/g;
        $Line =~ s/TemplateAuthor/$AuthorName/g;
        $Line =~ s/TemplateProjectFolder/$PWD/g;
      }
      $OldError = $Error;
      open(OUT, ">$File2Process") or $Error .= "Cannot write to file '$File2Process' ($!)\n";
      if ($OldError eq $Error) {
        print OUT join("", @Lines);
        close(OUT);
      }
    }
  }
  if ($Error) { print $Error; }
}

my $UserFile = readFile("QtCreator/${ProjectName}.creator.user");
my $Pos1 = 0;
print "replace -> '$PWD'\n"; #D
do { # update all build directory fields in QtCreator user file
  $Pos1 = index($UserFile, 'BuildDirectory">', $Pos1) + 16;
  if ($Pos1 > 15) {
    my $Pos2 = index($UserFile, '</value>', $Pos1);
    substr($UserFile, $Pos1, $Pos2 - $Pos1) = $PWD.'/QtCreator';
  }
} while ($Pos1 > 15);
writeFile("QtCreator/${ProjectName}.creator.user", $UserFile);

system("./createLinks.sh >/dev/null");
system("_/clean.sh");
system("_/createActivateScript.pl ./");
system("./compile.sh NOCOMPILE");

# extension setup may have changed during compile.sh: recreacte activate-script
system("_/clean.sh");
system("_/createActivateScript.pl ./");
system("./compile.sh NOFLASH");

print "\n";
print "Project $ProjectName has been created successfully.\n";
my $PWD2 = `pwd`; chop($PWD2);
print <<"END_OF_INFO";

Next steps:

1) connect JTAG programmer (e.g. Olimex ARB-USB TinyH) to USB port
2) connect prototype board to JTAG programmer.
3) cd $PWD2
4) ./qtcreator.sh

END_OF_INFO

sub readFile {
  my $FileName = shift;
  my @Lines;

  open(IN, "<$FileName") or die("Cannot read file '$FileName' ($!)");
  @Lines = <IN>;  
  close(IN);
  
  return join('', @Lines);
}
sub writeFile {
  my $FileName = shift;
  my $Content  = shift;
  
  open(OUT, ">$FileName") or die("Cannot write to file '$FileName' ($!)");
  print OUT $Content;  
  close(OUT);
}

exit 0;

sub exitOnError {
  my $Error = $_[0];
  
  if ($Error) 
  { 
    while (substr($Error, -1) eq "\n") { substr($Error, -1) = undef; } 
    print STDERR "$0 - ERROR: $Error\n";
    print STDERR "$0 - Project creation has been canceled.\n";
    exit 10; 
  }
}